@@ -10,17 +10,16 @@
 #include <memory>
 #include <vector>
 
-#include "core/fpdfapi/font/cpdf_tounicodemap.h"
+#include "build/build_config.h"
+#include "core/fpdfapi/parser/cpdf_dictionary.h"
 #include "core/fpdfapi/parser/cpdf_stream_acc.h"
 #include "core/fxcrt/fx_string.h"
 #include "core/fxcrt/fx_system.h"
 #include "core/fxcrt/unowned_ptr.h"
 #include "core/fxge/cfx_font.h"
-#include "core/fxge/fx_font.h"
 
 class CFX_SubstFont;
 class CPDF_CIDFont;
-class CPDF_Dictionary;
 class CPDF_Document;
 class CPDF_Object;
 class CPDF_TrueTypeFont;
@@ -32,8 +31,7 @@ class CPDF_Font {
  public:
   static std::unique_ptr<CPDF_Font> Create(CPDF_Document* pDoc,
                                            CPDF_Dictionary* pFontDict);
-  static CPDF_Font* GetStockFont(CPDF_Document* pDoc,
-                                 const ByteStringView& fontname);
+  static CPDF_Font* GetStockFont(CPDF_Document* pDoc, ByteStringView fontname);
   static const uint32_t kInvalidCharCode = static_cast<uint32_t>(-1);
 
   virtual ~CPDF_Font();
@@ -53,12 +51,11 @@ class CPDF_Font {
 
   virtual bool IsVertWriting() const;
   virtual bool IsUnicodeCompatible() const;
-  virtual uint32_t GetNextChar(const ByteStringView& pString,
-                               size_t* pOffset) const;
-  virtual size_t CountChar(const ByteStringView& pString) const;
+  virtual uint32_t GetNextChar(ByteStringView pString, size_t* pOffset) const;
+  virtual size_t CountChar(ByteStringView pString) const;
   virtual int AppendChar(char* buf, uint32_t charcode) const;
   virtual int GlyphFromCharCode(uint32_t charcode, bool* pVertGlyph) = 0;
-#if _FX_PLATFORM_ == _FX_PLATFORM_APPLE_
+#if defined(OS_MACOSX)
   virtual int GlyphFromCharCodeExt(uint32_t charcode);
 #endif
   virtual WideString UnicodeFromCharCode(uint32_t charcode) const;
@@ -77,7 +74,7 @@ class CPDF_Font {
   const FX_RECT& GetFontBBox() const { return m_FontBBox; }
   int GetTypeAscent() const { return m_Ascent; }
   int GetTypeDescent() const { return m_Descent; }
-  uint32_t GetStringWidth(const ByteStringView& pString);
+  uint32_t GetStringWidth(ByteStringView pString);
   uint32_t FallbackFontFromCharcode(uint32_t charcode);
   int FallbackGlyphFromCharcode(int fallbackFont, uint32_t charcode);
   int GetFontFlags() const { return m_Flags; }
@@ -85,9 +82,13 @@ class CPDF_Font {
   virtual uint32_t GetCharWidthF(uint32_t charcode) = 0;
   virtual FX_RECT GetCharBBox(uint32_t charcode) = 0;
 
+  // Can return nullptr for stock Type1 fonts. Always returns non-null for other
+  // font types.
   CPDF_Document* GetDocument() const { return m_pDocument.Get(); }
+
   CFX_Font* GetFont() { return &m_Font; }
   const CFX_Font* GetFont() const { return &m_Font; }
+
   CFX_Font* GetFontFallback(int position);
 
  protected:
@@ -95,6 +96,9 @@ class CPDF_Font {
 
   static int TT2PDF(int m, FXFT_Face face);
   static bool FT_UseTTCharmap(FXFT_Face face, int platform_id, int encoding_id);
+  static const char* GetAdobeCharName(int iBaseEncoding,
+                                      const std::vector<ByteString>& charnames,
+                                      uint32_t charcode);
 
   virtual bool Load() = 0;
 
@@ -102,15 +106,11 @@ class CPDF_Font {
   void LoadFontDescriptor(const CPDF_Dictionary* pDict);
   void CheckFontMetrics();
 
-  const char* GetAdobeCharName(int iBaseEncoding,
-                               const std::vector<ByteString>& charnames,
-                               int charcode);
-
   UnownedPtr<CPDF_Document> const m_pDocument;
   CFX_Font m_Font;
   std::vector<std::unique_ptr<CFX_Font>> m_FontFallbacks;
   RetainPtr<CPDF_StreamAcc> m_pFontFile;
-  UnownedPtr<CPDF_Dictionary> m_pFontDict;
+  RetainPtr<CPDF_Dictionary> m_pFontDict;
   ByteString m_BaseFont;
   mutable std::unique_ptr<CPDF_ToUnicodeMap> m_pToUnicodeMap;
   mutable bool m_bToUnicodeLoaded = false;
