@@ -6,10 +6,13 @@
 
 #include "core/fpdfapi/font/cpdf_tounicodemap.h"
 
+#include <utility>
+
 #include "core/fpdfapi/cpdf_modulemgr.h"
 #include "core/fpdfapi/font/cpdf_cid2unicodemap.h"
 #include "core/fpdfapi/page/cpdf_pagemodule.h"
 #include "core/fpdfapi/parser/cpdf_simple_parser.h"
+#include "core/fpdfapi/parser/cpdf_stream.h"
 #include "core/fxcrt/fx_extension.h"
 #include "core/fxcrt/fx_safe_types.h"
 #include "third_party/base/numerics/safe_conversions.h"
@@ -52,7 +55,7 @@ uint32_t CPDF_ToUnicodeMap::ReverseLookup(wchar_t unicode) const {
 }
 
 // Static.
-uint32_t CPDF_ToUnicodeMap::StringToCode(const ByteStringView& str) {
+uint32_t CPDF_ToUnicodeMap::StringToCode(ByteStringView str) {
   int len = str.GetLength();
   if (len == 0)
     return 0;
@@ -89,7 +92,7 @@ static WideString StringDataAdd(WideString str) {
 }
 
 // Static.
-WideString CPDF_ToUnicodeMap::StringToWideString(const ByteStringView& str) {
+WideString CPDF_ToUnicodeMap::StringToWideString(ByteStringView str) {
   int len = str.GetLength();
   if (len == 0)
     return WideString();
@@ -122,11 +125,11 @@ uint32_t CPDF_ToUnicodeMap::GetUnicode() {
   return uni.ValueOrDefault(0);
 }
 
-void CPDF_ToUnicodeMap::Load(CPDF_Stream* pStream) {
+void CPDF_ToUnicodeMap::Load(const CPDF_Stream* pStream) {
   CIDSet cid_set = CIDSET_UNKNOWN;
   auto pAcc = pdfium::MakeRetain<CPDF_StreamAcc>(pStream);
-  pAcc->LoadAllData(false);
-  CPDF_SimpleParser parser(pAcc->GetData(), pAcc->GetSize());
+  pAcc->LoadAllDataFiltered();
+  CPDF_SimpleParser parser(pAcc->GetSpan());
   while (1) {
     ByteStringView word = parser.GetWord();
     if (word.IsEmpty()) {
@@ -205,7 +208,7 @@ void CPDF_ToUnicodeMap::Load(CPDF_Stream* pStream) {
               m_Map[code] = GetUnicode();
               m_MultiCharBuf.AppendChar(retcode.GetLength());
               m_MultiCharBuf << retcode;
-              destcode = retcode;
+              destcode = std::move(retcode);
             }
           }
         }
@@ -225,7 +228,7 @@ void CPDF_ToUnicodeMap::Load(CPDF_Stream* pStream) {
                      ->GetPageModule()
                      ->GetFontGlobals()
                      ->GetCMapManager()
-                     ->GetCID2UnicodeMap(cid_set, false);
+                     ->GetCID2UnicodeMap(cid_set);
   } else {
     m_pBaseMap = nullptr;
   }
