@@ -15,7 +15,7 @@
 #include "core/fpdfapi/parser/cpdf_dictionary.h"
 #include "core/fpdfapi/parser/cpdf_stream.h"
 #include "core/fxcrt/fx_system.h"
-#include "third_party/base/stl_util.h"
+#include "third_party/base/ptr_util.h"
 
 namespace {
 
@@ -26,10 +26,11 @@ constexpr int kMaxType3FormLevel = 4;
 CPDF_Type3Font::CPDF_Type3Font(CPDF_Document* pDocument,
                                CPDF_Dictionary* pFontDict)
     : CPDF_SimpleFont(pDocument, pFontDict) {
+  ASSERT(GetDocument());
   memset(m_CharWidthL, 0, sizeof(m_CharWidthL));
 }
 
-CPDF_Type3Font::~CPDF_Type3Font() {}
+CPDF_Type3Font::~CPDF_Type3Font() = default;
 
 bool CPDF_Type3Font::IsType3Font() const {
   return true;
@@ -44,7 +45,7 @@ CPDF_Type3Font* CPDF_Type3Font::AsType3Font() {
 }
 
 bool CPDF_Type3Font::Load() {
-  m_pFontResources = m_pFontDict->GetDictFor("Resources");
+  m_pFontResources.Reset(m_pFontDict->GetDictFor("Resources"));
   const CPDF_Array* pMatrix = m_pFontDict->GetArrayFor("FontMatrix");
   float xscale = 1.0f;
   float yscale = 1.0f;
@@ -68,7 +69,7 @@ bool CPDF_Type3Font::Load() {
   if (StartChar >= 0 && static_cast<size_t>(StartChar) < kCharLimit) {
     const CPDF_Array* pWidthArray = m_pFontDict->GetArrayFor("Widths");
     if (pWidthArray) {
-      size_t count = std::min(pWidthArray->GetCount(), kCharLimit);
+      size_t count = std::min(pWidthArray->size(), kCharLimit);
       count = std::min(count, kCharLimit - StartChar);
       for (size_t i = 0; i < count; i++) {
         m_CharWidthL[StartChar + i] =
@@ -77,7 +78,7 @@ bool CPDF_Type3Font::Load() {
       }
     }
   }
-  m_pCharProcs = m_pFontDict->GetDictFor("CharProcs");
+  m_pCharProcs.Reset(m_pFontDict->GetDictFor("CharProcs"));
   if (m_pFontDict->GetDirectObjectFor("Encoding"))
     LoadPDFEncoding(false, false);
   return true;
@@ -125,7 +126,7 @@ CPDF_Type3Char* CPDF_Type3Font::LoadChar(uint32_t charcode) {
   pNewChar->Transform(m_FontMatrix);
   m_CacheMap[charcode] = std::move(pNewChar);
   CPDF_Type3Char* pCachedChar = m_CacheMap[charcode].get();
-  if (pCachedChar->form()->GetPageObjectList()->empty())
+  if (pCachedChar->form()->GetPageObjectCount() == 0)
     pCachedChar->ResetForm();
   return pCachedChar;
 }
