@@ -12,34 +12,40 @@
 
 #include "core/fpdfapi/page/cpdf_pageobjectholder.h"
 
-class CPDF_Document;
-class CPDF_Dictionary;
-class CPDF_Stream;
-class CPDF_AllStates;
 class CFX_Matrix;
+class CPDF_AllStates;
+class CPDF_Dictionary;
+class CPDF_Document;
+class CPDF_Stream;
 class CPDF_Type3Char;
 
-class CPDF_Form : public CPDF_PageObjectHolder {
+class CPDF_Form final : public CPDF_PageObjectHolder {
  public:
+  // Helper method to choose the first non-null resources dictionary.
+  static CPDF_Dictionary* ChooseResourcesDict(CPDF_Dictionary* pResources,
+                                              CPDF_Dictionary* pParentResources,
+                                              CPDF_Dictionary* pPageResources);
+
+  CPDF_Form(CPDF_Document* pDocument,
+            CPDF_Dictionary* pPageResources,
+            CPDF_Stream* pFormStream);
   CPDF_Form(CPDF_Document* pDocument,
             CPDF_Dictionary* pPageResources,
             CPDF_Stream* pFormStream,
-            CPDF_Dictionary* pParentResources = nullptr);
+            CPDF_Dictionary* pParentResources);
   ~CPDF_Form() override;
 
-  void ParseContent();
-  void ParseContentWithParams(CPDF_AllStates* pGraphicStates,
-                              const CFX_Matrix* pParentMatrix,
-                              CPDF_Type3Char* pType3Char,
-                              std::set<const uint8_t*>* parsedSet);
+  void ParseContent(const CPDF_AllStates* pGraphicStates,
+                    const CFX_Matrix* pParentMatrix,
+                    CPDF_Type3Char* pType3Char,
+                    std::set<const uint8_t*>* parsedSet);
+
+  const CPDF_Stream* GetStream() const;
 
  private:
-  void StartParse(CPDF_AllStates* pGraphicStates,
-                  const CFX_Matrix* pParentMatrix,
-                  CPDF_Type3Char* pType3Char,
-                  std::set<const uint8_t*>* parsedSet);
-
   std::unique_ptr<std::set<const uint8_t*>> m_ParsedSet;
+
+  RetainPtr<CPDF_Stream> const m_pFormStream;
 };
 
 #endif  // CORE_FPDFAPI_PAGE_CPDF_FORM_H_
