@@ -10,7 +10,6 @@
 
 #include "core/fpdfapi/parser/cpdf_array.h"
 #include "core/fpdfapi/parser/cpdf_dictionary.h"
-#include "core/fxcrt/fx_memory.h"
 
 namespace {
 
@@ -43,14 +42,14 @@ bool CPDF_StitchFunc::v_Init(const CPDF_Object* pObj,
   if (!pEncodeArray)
     return false;
 
-  const uint32_t nSubs = pFunctionsArray->GetCount();
+  const uint32_t nSubs = pFunctionsArray->size();
   if (nSubs == 0)
     return false;
 
   // Check array sizes. The checks are slightly relaxed to allow the "Bounds"
   // and "Encode" arrays to have more than the required number of elements.
   {
-    if (pBoundsArray->GetCount() < nSubs - 1)
+    if (pBoundsArray->size() < nSubs - 1)
       return false;
 
     FX_SAFE_UINT32 nExpectedEncodeSize = nSubs;
@@ -58,7 +57,7 @@ bool CPDF_StitchFunc::v_Init(const CPDF_Object* pObj,
     if (!nExpectedEncodeSize.IsValid())
       return false;
 
-    if (pEncodeArray->GetCount() < nExpectedEncodeSize.ValueOrDie())
+    if (pEncodeArray->size() < nExpectedEncodeSize.ValueOrDie())
       return false;
   }
 
