@@ -13,7 +13,8 @@
 #include "core/fpdfapi/parser/cpdf_stream.h"
 #include "core/fpdfapi/parser/cpdf_stream_acc.h"
 #include "core/fpdfapi/parser/cpdf_syntax_parser.h"
-#include "core/fxcrt/cfx_memorystream.h"
+#include "core/fxcrt/cfx_readonlymemorystream.h"
+#include "third_party/base/ptr_util.h"
 #include "third_party/base/stl_util.h"
 
 // static
@@ -75,15 +76,14 @@ bool CPDF_ObjectStream::HasObject(uint32_t obj_number) const {
   return pdfium::ContainsKey(objects_offsets_, obj_number);
 }
 
-std::unique_ptr<CPDF_Object> CPDF_ObjectStream::ParseObject(
+RetainPtr<CPDF_Object> CPDF_ObjectStream::ParseObject(
     CPDF_IndirectObjectHolder* pObjList,
     uint32_t obj_number) const {
   const auto it = objects_offsets_.find(obj_number);
   if (it == objects_offsets_.end())
     return nullptr;
 
-  std::unique_ptr<CPDF_Object> result =
-      ParseObjectAtOffset(pObjList, it->second);
+  RetainPtr<CPDF_Object> result = ParseObjectAtOffset(pObjList, it->second);
   if (!result)
     return nullptr;
 
@@ -96,7 +96,7 @@ void CPDF_ObjectStream::Init(const CPDF_Stream* stream) {
     auto stream_acc = pdfium::MakeRetain<CPDF_StreamAcc>(stream);
     stream_acc->LoadAllDataFiltered();
     const uint32_t data_size = stream_acc->GetSize();
-    data_stream_ = pdfium::MakeRetain<CFX_MemoryStream>(
+    data_stream_ = pdfium::MakeRetain<CFX_ReadOnlyMemoryStream>(
         stream_acc->DetachData(), data_size);
   }
 
@@ -115,7 +115,7 @@ void CPDF_ObjectStream::Init(const CPDF_Stream* stream) {
   }
 }
 
-std::unique_ptr<CPDF_Object> CPDF_ObjectStream::ParseObjectAtOffset(
+RetainPtr<CPDF_Object> CPDF_ObjectStream::ParseObjectAtOffset(
     CPDF_IndirectObjectHolder* pObjList,
     uint32_t object_offset) const {
   FX_SAFE_FILESIZE offset_in_stream = first_object_offset_;
