@@ -24,6 +24,7 @@
 #include "core/fxcodec/fx_codec.h"
 #include "core/fxcrt/fx_extension.h"
 #include "third_party/base/numerics/safe_math.h"
+#include "third_party/base/stl_util.h"
 
 namespace {
 
@@ -87,6 +88,30 @@ const uint16_t PDFDocEncoding[256] = {
     0x00f3, 0x00f4, 0x00f5, 0x00f6, 0x00f7, 0x00f8, 0x00f9, 0x00fa, 0x00fb,
     0x00fc, 0x00fd, 0x00fe, 0x00ff};
 
+bool ValidateDecoderPipeline(const CPDF_Array* pDecoders) {
+  size_t count = pDecoders->size();
+  if (count == 0)
+    return true;
+
+  for (size_t i = 0; i < count; ++i) {
+    if (!pDecoders->GetObjectAt(i)->IsName())
+      return false;
+  }
+
+  if (count == 1)
+    return true;
+
+  // TODO(thestig): Consolidate all the places that use these filter names.
+  static const char kValidDecoders[][16] = {
+      "FlateDecode",    "Fl",  "LZWDecode",       "LZW", "ASCII85Decode", "A85",
+      "ASCIIHexDecode", "AHx", "RunLengthDecode", "RL"};
+  for (size_t i = 0; i < count - 1; ++i) {
+    if (!pdfium::ContainsValue(kValidDecoders, pDecoders->GetStringAt(i)))
+      return false;
+  }
+  return true;
+}
+
 uint32_t A85Decode(pdfium::span<const uint8_t> src_span,
                    std::unique_ptr<uint8_t, FxFreeDeleter>* dest_buf,
                    uint32_t* dest_size) {
@@ -345,22 +370,24 @@ bool PDF_DataDecode(pdfium::span<const uint8_t> src_span,
                     const CPDF_Dictionary* pDict,
                     uint32_t last_estimated_size,
                     bool bImageAcc,
-                    uint8_t** dest_buf,
+                    std::unique_ptr<uint8_t, FxFreeDeleter>* dest_buf,
                     uint32_t* dest_size,
                     ByteString* ImageEncoding,
                     UnownedPtr<const CPDF_Dictionary>* pImageParams) {
-  const CPDF_Object* pDecoder =
-      pDict ? pDict->GetDirectObjectFor("Filter") : nullptr;
+  const CPDF_Object* pDecoder = pDict->GetDirectObjectFor("Filter");
   if (!pDecoder || (!pDecoder->IsArray() && !pDecoder->IsName()))
     return false;
 
   const CPDF_Object* pParams =
-      pDict ? pDict->GetDirectObjectFor(pdfium::stream::kDecodeParms) : nullptr;
+      pDict->GetDirectObjectFor(pdfium::stream::kDecodeParms);
 
   std::vector<std::pair<ByteString, const CPDF_Object*>> DecoderArray;
   if (const CPDF_Array* pDecoders = pDecoder->AsArray()) {
+    if (!ValidateDecoderPipeline(pDecoders))
+      return false;
+
     const CPDF_Array* pParamsArray = ToArray(pParams);
-    for (size_t i = 0; i < pDecoders->GetCount(); ++i) {
+    for (size_t i = 0; i < pDecoders->size(); ++i) {
       DecoderArray.push_back(
           {pDecoders->GetStringAt(i),
            pParamsArray ? pParamsArray->GetDictAt(i) : nullptr});
@@ -369,14 +396,17 @@ bool PDF_DataDecode(pdfium::span<const uint8_t> src_span,
     DecoderArray.push_back(
         {pDecoder->GetString(), pParams ? pParams->GetDict() : nullptr});
   }
-  pdfium::span<uint8_t> last_span(const_cast<uint8_t*>(src_span.data()),
-                                  src_span.size());
+
+  std::unique_ptr<uint8_t, FxFreeDeleter> result;
+  // May be changed to point to |result| in the for-loop below. So put it below
+  // |result| and let it get destroyed first.
+  pdfium::span<const uint8_t> last_span = src_span;
   size_t nSize = DecoderArray.size();
   for (size_t i = 0; i < nSize; ++i) {
     int estimated_size = i == nSize - 1 ? last_estimated_size : 0;
     ByteString decoder = DecoderArray[i].first;
     const CPDF_Dictionary* pParam = ToDictionary(DecoderArray[i].second);
-    uint8_t* new_buf = nullptr;
+    std::unique_ptr<uint8_t, FxFreeDeleter> new_buf;
     uint32_t new_size = 0xFFFFFFFF;
     uint32_t offset = FX_INVALID_OFFSET;
     if (decoder == "Crypt")
@@ -384,39 +414,29 @@ bool PDF_DataDecode(pdfium::span<const uint8_t> src_span,
     if (decoder == "FlateDecode" || decoder == "Fl") {
       if (bImageAcc && i == nSize - 1) {
         *ImageEncoding = "FlateDecode";
-        *dest_buf = last_span.data();
+        *dest_buf = std::move(result);
         *dest_size = last_span.size();
         *pImageParams = pParam;
         return true;
       }
-      std::unique_ptr<uint8_t, FxFreeDeleter> result;
       offset = FlateOrLZWDecode(false, last_span, pParam, estimated_size,
-                                &result, &new_size);
-      new_buf = result.release();
+                                &new_buf, &new_size);
     } else if (decoder == "LZWDecode" || decoder == "LZW") {
-      std::unique_ptr<uint8_t, FxFreeDeleter> result;
       offset = FlateOrLZWDecode(true, last_span, pParam, estimated_size,
-                                &result, &new_size);
-      new_buf = result.release();
+                                &new_buf, &new_size);
     } else if (decoder == "ASCII85Decode" || decoder == "A85") {
-      std::unique_ptr<uint8_t, FxFreeDeleter> result;
-      offset = A85Decode(last_span, &result, &new_size);
-      new_buf = result.release();
+      offset = A85Decode(last_span, &new_buf, &new_size);
     } else if (decoder == "ASCIIHexDecode" || decoder == "AHx") {
-      std::unique_ptr<uint8_t, FxFreeDeleter> result;
-      offset = HexDecode(last_span, &result, &new_size);
-      new_buf = result.release();
+      offset = HexDecode(last_span, &new_buf, &new_size);
     } else if (decoder == "RunLengthDecode" || decoder == "RL") {
       if (bImageAcc && i == nSize - 1) {
         *ImageEncoding = "RunLengthDecode";
-        *dest_buf = last_span.data();
+        *dest_buf = std::move(result);
         *dest_size = last_span.size();
         *pImageParams = pParam;
         return true;
       }
-      std::unique_ptr<uint8_t, FxFreeDeleter> result;
-      offset = RunLengthDecode(last_span, &result, &new_size);
-      new_buf = result.release();
+      offset = RunLengthDecode(last_span, &new_buf, &new_size);
     } else {
       // If we get here, assume it's an image decoder.
       if (decoder == "DCT")
@@ -425,42 +445,37 @@ bool PDF_DataDecode(pdfium::span<const uint8_t> src_span,
         decoder = "CCITTFaxDecode";
       *ImageEncoding = std::move(decoder);
       *pImageParams = pParam;
-      *dest_buf = last_span.data();
+      *dest_buf = std::move(result);
       *dest_size = last_span.size();
       return true;
     }
-    if (last_span.data() != src_span.data()) {
-      auto* temp = last_span.data();
-      last_span = {};  // Spans can't outlive their data.
-      FX_Free(temp);
-    }
-    if (offset == FX_INVALID_OFFSET) {
-      FX_Free(new_buf);
+    if (offset == FX_INVALID_OFFSET)
       return false;
-    }
-    last_span = {new_buf, new_size};
+
+    last_span = {new_buf.get(), new_size};
+    result = std::move(new_buf);
   }
   ImageEncoding->clear();
   *pImageParams = nullptr;
-  *dest_buf = last_span.data();
+  *dest_buf = std::move(result);
   *dest_size = last_span.size();
   return true;
 }
 
-WideString PDF_DecodeText(const uint8_t* src_data, uint32_t src_len) {
+WideString PDF_DecodeText(pdfium::span<const uint8_t> span) {
   int dest_pos = 0;
   WideString result;
-  if (src_len >= 2 && ((src_data[0] == 0xfe && src_data[1] == 0xff) ||
-                       (src_data[0] == 0xff && src_data[1] == 0xfe))) {
-    uint32_t max_chars = (src_len - 2) / 2;
+  if (span.size() >= 2 && ((span[0] == 0xfe && span[1] == 0xff) ||
+                           (span[0] == 0xff && span[1] == 0xfe))) {
+    uint32_t max_chars = (span.size() - 2) / 2;
     if (!max_chars)
       return result;
 
     pdfium::span<wchar_t> dest_buf = result.GetBuffer(max_chars);
     uint16_t (*GetUnicodeFromBytes)(const uint8_t*) =
-        src_data[0] == 0xfe ? GetUnicodeFromBigEndianBytes
-                            : GetUnicodeFromLittleEndianBytes;
-    const uint8_t* unicode_str = src_data + 2;
+        span[0] == 0xfe ? GetUnicodeFromBigEndianBytes
+                        : GetUnicodeFromLittleEndianBytes;
+    const uint8_t* unicode_str = &span[2];
     for (uint32_t i = 0; i < max_chars * 2; i += 2) {
       uint16_t unicode = GetUnicodeFromBytes(unicode_str + i);
 
@@ -483,20 +498,15 @@ WideString PDF_DecodeText(const uint8_t* src_data, uint32_t src_len) {
       dest_buf[dest_pos++] = unicode;
     }
   } else {
-    pdfium::span<wchar_t> dest_buf = result.GetBuffer(src_len);
-    for (uint32_t i = 0; i < src_len; ++i)
-      dest_buf[i] = PDFDocEncoding[src_data[i]];
-    dest_pos = src_len;
+    pdfium::span<wchar_t> dest_buf = result.GetBuffer(span.size());
+    for (uint32_t i = 0; i < span.size(); ++i)
+      dest_buf[i] = PDFDocEncoding[span[i]];
+    dest_pos = span.size();
   }
   result.ReleaseBuffer(dest_pos);
   return result;
 }
 
-WideString PDF_DecodeText(const ByteString& bstr) {
-  return PDF_DecodeText(reinterpret_cast<const uint8_t*>(bstr.c_str()),
-                        bstr.GetLength());
-}
-
 ByteString PDF_EncodeText(const WideString& str) {
   size_t i = 0;
   size_t len = str.GetLength();
@@ -574,7 +584,7 @@ ByteString PDF_EncodeString(const ByteString& src, bool bHex) {
 }
 
 bool FlateEncode(pdfium::span<const uint8_t> src_span,
-                 uint8_t** dest_buf,
+                 std::unique_ptr<uint8_t, FxFreeDeleter>* dest_buf,
                  uint32_t* dest_size) {
   CCodec_ModuleMgr* pEncoders = CPDF_ModuleMgr::Get()->GetCodecModule();
   return pEncoders->GetFlateModule()->Encode(src_span.data(), src_span.size(),
