@@ -6,7 +6,10 @@
 
 #include "core/fpdfapi/render/cpdf_docrenderdata.h"
 
+#include <array>
 #include <memory>
+#include <utility>
+#include <vector>
 
 #include "core/fpdfapi/font/cpdf_type3font.h"
 #include "core/fpdfapi/page/cpdf_function.h"
@@ -25,24 +28,7 @@ const int kMaxOutputs = 16;
 CPDF_DocRenderData::CPDF_DocRenderData(CPDF_Document* pPDFDoc)
     : m_pPDFDoc(pPDFDoc) {}
 
-CPDF_DocRenderData::~CPDF_DocRenderData() {
-  Clear(true);
-}
-
-void CPDF_DocRenderData::Clear(bool bRelease) {
-  for (auto it = m_Type3FaceMap.begin(); it != m_Type3FaceMap.end();) {
-    auto curr_it = it++;
-    if (bRelease || curr_it->second->HasOneRef()) {
-      m_Type3FaceMap.erase(curr_it);
-    }
-  }
-
-  for (auto it = m_TransferFuncMap.begin(); it != m_TransferFuncMap.end();) {
-    auto curr_it = it++;
-    if (bRelease || curr_it->second->HasOneRef())
-      m_TransferFuncMap.erase(curr_it);
-  }
-}
+CPDF_DocRenderData::~CPDF_DocRenderData() = default;
 
 RetainPtr<CPDF_Type3Cache> CPDF_DocRenderData::GetCachedType3(
     CPDF_Type3Font* pFont) {
@@ -70,12 +56,22 @@ RetainPtr<CPDF_TransferFunc> CPDF_DocRenderData::GetTransferFunc(
   if (it != m_TransferFuncMap.end())
     return it->second;
 
+  m_TransferFuncMap[pObj] = CreateTransferFunc(pObj);
+  return m_TransferFuncMap[pObj];
+}
+
+void CPDF_DocRenderData::MaybePurgeTransferFunc(const CPDF_Object* pObj) {
+  auto it = m_TransferFuncMap.find(pObj);
+  if (it != m_TransferFuncMap.end() && it->second->HasOneRef())
+    m_TransferFuncMap.erase(it);
+}
+
+RetainPtr<CPDF_TransferFunc> CPDF_DocRenderData::CreateTransferFunc(
+    const CPDF_Object* pObj) const {
   std::unique_ptr<CPDF_Function> pFuncs[3];
-  bool bUniTransfer = true;
-  bool bIdentity = true;
-  if (const CPDF_Array* pArray = pObj->AsArray()) {
-    bUniTransfer = false;
-    if (pArray->GetCount() < 3)
+  const CPDF_Array* pArray = pObj->AsArray();
+  if (pArray) {
+    if (pArray->size() < 3)
       return nullptr;
 
     for (uint32_t i = 0; i < 3; ++i) {
@@ -88,44 +84,43 @@ RetainPtr<CPDF_TransferFunc> CPDF_DocRenderData::GetTransferFunc(
     if (!pFuncs[0])
       return nullptr;
   }
-  auto pTransfer = pdfium::MakeRetain<CPDF_TransferFunc>(m_pPDFDoc.Get());
-  m_TransferFuncMap[pObj] = pTransfer;
 
-  float input;
   int noutput;
   float output[kMaxOutputs];
   memset(output, 0, sizeof(output));
-  for (int v = 0; v < 256; ++v) {
-    input = (float)v / 255.0f;
-    if (bUniTransfer) {
-      if (pFuncs[0] && pFuncs[0]->CountOutputs() <= kMaxOutputs)
-        pFuncs[0]->Call(&input, 1, output, &noutput);
-      int o = FXSYS_round(output[0] * 255);
-      if (o != v)
-        bIdentity = false;
-      for (int i = 0; i < 3; ++i)
-        pTransfer->GetSamples()[i * 256 + v] = o;
+
+  bool bIdentity = true;
+  std::vector<uint8_t> samples_r(CPDF_TransferFunc::kChannelSampleSize);
+  std::vector<uint8_t> samples_g(CPDF_TransferFunc::kChannelSampleSize);
+  std::vector<uint8_t> samples_b(CPDF_TransferFunc::kChannelSampleSize);
+  std::array<pdfium::span<uint8_t>, 3> samples = {samples_r, samples_g,
+                                                  samples_b};
+  for (size_t v = 0; v < CPDF_TransferFunc::kChannelSampleSize; ++v) {
+    float input = static_cast<float>(v) / 255.0f;
+    if (pArray) {
+      for (int i = 0; i < 3; ++i) {
+        if (pFuncs[i]->CountOutputs() > kMaxOutputs) {
+          samples[i][v] = v;
+          continue;
+        }
+        pFuncs[i]->Call(&input, 1, output, &noutput);
+        size_t o = FXSYS_round(output[0] * 255);
+        if (o != v)
+          bIdentity = false;
+        samples[i][v] = o;
+      }
       continue;
     }
-    for (int i = 0; i < 3; ++i) {
-      if (!pFuncs[i] || pFuncs[i]->CountOutputs() > kMaxOutputs) {
-        pTransfer->GetSamples()[i * 256 + v] = v;
-        continue;
-      }
-      pFuncs[i]->Call(&input, 1, output, &noutput);
-      int o = FXSYS_round(output[0] * 255);
-      if (o != v)
-        bIdentity = false;
-      pTransfer->GetSamples()[i * 256 + v] = o;
-    }
+    if (pFuncs[0]->CountOutputs() <= kMaxOutputs)
+      pFuncs[0]->Call(&input, 1, output, &noutput);
+    size_t o = FXSYS_round(output[0] * 255);
+    if (o != v)
+      bIdentity = false;
+    for (auto& channel : samples)
+      channel[v] = o;
   }
 
-  pTransfer->SetIdentity(bIdentity);
-  return pTransfer;
-}
-
-void CPDF_DocRenderData::MaybePurgeTransferFunc(const CPDF_Object* pObj) {
-  auto it = m_TransferFuncMap.find(pObj);
-  if (it != m_TransferFuncMap.end() && it->second->HasOneRef())
-    m_TransferFuncMap.erase(it);
+  return pdfium::MakeRetain<CPDF_TransferFunc>(
+      m_pPDFDoc.Get(), bIdentity, std::move(samples_r), std::move(samples_g),
+      std::move(samples_b));
 }
