@@ -7,19 +7,16 @@
 #ifndef CORE_FPDFAPI_RENDER_CPDF_IMAGECACHEENTRY_H_
 #define CORE_FPDFAPI_RENDER_CPDF_IMAGECACHEENTRY_H_
 
-#include <memory>
-
+#include "core/fpdfapi/render/cpdf_dibbase.h"
 #include "core/fxcrt/fx_system.h"
 #include "core/fxcrt/retain_ptr.h"
 #include "core/fxcrt/unowned_ptr.h"
 
-class CFX_DIBSource;
-class CFX_DIBitmap;
 class CPDF_Dictionary;
 class CPDF_Document;
 class CPDF_Image;
 class CPDF_RenderStatus;
-class IFX_PauseIndicator;
+class PauseIndicatorIface;
 
 class CPDF_ImageCacheEntry {
  public:
@@ -27,22 +24,27 @@ class CPDF_ImageCacheEntry {
                        const RetainPtr<CPDF_Image>& pImage);
   ~CPDF_ImageCacheEntry();
 
-  void Reset(const RetainPtr<CFX_DIBitmap>& pBitmap);
+  void Reset();
   uint32_t EstimateSize() const { return m_dwCacheSize; }
   uint32_t GetTimeCount() const { return m_dwTimeCount; }
   CPDF_Image* GetImage() const { return m_pImage.Get(); }
-  int StartGetCachedBitmap(CPDF_Dictionary* pFormResources,
-                           CPDF_Dictionary* pPageResources,
-                           bool bStdCS,
-                           uint32_t GroupFamily,
-                           bool bLoadMask,
-                           CPDF_RenderStatus* pRenderStatus);
-  int Continue(IFX_PauseIndicator* pPause, CPDF_RenderStatus* pRenderStatus);
-  RetainPtr<CFX_DIBSource> DetachBitmap();
-  RetainPtr<CFX_DIBSource> DetachMask();
 
-  int m_dwTimeCount;
-  uint32_t m_MatteColor;
+  CPDF_DIBBase::LoadState StartGetCachedBitmap(
+      const CPDF_Dictionary* pFormResources,
+      CPDF_Dictionary* pPageResources,
+      bool bStdCS,
+      uint32_t GroupFamily,
+      bool bLoadMask,
+      CPDF_RenderStatus* pRenderStatus);
+
+  // Returns whether to Continue() or not.
+  bool Continue(PauseIndicatorIface* pPause, CPDF_RenderStatus* pRenderStatus);
+
+  RetainPtr<CFX_DIBBase> DetachBitmap();
+  RetainPtr<CFX_DIBBase> DetachMask();
+
+  int m_dwTimeCount = 0;
+  uint32_t m_MatteColor = 0;
 
  private:
   void ContinueGetCachedBitmap(CPDF_RenderStatus* pRenderStatus);
@@ -50,11 +52,11 @@ class CPDF_ImageCacheEntry {
 
   UnownedPtr<CPDF_Document> const m_pDocument;
   RetainPtr<CPDF_Image> const m_pImage;
-  RetainPtr<CFX_DIBSource> m_pCurBitmap;
-  RetainPtr<CFX_DIBSource> m_pCurMask;
-  RetainPtr<CFX_DIBSource> m_pCachedBitmap;
-  RetainPtr<CFX_DIBSource> m_pCachedMask;
-  uint32_t m_dwCacheSize;
+  RetainPtr<CFX_DIBBase> m_pCurBitmap;
+  RetainPtr<CFX_DIBBase> m_pCurMask;
+  RetainPtr<CFX_DIBBase> m_pCachedBitmap;
+  RetainPtr<CFX_DIBBase> m_pCachedMask;
+  uint32_t m_dwCacheSize = 0;
 };
 
 #endif  // CORE_FPDFAPI_RENDER_CPDF_IMAGECACHEENTRY_H_
