@@ -16,11 +16,13 @@
 #include "core/fpdfapi/render/cpdf_renderoptions.h"
 #include "core/fpdfapi/render/cpdf_renderstatus.h"
 #include "core/fpdfapi/render/cpdf_textrenderer.h"
+#include "core/fxge/cfx_defaultrenderdevice.h"
 #include "core/fxge/cfx_renderdevice.h"
+#include "core/fxge/dib/cfx_dibitmap.h"
 #include "core/fxge/fx_dib.h"
 
 CPDF_RenderContext::CPDF_RenderContext(CPDF_Page* pPage)
-    : m_pDocument(pPage->m_pDocument.Get()),
+    : m_pDocument(pPage->GetDocument()),
       m_pPageResources(pPage->m_pPageResources.Get()),
       m_pPageCache(pPage->GetRenderCache()) {}
 
@@ -37,8 +39,8 @@ void CPDF_RenderContext::GetBackground(const RetainPtr<CFX_DIBitmap>& pBuffer,
   CFX_DefaultRenderDevice device;
   device.Attach(pBuffer, false, nullptr, false);
 
-  FX_RECT rect(0, 0, device.GetWidth(), device.GetHeight());
-  device.FillRect(&rect, 0xffffffff);
+  device.FillRect(FX_RECT(0, 0, device.GetWidth(), device.GetHeight()),
+                  0xffffffff);
   Render(&device, pObj, pOptions, pFinalMatrix);
 }
 
@@ -48,8 +50,6 @@ void CPDF_RenderContext::AppendLayer(CPDF_PageObjectHolder* pObjectHolder,
   m_Layers.back().m_pObjectHolder = pObjectHolder;
   if (pObject2Device)
     m_Layers.back().m_Matrix = *pObject2Device;
-  else
-    m_Layers.back().m_Matrix.SetIdentity();
 }
 
 void CPDF_RenderContext::Render(CFX_RenderDevice* pDevice,
@@ -64,23 +64,24 @@ void CPDF_RenderContext::Render(CFX_RenderDevice* pDevice,
                                 const CFX_Matrix* pLastMatrix) {
   for (auto& layer : m_Layers) {
     CFX_RenderDevice::StateRestorer restorer(pDevice);
-    CPDF_RenderStatus status;
+    CPDF_RenderStatus status(this, pDevice);
+    if (pOptions)
+      status.SetOptions(*pOptions);
+    status.SetStopObject(pStopObj);
+    status.SetTransparency(layer.m_pObjectHolder->GetTransparency());
     if (pLastMatrix) {
-      CFX_Matrix FinalMatrix = layer.m_Matrix;
-      FinalMatrix.Concat(*pLastMatrix);
-      status.Initialize(this, pDevice, pLastMatrix, pStopObj, nullptr, nullptr,
-                        pOptions, layer.m_pObjectHolder->m_Transparency, false,
-                        nullptr);
-      status.RenderObjectList(layer.m_pObjectHolder.Get(), &FinalMatrix);
+      const CFX_Matrix& last_matrix = *pLastMatrix;
+      CFX_Matrix final_matrix = layer.m_Matrix * last_matrix;
+      status.SetDeviceMatrix(last_matrix);
+      status.Initialize(nullptr, nullptr);
+      status.RenderObjectList(layer.m_pObjectHolder.Get(), final_matrix);
     } else {
-      status.Initialize(this, pDevice, nullptr, pStopObj, nullptr, nullptr,
-                        pOptions, layer.m_pObjectHolder->m_Transparency, false,
-                        nullptr);
-      status.RenderObjectList(layer.m_pObjectHolder.Get(), &layer.m_Matrix);
+      status.Initialize(nullptr, nullptr);
+      status.RenderObjectList(layer.m_pObjectHolder.Get(), layer.m_Matrix);
     }
-    if (status.GetRenderOptions()->HasFlag(RENDER_LIMITEDIMAGECACHE)) {
+    if (status.GetRenderOptions().GetOptions().bLimitedImageCache) {
       m_pPageCache->CacheOptimization(
-          status.GetRenderOptions()->GetCacheSizeLimit());
+          status.GetRenderOptions().GetCacheSizeLimit());
     }
     if (status.IsStopped())
       break;
