@@ -27,7 +27,9 @@
 
 namespace {
 
-const float kDefaultFontSize = 1.0f;
+constexpr float kDefaultFontSize = 1.0f;
+constexpr float kSizeEpsilon = 0.01f;
+
 const uint16_t* const g_UnicodeData_Normalization_Maps[] = {
     g_UnicodeData_Normalization_Map2, g_UnicodeData_Normalization_Map3,
     g_UnicodeData_Normalization_Map4};
@@ -46,26 +48,27 @@ float NormalizeThreshold(float threshold, int t1, int t2, int t3) {
 
 float CalculateBaseSpace(const CPDF_TextObject* pTextObj,
                          const CFX_Matrix& matrix) {
-  float baseSpace = 0.0;
   const size_t nItems = pTextObj->CountItems();
-  if (pTextObj->m_TextState.GetCharSpace() && nItems >= 3) {
-    bool bAllChar = true;
-    float spacing =
-        matrix.TransformDistance(pTextObj->m_TextState.GetCharSpace());
-    baseSpace = spacing;
-    for (size_t i = 0; i < nItems; ++i) {
-      CPDF_TextObjectItem item;
-      pTextObj->GetItemInfo(i, &item);
-      if (item.m_CharCode == static_cast<uint32_t>(-1)) {
-        float fontsize_h = pTextObj->m_TextState.GetFontSizeH();
-        float kerning = -fontsize_h * item.m_Origin.x / 1000;
-        baseSpace = std::min(baseSpace, kerning + spacing);
-        bAllChar = false;
-      }
+  if (!pTextObj->m_TextState.GetCharSpace() || nItems < 3)
+    return 0.0f;
+
+  bool bAllChar = true;
+  float spacing =
+      matrix.TransformDistance(pTextObj->m_TextState.GetCharSpace());
+  float baseSpace = spacing;
+  for (size_t i = 0; i < nItems; ++i) {
+    CPDF_TextObjectItem item;
+    pTextObj->GetItemInfo(i, &item);
+    if (item.m_CharCode == static_cast<uint32_t>(-1)) {
+      float fontsize_h = pTextObj->m_TextState.GetFontSizeH();
+      float kerning = -fontsize_h * item.m_Origin.x / 1000;
+      baseSpace = std::min(baseSpace, kerning + spacing);
+      bAllChar = false;
     }
-    if (baseSpace < 0.0 || (nItems == 3 && !bAllChar))
-      baseSpace = 0.0;
   }
+  if (baseSpace < 0.0 || (nItems == 3 && !bAllChar))
+    return 0.0f;
+
   return baseSpace;
 }
 
@@ -88,7 +91,7 @@ size_t Unicode_GetNormalization(wchar_t wch, wchar_t* pDst) {
   wFind >>= 12;
   const uint16_t* pMap = g_UnicodeData_Normalization_Maps[wFind - 2] + wch;
   if (wFind == 4)
-    wFind = (wchar_t)(*pMap++);
+    wFind = static_cast<wchar_t>(*pMap++);
 
   if (pDst) {
     wchar_t n = wFind;
@@ -151,8 +154,11 @@ uint32_t GetCharWidth(uint32_t charCode, CPDF_Font* pFont) {
   if (w > 0)
     return w;
 
-  ASSERT(pFont->GetCharBBox(charCode).Width() >= 0);
-  return pFont->GetCharBBox(charCode).Width();
+  FX_RECT rect = pFont->GetCharBBox(charCode);
+  if (!rect.Valid())
+    return 0;
+
+  return std::max(rect.Width(), 0);
 }
 
 bool GenerateSpace(const CFX_PointF& pos,
@@ -160,19 +166,41 @@ bool GenerateSpace(const CFX_PointF& pos,
                    float this_width,
                    float last_width,
                    float threshold) {
-  if (fabs(last_pos + last_width - pos.x) > threshold) {
-    if ((pos.x - last_pos - last_width) > threshold ||
-        (last_pos - pos.x - last_width) > threshold) {
-      return true;
-    }
-    if (pos.x < 0 && (last_pos - pos.x - last_width) > threshold)
-      return true;
-    if ((pos.x - last_pos - last_width) > this_width ||
-        (pos.x - last_pos - this_width) > last_width) {
-      return true;
-    }
+  if (fabs(last_pos + last_width - pos.x) <= threshold)
+    return false;
+
+  float threshold_pos = threshold + last_width;
+  float pos_difference = pos.x - last_pos;
+  if (fabs(pos_difference) > threshold_pos)
+    return true;
+  if (pos.x < 0 && -threshold_pos > pos_difference)
+    return true;
+  return pos_difference > this_width + last_width;
+}
+
+bool EndHorizontalLine(const CFX_FloatRect& this_rect,
+                       const CFX_FloatRect& prev_rect) {
+  if (this_rect.Height() <= 4.5 || prev_rect.Height() <= 4.5)
+    return false;
+
+  float top = std::min(this_rect.top, prev_rect.top);
+  float bottom = std::max(this_rect.bottom, prev_rect.bottom);
+  return bottom >= top;
+}
+
+bool EndVerticalLine(const CFX_FloatRect& this_rect,
+                     const CFX_FloatRect& prev_rect,
+                     const CFX_FloatRect& curline_rect,
+                     float this_fontsize,
+                     float prev_fontsize) {
+  if (this_rect.Width() <= this_fontsize * 0.1f ||
+      prev_rect.Width() <= prev_fontsize * 0.1f) {
+    return false;
   }
-  return false;
+
+  float left = std::max(this_rect.left, curline_rect.left);
+  float right = std::min(this_rect.right, curline_rect.right);
+  return right <= left;
 }
 
 }  // namespace
@@ -195,7 +223,7 @@ PAGECHAR_INFO::~PAGECHAR_INFO() {}
 
 CPDF_TextPage::CPDF_TextPage(const CPDF_Page* pPage, FPDFText_Direction flags)
     : m_pPage(pPage), m_parserflag(flags) {
-  m_TextBuf.EstimateSize(0, 10240);
+  m_TextBuf.SetAllocStep(10240);
   const FX_RECT rect(0, 0, static_cast<int>(pPage->GetPageWidth()),
                      static_cast<int>(pPage->GetPageHeight()));
   m_DisplayMatrix = pPage->GetDisplayMatrix(rect, 0);
@@ -232,7 +260,7 @@ void CPDF_TextPage::ParseTextPage() {
   if (nCount)
     m_CharIndex.push_back(0);
 
-  for (int i = 0; i < nCount; i++) {
+  for (int i = 0; i < nCount; ++i) {
     int indexSize = pdfium::CollectionSize<int>(m_CharIndex);
     const PAGECHAR_INFO& charinfo = m_CharList[i];
     if (charinfo.m_Flag == FPDFTEXT_CHAR_GENERATED ||
@@ -312,8 +340,8 @@ std::vector<CFX_FloatRect> CPDF_TextPage::GetRectArray(int start,
     PAGECHAR_INFO info_curchar = m_CharList[curPos++];
     if (info_curchar.m_Flag == FPDFTEXT_CHAR_GENERATED)
       continue;
-    if (info_curchar.m_CharBox.Width() < 0.01 ||
-        info_curchar.m_CharBox.Height() < 0.01) {
+    if (info_curchar.m_CharBox.Width() < kSizeEpsilon ||
+        info_curchar.m_CharBox.Height() < kSizeEpsilon) {
       continue;
     }
     if (!pCurObj)
@@ -324,40 +352,16 @@ std::vector<CFX_FloatRect> CPDF_TextPage::GetRectArray(int start,
       bFlagNewRect = true;
     }
     if (bFlagNewRect) {
-      CFX_Matrix matrix = info_curchar.m_pTextObj->GetTextMatrix();
-      matrix.Concat(info_curchar.m_Matrix);
-
-      CFX_PointF origin = matrix.GetInverse().Transform(info_curchar.m_Origin);
-      rect.left = info_curchar.m_CharBox.left;
-      rect.right = info_curchar.m_CharBox.right;
-      if (pCurObj->GetFont()->GetTypeDescent()) {
-        const float fFontSize = pCurObj->GetFontSize() / 1000;
-        rect.bottom =
-            origin.y + pCurObj->GetFont()->GetTypeDescent() * fFontSize;
-        rect.bottom = matrix.Transform(CFX_PointF(origin.x, rect.bottom)).y;
-      } else {
-        rect.bottom = info_curchar.m_CharBox.bottom;
-      }
-      if (pCurObj->GetFont()->GetTypeAscent()) {
-        const float fFontSize = pCurObj->GetFontSize() / 1000;
-        rect.top = origin.y + pCurObj->GetFont()->GetTypeAscent() * fFontSize;
-        float xPosTemp =
-            GetCharWidth(info_curchar.m_CharCode, pCurObj->GetFont());
-        xPosTemp = xPosTemp * fFontSize + origin.x;
-        rect.top = matrix.Transform(CFX_PointF(xPosTemp, rect.top)).y;
-      } else {
-        rect.top = info_curchar.m_CharBox.top;
-      }
       bFlagNewRect = false;
       rect = info_curchar.m_CharBox;
       rect.Normalize();
-    } else {
-      info_curchar.m_CharBox.Normalize();
-      rect.left = std::min(rect.left, info_curchar.m_CharBox.left);
-      rect.right = std::max(rect.right, info_curchar.m_CharBox.right);
-      rect.top = std::max(rect.top, info_curchar.m_CharBox.top);
-      rect.bottom = std::min(rect.bottom, info_curchar.m_CharBox.bottom);
+      continue;
     }
+    info_curchar.m_CharBox.Normalize();
+    rect.left = std::min(rect.left, info_curchar.m_CharBox.left);
+    rect.right = std::max(rect.right, info_curchar.m_CharBox.right);
+    rect.top = std::max(rect.top, info_curchar.m_CharBox.top);
+    rect.bottom = std::min(rect.bottom, info_curchar.m_CharBox.bottom);
   }
   rects.push_back(rect);
   return rects;
@@ -404,7 +408,8 @@ int CPDF_TextPage::GetIndexAtPos(const CFX_PointF& point,
   return pos < nCount ? pos : NearPos;
 }
 
-WideString CPDF_TextPage::GetTextByRect(const CFX_FloatRect& rect) const {
+WideString CPDF_TextPage::GetTextByPredicate(
+    const std::function<bool(const PAGECHAR_INFO&)>& predicate) const {
   if (!m_bIsParsed)
     return WideString();
 
@@ -413,7 +418,7 @@ WideString CPDF_TextPage::GetTextByRect(const CFX_FloatRect& rect) const {
   bool IsAddLineFeed = false;
   WideString strText;
   for (const auto& charinfo : m_CharList) {
-    if (IsRectIntersect(rect, charinfo.m_CharBox)) {
+    if (predicate(charinfo)) {
       if (fabs(posy - charinfo.m_Origin.y) > 0 && !IsContainPreChar &&
           IsAddLineFeed) {
         posy = charinfo.m_Origin.y;
@@ -424,9 +429,9 @@ WideString CPDF_TextPage::GetTextByRect(const CFX_FloatRect& rect) const {
       IsAddLineFeed = false;
       if (charinfo.m_Unicode)
         strText += charinfo.m_Unicode;
-    } else if (charinfo.m_Unicode == 32) {
-      if (IsContainPreChar && charinfo.m_Unicode) {
-        strText += charinfo.m_Unicode;
+    } else if (charinfo.m_Unicode == L' ') {
+      if (IsContainPreChar) {
+        strText += L' ';
         IsContainPreChar = false;
         IsAddLineFeed = false;
       }
@@ -438,8 +443,21 @@ WideString CPDF_TextPage::GetTextByRect(const CFX_FloatRect& rect) const {
   return strText;
 }
 
-void CPDF_TextPage::GetCharInfo(int index, FPDF_CHAR_INFO* info) const {
-  if (!m_bIsParsed || !pdfium::IndexInBounds(m_CharList, index))
+WideString CPDF_TextPage::GetTextByRect(const CFX_FloatRect& rect) const {
+  return GetTextByPredicate([&rect](const PAGECHAR_INFO& charinfo) {
+    return IsRectIntersect(rect, charinfo.m_CharBox);
+  });
+}
+
+WideString CPDF_TextPage::GetTextByObject(
+    const CPDF_TextObject* pTextObj) const {
+  return GetTextByPredicate([pTextObj](const PAGECHAR_INFO& charinfo) {
+    return charinfo.m_pTextObj == pTextObj;
+  });
+}
+
+void CPDF_TextPage::GetCharInfo(size_t index, FPDF_CHAR_INFO* info) const {
+  if (!m_bIsParsed || index >= size())
     return;
 
   const PAGECHAR_INFO& charinfo = m_CharList[index];
@@ -455,60 +473,43 @@ void CPDF_TextPage::GetCharInfo(int index, FPDF_CHAR_INFO* info) const {
   info->m_Matrix = charinfo.m_Matrix;
 }
 
-void CPDF_TextPage::CheckMarkedContentObject(int32_t* pStart,
-                                             int32_t* pCount) const {
-  int start = *pStart;
-  const int nCount = *pCount;
-  PAGECHAR_INFO charinfo = m_CharList[start];
-  PAGECHAR_INFO charinfo2 = m_CharList[start + nCount - 1];
-  if (FPDFTEXT_CHAR_PIECE != charinfo.m_Flag &&
-      FPDFTEXT_CHAR_PIECE != charinfo2.m_Flag) {
-    return;
-  }
-  if (FPDFTEXT_CHAR_PIECE == charinfo.m_Flag) {
-    PAGECHAR_INFO charinfo1 = charinfo;
-    while (FPDFTEXT_CHAR_PIECE == charinfo1.m_Flag &&
-           charinfo1.m_Index == charinfo.m_Index) {
-      start--;
-      if (start < 0)
-        break;
-      charinfo1 = m_CharList[start];
-    }
-    start++;
-    *pStart = start;
-  }
-  if (FPDFTEXT_CHAR_PIECE == charinfo2.m_Flag) {
-    PAGECHAR_INFO charinfo3 = charinfo2;
-    int endIndex = start + nCount - 1;
-    const int nCount = CountChars();
-    while (FPDFTEXT_CHAR_PIECE == charinfo3.m_Flag &&
-           charinfo3.m_Index == charinfo2.m_Index) {
-      endIndex++;
-      if (endIndex >= nCount)
-        break;
-      charinfo3 = m_CharList[endIndex];
-    }
-    endIndex--;
-    *pCount = endIndex - start + 1;
-  }
-}
-
 WideString CPDF_TextPage::GetPageText(int start, int count) const {
   if (start < 0 || start >= CountChars() || count <= 0 || !m_bIsParsed ||
       m_CharList.empty() || m_TextBuf.GetLength() == 0) {
-    return L"";
+    return WideString();
   }
 
+  const int count_chars = CountChars();
   int text_start = TextIndexFromCharIndex(start);
-  if (text_start < 0)
-    return L"";
 
-  count = std::min(count, CountChars() - start);
+  // If the character at |start| is a non-printing character, then
+  // TextIndexFromCharIndex will return -1, so scan ahead to the first printing
+  // character.
+  while (text_start < 0) {
+    if (start >= count_chars)
+      return WideString();
+    start++;
+    text_start = TextIndexFromCharIndex(start);
+  }
+
+  count = std::min(count, count_chars - start);
 
   int last = start + count - 1;
   int text_last = TextIndexFromCharIndex(last);
-  if (text_last < 0 || text_last < text_start)
-    return L"";
+
+  // If the character at |last| is a non-printing character, then
+  // TextIndexFromCharIndex will return -1, so scan back to the last printing
+  // character.
+  while (text_last < 0) {
+    if (last < text_start)
+      return WideString();
+
+    last--;
+    text_last = TextIndexFromCharIndex(last);
+  }
+
+  if (text_last < text_start)
+    return WideString();
 
   int text_count = text_last - text_start + 1;
 
@@ -534,7 +535,7 @@ bool CPDF_TextPage::GetRect(int rectIndex, CFX_FloatRect* pRect) const {
 
 CPDF_TextPage::TextOrientation CPDF_TextPage::FindTextlineFlowOrientation()
     const {
-  if (m_pPage->GetPageObjectList()->empty())
+  if (m_pPage->GetPageObjectCount() == 0)
     return TextOrientation::Unknown;
 
   const int32_t nPageWidth = static_cast<int32_t>(m_pPage->GetPageWidth());
@@ -549,15 +550,17 @@ CPDF_TextPage::TextOrientation CPDF_TextPage::FindTextlineFlowOrientation()
   int32_t nEndH = 0;
   int32_t nStartV = nPageHeight;
   int32_t nEndV = 0;
-  for (const auto& pPageObj : *m_pPage->GetPageObjectList()) {
+  for (const auto& pPageObj : *m_pPage) {
     if (!pPageObj->IsText())
       continue;
 
-    int32_t minH = std::max(static_cast<int32_t>(pPageObj->m_Left), 0);
+    int32_t minH = std::max(static_cast<int32_t>(pPageObj->GetRect().left), 0);
     int32_t maxH =
-        std::min(static_cast<int32_t>(pPageObj->m_Right), nPageWidth);
-    int32_t minV = std::max(static_cast<int32_t>(pPageObj->m_Bottom), 0);
-    int32_t maxV = std::min(static_cast<int32_t>(pPageObj->m_Top), nPageHeight);
+        std::min(static_cast<int32_t>(pPageObj->GetRect().right), nPageWidth);
+    int32_t minV =
+        std::max(static_cast<int32_t>(pPageObj->GetRect().bottom), 0);
+    int32_t maxV =
+        std::min(static_cast<int32_t>(pPageObj->GetRect().top), nPageHeight);
     if (minH >= maxH || minV >= maxV)
       continue;
 
@@ -572,7 +575,7 @@ CPDF_TextPage::TextOrientation CPDF_TextPage::FindTextlineFlowOrientation()
     nEndV = std::max(nEndV, maxV);
 
     if (fLineHeight <= 0.0f)
-      fLineHeight = pPageObj->m_Top - pPageObj->m_Bottom;
+      fLineHeight = pPageObj->GetRect().Height();
   }
   const int32_t nDoubleLineHeight = 2 * fLineHeight;
   if ((nEndV - nStartV) < nDoubleLineHeight)
@@ -605,21 +608,20 @@ void CPDF_TextPage::AppendGeneratedCharacter(wchar_t unicode,
 }
 
 void CPDF_TextPage::ProcessObject() {
-  if (m_pPage->GetPageObjectList()->empty())
+  if (m_pPage->GetPageObjectCount() == 0)
     return;
 
   m_TextlineDir = FindTextlineFlowOrientation();
-  const CPDF_PageObjectList* pObjList = m_pPage->GetPageObjectList();
-  for (auto it = pObjList->begin(); it != pObjList->end(); ++it) {
-    if (CPDF_PageObject* pObj = it->get()) {
-      if (pObj->IsText()) {
-        CFX_Matrix matrix;
-        ProcessTextObject(pObj->AsText(), matrix, pObjList, it);
-      } else if (pObj->IsForm()) {
-        CFX_Matrix formMatrix;
-        ProcessFormObject(pObj->AsForm(), formMatrix);
-      }
-    }
+  for (auto it = m_pPage->begin(); it != m_pPage->end(); ++it) {
+    CPDF_PageObject* pObj = it->get();
+    if (!pObj)
+      continue;
+
+    CFX_Matrix matrix;
+    if (pObj->IsText())
+      ProcessTextObject(pObj->AsText(), matrix, m_pPage.Get(), it);
+    else if (pObj->IsForm())
+      ProcessFormObject(pObj->AsForm(), matrix);
   }
   for (const auto& obj : m_LineObj)
     ProcessTextObject(obj);
@@ -630,21 +632,17 @@ void CPDF_TextPage::ProcessObject() {
 
 void CPDF_TextPage::ProcessFormObject(CPDF_FormObject* pFormObj,
                                       const CFX_Matrix& formMatrix) {
-  const CPDF_PageObjectList* pObjectList =
-      pFormObj->form()->GetPageObjectList();
-  if (pObjectList->empty())
-    return;
+  CFX_Matrix curFormMatrix = pFormObj->form_matrix() * formMatrix;
+  const CPDF_PageObjectHolder* pHolder = pFormObj->form();
+  for (auto it = pHolder->begin(); it != pHolder->end(); ++it) {
+    CPDF_PageObject* pPageObj = it->get();
+    if (!pPageObj)
+      continue;
 
-  CFX_Matrix curFormMatrix = pFormObj->form_matrix();
-  curFormMatrix.Concat(formMatrix);
-
-  for (auto it = pObjectList->begin(); it != pObjectList->end(); ++it) {
-    if (CPDF_PageObject* pPageObj = it->get()) {
-      if (pPageObj->IsText())
-        ProcessTextObject(pPageObj->AsText(), curFormMatrix, pObjectList, it);
-      else if (pPageObj->IsForm())
-        ProcessFormObject(pPageObj->AsForm(), curFormMatrix);
-    }
+    if (pPageObj->IsText())
+      ProcessTextObject(pPageObj->AsText(), curFormMatrix, pHolder, it);
+    else if (pPageObj->IsForm())
+      ProcessFormObject(pPageObj->AsForm(), curFormMatrix);
   }
 }
 
@@ -658,22 +656,23 @@ void CPDF_TextPage::AddCharInfoByLRDirection(wchar_t wChar,
   }
 
   info2.m_Index = m_TextBuf.GetLength();
-  if (wChar >= 0xFB00 && wChar <= 0xFB06) {
-    size_t nCount = Unicode_GetNormalization(wChar, nullptr);
-    if (nCount >= 1) {
-      std::unique_ptr<wchar_t, FxFreeDeleter> pDst(FX_Alloc(wchar_t, nCount));
-      Unicode_GetNormalization(wChar, pDst.get());
-      for (size_t nIndex = 0; nIndex < nCount; nIndex++) {
-        info2.m_Unicode = pDst.get()[nIndex];
-        info2.m_Flag = FPDFTEXT_CHAR_PIECE;
-        m_TextBuf.AppendChar(info2.m_Unicode);
-        m_CharList.push_back(info2);
-      }
-      return;
-    }
+  size_t nCount = 0;
+  if (wChar >= 0xFB00 && wChar <= 0xFB06)
+    nCount = Unicode_GetNormalization(wChar, nullptr);
+  if (nCount == 0) {
+    m_TextBuf.AppendChar(wChar);
+    m_CharList.push_back(info2);
+    return;
+  }
+
+  std::unique_ptr<wchar_t, FxFreeDeleter> pDst(FX_Alloc(wchar_t, nCount));
+  Unicode_GetNormalization(wChar, pDst.get());
+  for (size_t nIndex = 0; nIndex < nCount; ++nIndex) {
+    info2.m_Unicode = pDst.get()[nIndex];
+    info2.m_Flag = FPDFTEXT_CHAR_PIECE;
+    m_TextBuf.AppendChar(info2.m_Unicode);
+    m_CharList.push_back(info2);
   }
-  m_TextBuf.AppendChar(wChar);
-  m_CharList.push_back(info2);
 }
 
 void CPDF_TextPage::AddCharInfoByRLDirection(wchar_t wChar,
@@ -688,20 +687,21 @@ void CPDF_TextPage::AddCharInfoByRLDirection(wchar_t wChar,
   info2.m_Index = m_TextBuf.GetLength();
   wChar = FX_GetMirrorChar(wChar);
   size_t nCount = Unicode_GetNormalization(wChar, nullptr);
-  if (nCount >= 1) {
-    std::unique_ptr<wchar_t, FxFreeDeleter> pDst(FX_Alloc(wchar_t, nCount));
-    Unicode_GetNormalization(wChar, pDst.get());
-    for (size_t nIndex = 0; nIndex < nCount; nIndex++) {
-      info2.m_Unicode = pDst.get()[nIndex];
-      info2.m_Flag = FPDFTEXT_CHAR_PIECE;
-      m_TextBuf.AppendChar(info2.m_Unicode);
-      m_CharList.push_back(info2);
-    }
+  if (nCount == 0) {
+    info2.m_Unicode = wChar;
+    m_TextBuf.AppendChar(info2.m_Unicode);
+    m_CharList.push_back(info2);
     return;
   }
-  info2.m_Unicode = wChar;
-  m_TextBuf.AppendChar(info2.m_Unicode);
-  m_CharList.push_back(info2);
+
+  std::unique_ptr<wchar_t, FxFreeDeleter> pDst(FX_Alloc(wchar_t, nCount));
+  Unicode_GetNormalization(wChar, pDst.get());
+  for (size_t nIndex = 0; nIndex < nCount; ++nIndex) {
+    info2.m_Unicode = pDst.get()[nIndex];
+    info2.m_Flag = FPDFTEXT_CHAR_PIECE;
+    m_TextBuf.AppendChar(info2.m_Unicode);
+    m_CharList.push_back(info2);
+  }
 }
 
 void CPDF_TextPage::CloseTempLine() {
@@ -710,7 +710,7 @@ void CPDF_TextPage::CloseTempLine() {
 
   WideString str = m_TempTextBuf.MakeString();
   bool bPrevSpace = false;
-  for (size_t i = 0; i < str.GetLength(); i++) {
+  for (size_t i = 0; i < str.GetLength(); ++i) {
     if (str[i] != ' ') {
       bPrevSpace = false;
       continue;
@@ -719,7 +719,7 @@ void CPDF_TextPage::CloseTempLine() {
       m_TempTextBuf.Delete(i, 1);
       m_TempCharList.erase(m_TempCharList.begin() + i);
       str.Delete(i);
-      i--;
+      --i;
     }
     bPrevSpace = true;
   }
@@ -736,7 +736,7 @@ void CPDF_TextPage::CloseTempLine() {
         AddCharInfoByRLDirection(str[m - 1], m_TempCharList[m - 1]);
     } else {
       eCurrentDirection = CFX_BidiChar::LEFT;
-      for (int m = segment.start; m < segment.start + segment.count; m++)
+      for (int m = segment.start; m < segment.start + segment.count; ++m)
         AddCharInfoByLRDirection(str[m], m_TempCharList[m]);
     }
   }
@@ -747,9 +747,9 @@ void CPDF_TextPage::CloseTempLine() {
 void CPDF_TextPage::ProcessTextObject(
     CPDF_TextObject* pTextObj,
     const CFX_Matrix& formMatrix,
-    const CPDF_PageObjectList* pObjList,
-    CPDF_PageObjectList::const_iterator ObjPos) {
-  if (fabs(pTextObj->m_Right - pTextObj->m_Left) < 0.01f)
+    const CPDF_PageObjectHolder* pObjList,
+    CPDF_PageObjectHolder::const_iterator ObjPos) {
+  if (fabs(pTextObj->GetRect().Width()) < kSizeEpsilon)
     return;
 
   size_t count = m_LineObj.size();
@@ -774,27 +774,24 @@ void CPDF_TextPage::ProcessTextObject(
       GetCharWidth(item.m_CharCode, prev_Obj.m_pTextObj->GetFont()) *
       prev_Obj.m_pTextObj->GetFontSize() / 1000;
 
-  CFX_Matrix prev_matrix = prev_Obj.m_pTextObj->GetTextMatrix();
-  prev_width = fabs(prev_width);
-  prev_matrix.Concat(prev_Obj.m_formMatrix);
-  prev_width = prev_matrix.TransformDistance(prev_width);
+  CFX_Matrix prev_matrix =
+      prev_Obj.m_pTextObj->GetTextMatrix() * prev_Obj.m_formMatrix;
+  prev_width = prev_matrix.TransformDistance(fabs(prev_width));
   pTextObj->GetItemInfo(0, &item);
   float this_width = GetCharWidth(item.m_CharCode, pTextObj->GetFont()) *
                      pTextObj->GetFontSize() / 1000;
   this_width = fabs(this_width);
 
-  CFX_Matrix this_matrix = pTextObj->GetTextMatrix();
-  this_width = fabs(this_width);
-  this_matrix.Concat(formMatrix);
-  this_width = this_matrix.TransformDistance(this_width);
+  CFX_Matrix this_matrix = pTextObj->GetTextMatrix() * formMatrix;
+  this_width = this_matrix.TransformDistance(fabs(this_width));
 
-  float threshold = prev_width > this_width ? prev_width / 4 : this_width / 4;
+  float threshold = std::max(prev_width, this_width) / 4;
   CFX_PointF prev_pos = m_DisplayMatrix.Transform(
       prev_Obj.m_formMatrix.Transform(prev_Obj.m_pTextObj->GetPos()));
   CFX_PointF this_pos =
       m_DisplayMatrix.Transform(formMatrix.Transform(pTextObj->GetPos()));
   if (fabs(this_pos.y - prev_pos.y) > threshold * 2) {
-    for (size_t i = 0; i < count; i++)
+    for (size_t i = 0; i < count; ++i)
       ProcessTextObject(m_LineObj[i]);
     m_LineObj.clear();
     m_LineObj.push_back(Obj);
@@ -816,19 +813,16 @@ void CPDF_TextPage::ProcessTextObject(
 
 FPDFText_MarkedContent CPDF_TextPage::PreMarkedContent(PDFTEXT_Obj Obj) {
   CPDF_TextObject* pTextObj = Obj.m_pTextObj.Get();
-  if (!pTextObj->m_ContentMark.HasRef())
-    return FPDFText_MarkedContent::Pass;
-
-  size_t nContentMark = pTextObj->m_ContentMark.CountItems();
-  if (nContentMark == 0)
+  size_t nContentMarks = pTextObj->m_ContentMarks.CountItems();
+  if (nContentMarks == 0)
     return FPDFText_MarkedContent::Pass;
 
   WideString actText;
   bool bExist = false;
   const CPDF_Dictionary* pDict = nullptr;
-  for (size_t i = 0; i < nContentMark; ++i) {
-    const CPDF_ContentMarkItem& item = pTextObj->m_ContentMark.GetItem(i);
-    pDict = item.GetParam();
+  for (size_t i = 0; i < nContentMarks; ++i) {
+    const CPDF_ContentMarkItem* item = pTextObj->m_ContentMarks.GetItem(i);
+    pDict = item->GetParam();
     if (!pDict)
       continue;
     const CPDF_String* temp = ToString(pDict->GetObjectFor("ActualText"));
@@ -841,9 +835,9 @@ FPDFText_MarkedContent CPDF_TextPage::PreMarkedContent(PDFTEXT_Obj Obj) {
     return FPDFText_MarkedContent::Pass;
 
   if (m_pPreTextObj) {
-    const CPDF_ContentMark& mark = m_pPreTextObj->m_ContentMark;
-    if (mark.HasRef() && mark.CountItems() == nContentMark &&
-        mark.GetItem(nContentMark - 1).GetParam() == pDict) {
+    const CPDF_ContentMarks& marks = m_pPreTextObj->m_ContentMarks;
+    if (marks.CountItems() == nContentMarks &&
+        marks.GetItem(nContentMarks - 1)->GetParam() == pDict) {
       return FPDFText_MarkedContent::Done;
     }
   }
@@ -853,7 +847,7 @@ FPDFText_MarkedContent CPDF_TextPage::PreMarkedContent(PDFTEXT_Obj Obj) {
 
   CPDF_Font* pFont = pTextObj->GetFont();
   bExist = false;
-  for (size_t i = 0; i < actText.GetLength(); i++) {
+  for (size_t i = 0; i < actText.GetLength(); ++i) {
     if (pFont->CharCodeFromUnicode(actText[i]) != CPDF_Font::kInvalidCharCode) {
       bExist = true;
       break;
@@ -863,7 +857,7 @@ FPDFText_MarkedContent CPDF_TextPage::PreMarkedContent(PDFTEXT_Obj Obj) {
     return FPDFText_MarkedContent::Pass;
 
   bExist = false;
-  for (size_t i = 0; i < actText.GetLength(); i++) {
+  for (size_t i = 0; i < actText.GetLength(); ++i) {
     wchar_t wChar = actText[i];
     if ((wChar > 0x80 && wChar < 0xFFFD) || (wChar <= 0x80 && isprint(wChar))) {
       bExist = true;
@@ -878,17 +872,15 @@ FPDFText_MarkedContent CPDF_TextPage::PreMarkedContent(PDFTEXT_Obj Obj) {
 
 void CPDF_TextPage::ProcessMarkedContent(PDFTEXT_Obj Obj) {
   CPDF_TextObject* pTextObj = Obj.m_pTextObj.Get();
-  if (!pTextObj->m_ContentMark.HasRef())
-    return;
 
-  int nContentMark = pTextObj->m_ContentMark.CountItems();
-  if (nContentMark < 1)
+  size_t nContentMarks = pTextObj->m_ContentMarks.CountItems();
+  if (nContentMarks == 0)
     return;
 
   WideString actText;
-  for (int n = 0; n < nContentMark; n++) {
-    const CPDF_ContentMarkItem& item = pTextObj->m_ContentMark.GetItem(n);
-    const CPDF_Dictionary* pDict = item.GetParam();
+  for (size_t n = 0; n < nContentMarks; ++n) {
+    const CPDF_ContentMarkItem* item = pTextObj->m_ContentMarks.GetItem(n);
+    const CPDF_Dictionary* pDict = item->GetParam();
     if (pDict)
       actText = pDict->GetUnicodeTextFor("ActualText");
   }
@@ -896,10 +888,9 @@ void CPDF_TextPage::ProcessMarkedContent(PDFTEXT_Obj Obj) {
     return;
 
   CPDF_Font* pFont = pTextObj->GetFont();
-  CFX_Matrix matrix = pTextObj->GetTextMatrix();
-  matrix.Concat(Obj.m_formMatrix);
+  CFX_Matrix matrix = pTextObj->GetTextMatrix() * Obj.m_formMatrix;
 
-  for (size_t k = 0; k < actText.GetLength(); k++) {
+  for (size_t k = 0; k < actText.GetLength(); ++k) {
     wchar_t wChar = actText[k];
     if (wChar <= 0x80 && !isprint(wChar))
       wChar = 0x20;
@@ -933,25 +924,24 @@ void CPDF_TextPage::SwapTempTextBuf(int32_t iCharListStartAppend,
                                     int32_t iBufStartAppend) {
   int32_t i = iCharListStartAppend;
   int32_t j = pdfium::CollectionSize<int32_t>(m_TempCharList) - 1;
-  for (; i < j; i++, j--) {
+  for (; i < j; ++i, --j) {
     std::swap(m_TempCharList[i], m_TempCharList[j]);
     std::swap(m_TempCharList[i].m_Index, m_TempCharList[j].m_Index);
   }
   wchar_t* pTempBuffer = m_TempTextBuf.GetBuffer();
   i = iBufStartAppend;
   j = m_TempTextBuf.GetLength() - 1;
-  for (; i < j; i++, j--)
+  for (; i < j; ++i, --j)
     std::swap(pTempBuffer[i], pTempBuffer[j]);
 }
 
 void CPDF_TextPage::ProcessTextObject(PDFTEXT_Obj Obj) {
   CPDF_TextObject* pTextObj = Obj.m_pTextObj.Get();
-  if (fabs(pTextObj->m_Right - pTextObj->m_Left) < 0.01f)
+  if (fabs(pTextObj->GetRect().Width()) < kSizeEpsilon)
     return;
   CFX_Matrix formMatrix = Obj.m_formMatrix;
   CPDF_Font* pFont = pTextObj->GetFont();
-  CFX_Matrix matrix = pTextObj->GetTextMatrix();
-  matrix.Concat(formMatrix);
+  CFX_Matrix matrix = pTextObj->GetTextMatrix() * formMatrix;
 
   FPDFText_MarkedContent ePreMKC = PreMarkedContent(Obj);
   if (ePreMKC == FPDFText_MarkedContent::Done) {
@@ -997,7 +987,7 @@ void CPDF_TextPage::ProcessTextObject(PDFTEXT_Obj Obj) {
           if (wstrItem.IsEmpty())
             wstrItem += (wchar_t)item.m_CharCode;
           wchar_t curChar = wstrItem[0];
-          if (curChar == 0x2D || curChar == 0xAD)
+          if (IsHyphenCode(curChar))
             return;
         }
         while (m_TempTextBuf.GetSize() > 0 &&
@@ -1110,11 +1100,13 @@ void CPDF_TextPage::ProcessTextObject(PDFTEXT_Obj Obj) {
     charinfo.m_CharBox.left = rect.left * fFontSize + item.m_Origin.x;
     charinfo.m_CharBox.right = rect.right * fFontSize + item.m_Origin.x;
     charinfo.m_CharBox.bottom = rect.bottom * fFontSize + item.m_Origin.y;
-    if (fabsf(charinfo.m_CharBox.top - charinfo.m_CharBox.bottom) < 0.01f) {
+    if (fabsf(charinfo.m_CharBox.top - charinfo.m_CharBox.bottom) <
+        kSizeEpsilon) {
       charinfo.m_CharBox.top =
           charinfo.m_CharBox.bottom + pTextObj->GetFontSize();
     }
-    if (fabsf(charinfo.m_CharBox.right - charinfo.m_CharBox.left) < 0.01f) {
+    if (fabsf(charinfo.m_CharBox.right - charinfo.m_CharBox.left) <
+        kSizeEpsilon) {
       charinfo.m_CharBox.right =
           charinfo.m_CharBox.left + pTextObj->GetCharWidth(charinfo.m_CharCode);
     }
@@ -1132,7 +1124,7 @@ void CPDF_TextPage::ProcessTextObject(PDFTEXT_Obj Obj) {
     float threshold = charinfo.m_Matrix.TransformXDistance(
         static_cast<float>(TEXT_CHARRATIO_GAPDELTA) * pTextObj->GetFontSize());
     for (int n = pdfium::CollectionSize<int>(m_TempCharList);
-         n > pdfium::CollectionSize<int>(m_TempCharList) - count; n--) {
+         n > pdfium::CollectionSize<int>(m_TempCharList) - count; --n) {
       const PAGECHAR_INFO& charinfo1 = m_TempCharList[n - 1];
       CFX_PointF diff = charinfo1.m_Origin - charinfo.m_Origin;
       if (charinfo1.m_CharCode == charinfo.m_CharCode &&
@@ -1143,7 +1135,7 @@ void CPDF_TextPage::ProcessTextObject(PDFTEXT_Obj Obj) {
       }
     }
     if (!bDel) {
-      for (int nIndex = 0; nIndex < nTotal; nIndex++) {
+      for (int nIndex = 0; nIndex < nTotal; ++nIndex) {
         charinfo.m_Unicode = wstrItem[nIndex];
         if (charinfo.m_Unicode) {
           charinfo.m_Index = m_TextBuf.GetLength();
@@ -1203,7 +1195,7 @@ bool CPDF_TextPage::IsHyphen(wchar_t curChar) const {
     return false;
 
   auto iter = curText.rbegin();
-  for (; (iter + 1) != curText.rend() && *iter == 0x20; iter++) {
+  for (; (iter + 1) != curText.rend() && *iter == 0x20; ++iter) {
     // Do nothing
   }
 
@@ -1245,34 +1237,23 @@ CPDF_TextPage::GenerateCharacter CPDF_TextPage::ProcessInsertObject(
   CPDF_TextObjectItem item;
   pObj->GetItemInfo(0, &item);
 
-  CFX_FloatRect this_rect = pObj->GetRect();
-  CFX_FloatRect prev_rect = m_pPreTextObj->GetRect();
+  const CFX_FloatRect& this_rect = pObj->GetRect();
+  const CFX_FloatRect& prev_rect = m_pPreTextObj->GetRect();
 
   WideString wstrItem = pObj->GetFont()->UnicodeFromCharCode(item.m_CharCode);
   if (wstrItem.IsEmpty())
     wstrItem += static_cast<wchar_t>(item.m_CharCode);
   wchar_t curChar = wstrItem[0];
   if (WritingMode == TextOrientation::Horizontal) {
-    if (this_rect.Height() > 4.5 && prev_rect.Height() > 4.5) {
-      float top = this_rect.top < prev_rect.top ? this_rect.top : prev_rect.top;
-      float bottom = this_rect.bottom > prev_rect.bottom ? this_rect.bottom
-                                                         : prev_rect.bottom;
-      if (bottom >= top) {
-        return IsHyphen(curChar) ? GenerateCharacter::Hyphen
-                                 : GenerateCharacter::LineBreak;
-      }
+    if (EndHorizontalLine(this_rect, prev_rect)) {
+      return IsHyphen(curChar) ? GenerateCharacter::Hyphen
+                               : GenerateCharacter::LineBreak;
     }
   } else if (WritingMode == TextOrientation::Vertical) {
-    if (this_rect.Width() > pObj->GetFontSize() * 0.1f &&
-        prev_rect.Width() > m_pPreTextObj->GetFontSize() * 0.1f) {
-      float left = this_rect.left > m_CurlineRect.left ? this_rect.left
-                                                       : m_CurlineRect.left;
-      float right = this_rect.right < m_CurlineRect.right ? this_rect.right
-                                                          : m_CurlineRect.right;
-      if (right <= left) {
-        return IsHyphen(curChar) ? GenerateCharacter::Hyphen
-                                 : GenerateCharacter::LineBreak;
-      }
+    if (EndVerticalLine(this_rect, prev_rect, m_CurlineRect,
+                        pObj->GetFontSize(), m_pPreTextObj->GetFontSize())) {
+      return IsHyphen(curChar) ? GenerateCharacter::Hyphen
+                               : GenerateCharacter::LineBreak;
     }
   }
 
@@ -1282,13 +1263,10 @@ CPDF_TextPage::GenerateCharacter CPDF_TextPage::ProcessInsertObject(
   float last_width = nLastWidth * m_pPreTextObj->GetFontSize() / 1000;
   last_width = fabs(last_width);
   uint32_t nThisWidth = GetCharWidth(item.m_CharCode, pObj->GetFont());
-  float this_width = nThisWidth * pObj->GetFontSize() / 1000;
-  this_width = fabs(this_width);
-  float threshold = last_width > this_width ? last_width / 4 : this_width / 4;
-
-  CFX_Matrix prev_matrix = m_pPreTextObj->GetTextMatrix();
-  prev_matrix.Concat(m_perMatrix);
+  float this_width = fabs(nThisWidth * pObj->GetFontSize() / 1000);
+  float threshold = std::max(last_width, this_width) / 4;
 
+  CFX_Matrix prev_matrix = m_pPreTextObj->GetTextMatrix() * m_perMatrix;
   CFX_Matrix prev_reverse = prev_matrix.GetInverse();
 
   CFX_PointF pos = prev_reverse.Transform(formMatrix.Transform(pObj->GetPos()));
@@ -1297,14 +1275,12 @@ CPDF_TextPage::GenerateCharacter CPDF_TextPage::ProcessInsertObject(
 
   bool bNewline = false;
   if (WritingMode == TextOrientation::Horizontal) {
-    CFX_FloatRect rect1(m_pPreTextObj->m_Left, pObj->m_Bottom,
-                        m_pPreTextObj->m_Right, pObj->m_Top);
-    CFX_FloatRect rect2 = m_pPreTextObj->GetRect();
-    CFX_FloatRect rect3 = rect1;
-    rect1.Intersect(rect2);
-    if ((rect1.IsEmpty() && rect2.Height() > 5 && rect3.Height() > 5) ||
+    CFX_FloatRect rect = m_pPreTextObj->GetRect();
+    float rect_height = rect.Height();
+    rect.Normalize();
+    if ((rect.IsEmpty() && rect_height > 5) ||
         ((pos.y > threshold * 2 || pos.y < threshold * -3) &&
-         (fabs(pos.y) < 1 ? fabs(pos.x) < fabs(pos.y) : true))) {
+         (fabs(pos.y) >= 1 || fabs(pos.y) > fabs(pos.x)))) {
       bNewline = true;
       if (nItem > 1) {
         CPDF_TextObjectItem tempItem;
@@ -1314,14 +1290,16 @@ CPDF_TextPage::GenerateCharacter CPDF_TextPage::ProcessInsertObject(
             m_DisplayMatrix.a > 0.9 && m_DisplayMatrix.b < 0.1 &&
             m_DisplayMatrix.c < 0.1 && m_DisplayMatrix.d < -0.9 && m.b < 0.1 &&
             m.c < 0.1) {
-          CFX_FloatRect re(0, m_pPreTextObj->m_Bottom, 1000,
-                           m_pPreTextObj->m_Top);
+          CFX_FloatRect re(0, m_pPreTextObj->GetRect().bottom, 1000,
+                           m_pPreTextObj->GetRect().top);
           if (re.Contains(pObj->GetPos())) {
             bNewline = false;
           } else {
-            CFX_FloatRect rect(0, pObj->m_Bottom, 1000, pObj->m_Top);
-            if (rect.Contains(m_pPreTextObj->GetPos()))
+            if (CFX_FloatRect(0, pObj->GetRect().bottom, 1000,
+                              pObj->GetRect().top)
+                    .Contains(m_pPreTextObj->GetPos())) {
               bNewline = false;
+            }
           }
         }
       }
@@ -1332,33 +1310,34 @@ CPDF_TextPage::GenerateCharacter CPDF_TextPage::ProcessInsertObject(
                              : GenerateCharacter::LineBreak;
   }
 
-  if (pObj->CountChars() == 1 && (0x2D == curChar || 0xAD == curChar) &&
-      IsHyphen(curChar)) {
+  if (pObj->CountChars() == 1 && IsHyphenCode(curChar) && IsHyphen(curChar))
     return GenerateCharacter::Hyphen;
-  }
+
+  if (curChar == L' ')
+    return GenerateCharacter::None;
+
   WideString PrevStr =
       m_pPreTextObj->GetFont()->UnicodeFromCharCode(PrevItem.m_CharCode);
   wchar_t preChar = PrevStr.Last();
-  CFX_Matrix matrix = pObj->GetTextMatrix();
-  matrix.Concat(formMatrix);
-
-  threshold = static_cast<float>(std::max(nLastWidth, nThisWidth));
-  threshold = NormalizeThreshold(threshold, 400, 700, 800);
-  if (nLastWidth >= nThisWidth) {
-    threshold *= fabs(m_pPreTextObj->GetFontSize());
-  } else {
-    threshold *= fabs(pObj->GetFontSize());
-    threshold = matrix.TransformDistance(threshold);
-    threshold = prev_reverse.TransformDistance(threshold);
-  }
-  threshold /= 1000;
-  if ((threshold < 1.4881 && threshold > 1.4879) ||
-      (threshold < 1.39001 && threshold > 1.38999)) {
-    threshold *= 1.5;
-  }
-  if (curChar == L' ' || preChar == L' ')
+  if (preChar == L' ')
     return GenerateCharacter::None;
-  return GenerateSpace(pos, last_pos, this_width, last_width, threshold)
+
+  CFX_Matrix matrix = pObj->GetTextMatrix() * formMatrix;
+  float threshold2 = static_cast<float>(std::max(nLastWidth, nThisWidth));
+  threshold2 = NormalizeThreshold(threshold2, 400, 700, 800);
+  if (nLastWidth >= nThisWidth) {
+    threshold2 *= fabs(m_pPreTextObj->GetFontSize());
+  } else {
+    threshold2 *= fabs(pObj->GetFontSize());
+    threshold2 = matrix.TransformDistance(threshold2);
+    threshold2 = prev_reverse.TransformDistance(threshold2);
+  }
+  threshold2 /= 1000;
+  if ((threshold2 < 1.4881 && threshold2 > 1.4879) ||
+      (threshold2 < 1.39001 && threshold2 > 1.38999)) {
+    threshold2 *= 1.5;
+  }
+  return GenerateSpace(pos, last_pos, this_width, last_width, threshold2)
              ? GenerateCharacter::Space
              : GenerateCharacter::None;
 }
@@ -1369,7 +1348,7 @@ bool CPDF_TextPage::IsSameTextObject(CPDF_TextObject* pTextObj1,
     return false;
 
   CFX_FloatRect rcPreObj = pTextObj2->GetRect();
-  CFX_FloatRect rcCurObj = pTextObj1->GetRect();
+  const CFX_FloatRect& rcCurObj = pTextObj1->GetRect();
   if (rcPreObj.IsEmpty() && rcCurObj.IsEmpty()) {
     float dbXdif = fabs(rcPreObj.left - rcCurObj.left);
     size_t nCount = m_CharList.size();
@@ -1419,8 +1398,8 @@ bool CPDF_TextPage::IsSameTextObject(CPDF_TextObject* pTextObj1,
 
 bool CPDF_TextPage::IsSameAsPreTextObject(
     CPDF_TextObject* pTextObj,
-    const CPDF_PageObjectList* pObjList,
-    CPDF_PageObjectList::const_iterator iter) {
+    const CPDF_PageObjectHolder* pObjList,
+    CPDF_PageObjectHolder::const_iterator iter) {
   int i = 0;
   while (i < 5 && iter != pObjList->begin()) {
     --iter;
