@@ -9,6 +9,7 @@
 #include <limits>
 #include <memory>
 
+#include "core/fxcodec/codec/cfx_codec_memory.h"
 #include "core/fxcodec/codec/codec_int.h"
 #include "core/fxcodec/fx_codec.h"
 #include "core/fxcrt/fx_safe_types.h"
@@ -32,7 +33,7 @@ struct TiffDeleter {
 
 }  // namespace
 
-class CTiffContext final : public CCodec_TiffModule::Context {
+class CTiffContext final : public CodecModuleIface::Context {
  public:
   CTiffContext() = default;
   ~CTiffContext() override = default;
@@ -120,7 +121,7 @@ tsize_t tiff_read(thandle_t context, tdata_t buf, tsize_t length) {
     return 0;
 
   FX_FILESIZE offset = pTiffContext->offset();
-  if (!pTiffContext->io_in()->ReadBlock(buf, offset, length))
+  if (!pTiffContext->io_in()->ReadBlockAtOffset(buf, offset, length))
     return 0;
 
   pTiffContext->set_offset(increment.ValueOrDie());
@@ -257,31 +258,29 @@ bool CTiffContext::LoadFrameInfo(int32_t frame,
   TIFFGetField(m_tif_ctx.get(), TIFFTAG_BITSPERSAMPLE, &tif_bpc);
   TIFFGetField(m_tif_ctx.get(), TIFFTAG_ROWSPERSTRIP, &tif_rps);
 
-  if (pAttribute) {
-    pAttribute->m_wDPIUnit = FXCODEC_RESUNIT_INCH;
-    if (TIFFGetField(m_tif_ctx.get(), TIFFTAG_RESOLUTIONUNIT,
-                     &pAttribute->m_wDPIUnit)) {
-      pAttribute->m_wDPIUnit--;
-    }
-    Tiff_Exif_GetInfo<uint16_t>(m_tif_ctx.get(), TIFFTAG_ORIENTATION,
-                                pAttribute);
-    if (Tiff_Exif_GetInfo<float>(m_tif_ctx.get(), TIFFTAG_XRESOLUTION,
-                                 pAttribute)) {
-      void* val = pAttribute->m_Exif[TIFFTAG_XRESOLUTION];
-      float fDpi = val ? *reinterpret_cast<float*>(val) : 0;
-      pAttribute->m_nXDPI = (int32_t)(fDpi + 0.5f);
-    }
-    if (Tiff_Exif_GetInfo<float>(m_tif_ctx.get(), TIFFTAG_YRESOLUTION,
-                                 pAttribute)) {
-      void* val = pAttribute->m_Exif[TIFFTAG_YRESOLUTION];
-      float fDpi = val ? *reinterpret_cast<float*>(val) : 0;
-      pAttribute->m_nYDPI = (int32_t)(fDpi + 0.5f);
-    }
-    Tiff_Exif_GetStringInfo(m_tif_ctx.get(), TIFFTAG_IMAGEDESCRIPTION,
-                            pAttribute);
-    Tiff_Exif_GetStringInfo(m_tif_ctx.get(), TIFFTAG_MAKE, pAttribute);
-    Tiff_Exif_GetStringInfo(m_tif_ctx.get(), TIFFTAG_MODEL, pAttribute);
+  pAttribute->m_wDPIUnit = FXCODEC_RESUNIT_INCH;
+  if (TIFFGetField(m_tif_ctx.get(), TIFFTAG_RESOLUTIONUNIT,
+                   &pAttribute->m_wDPIUnit)) {
+    pAttribute->m_wDPIUnit--;
   }
+  Tiff_Exif_GetInfo<uint16_t>(m_tif_ctx.get(), TIFFTAG_ORIENTATION, pAttribute);
+  if (Tiff_Exif_GetInfo<float>(m_tif_ctx.get(), TIFFTAG_XRESOLUTION,
+                               pAttribute)) {
+    void* val = pAttribute->m_Exif[TIFFTAG_XRESOLUTION];
+    float fDpi = val ? *reinterpret_cast<float*>(val) : 0;
+    pAttribute->m_nXDPI = (int32_t)(fDpi + 0.5f);
+  }
+  if (Tiff_Exif_GetInfo<float>(m_tif_ctx.get(), TIFFTAG_YRESOLUTION,
+                               pAttribute)) {
+    void* val = pAttribute->m_Exif[TIFFTAG_YRESOLUTION];
+    float fDpi = val ? *reinterpret_cast<float*>(val) : 0;
+    pAttribute->m_nYDPI = (int32_t)(fDpi + 0.5f);
+  }
+  Tiff_Exif_GetStringInfo(m_tif_ctx.get(), TIFFTAG_IMAGEDESCRIPTION,
+                          pAttribute);
+  Tiff_Exif_GetStringInfo(m_tif_ctx.get(), TIFFTAG_MAKE, pAttribute);
+  Tiff_Exif_GetStringInfo(m_tif_ctx.get(), TIFFTAG_MODEL, pAttribute);
+
   pdfium::base::CheckedNumeric<int32_t> checked_width = tif_width;
   pdfium::base::CheckedNumeric<int32_t> checked_height = tif_height;
   if (!checked_width.IsValid() || !checked_height.IsValid())
@@ -487,7 +486,7 @@ bool CTiffContext::Decode(const RetainPtr<CFX_DIBitmap>& pDIBitmap) {
   return false;
 }
 
-std::unique_ptr<CCodec_TiffModule::Context> CCodec_TiffModule::CreateDecoder(
+std::unique_ptr<CodecModuleIface::Context> CCodec_TiffModule::CreateDecoder(
     const RetainPtr<IFX_SeekableReadStream>& file_ptr) {
   auto pDecoder = pdfium::MakeUnique<CTiffContext>();
   if (!pDecoder->InitDecoder(file_ptr))
@@ -496,6 +495,18 @@ std::unique_ptr<CCodec_TiffModule::Context> CCodec_TiffModule::CreateDecoder(
   return pDecoder;
 }
 
+FX_FILESIZE CCodec_TiffModule::GetAvailInput(Context* pContext) const {
+  NOTREACHED();
+  return 0;
+}
+
+bool CCodec_TiffModule::Input(Context* pContext,
+                              RetainPtr<CFX_CodecMemory> codec_memory,
+                              CFX_DIBAttribute*) {
+  NOTREACHED();
+  return false;
+}
+
 bool CCodec_TiffModule::LoadFrameInfo(Context* pContext,
                                       int32_t frame,
                                       int32_t* width,
@@ -503,6 +514,8 @@ bool CCodec_TiffModule::LoadFrameInfo(Context* pContext,
                                       int32_t* comps,
                                       int32_t* bpc,
                                       CFX_DIBAttribute* pAttribute) {
+  ASSERT(pAttribute);
+
   auto* ctx = static_cast<CTiffContext*>(pContext);
   return ctx->LoadFrameInfo(frame, width, height, comps, bpc, pAttribute);
 }
