@@ -11,6 +11,7 @@
 #include <algorithm>
 #include <cctype>
 #include <string>
+#include <utility>
 
 #include "core/fxcrt/cfx_utf8decoder.h"
 #include "core/fxcrt/fx_codepage.h"
@@ -18,6 +19,7 @@
 #include "core/fxcrt/fx_safe_types.h"
 #include "core/fxcrt/string_pool_template.h"
 #include "third_party/base/numerics/safe_math.h"
+#include "third_party/base/span.h"
 #include "third_party/base/stl_util.h"
 
 template class fxcrt::StringDataTemplate<char>;
@@ -27,6 +29,8 @@ template struct std::hash<ByteString>;
 
 namespace {
 
+constexpr char kTrimChars[] = "\x09\x0a\x0b\x0c\x0d\x20";
+
 const char* FX_strstr(const char* haystack,
                       int haystack_len,
                       const char* needle,
@@ -51,39 +55,6 @@ const char* FX_strstr(const char* haystack,
   return nullptr;
 }
 
-#ifndef NDEBUG
-bool IsValidCodePage(uint16_t codepage) {
-  switch (codepage) {
-    case FX_CODEPAGE_DefANSI:
-    case FX_CODEPAGE_ShiftJIS:
-    case FX_CODEPAGE_ChineseSimplified:
-    case FX_CODEPAGE_Hangul:
-    case FX_CODEPAGE_ChineseTraditional:
-      return true;
-    default:
-      return false;
-  }
-}
-#endif
-
-ByteString GetByteString(uint16_t codepage, const WideStringView& wstr) {
-  ASSERT(IsValidCodePage(codepage));
-
-  int src_len = wstr.GetLength();
-  int dest_len =
-      FXSYS_WideCharToMultiByte(codepage, 0, wstr.unterminated_c_str(), src_len,
-                                nullptr, 0, nullptr, nullptr);
-  if (!dest_len)
-    return ByteString();
-
-  ByteString bstr;
-  char* dest_buf = bstr.GetBuffer(dest_len);
-  FXSYS_WideCharToMultiByte(codepage, 0, wstr.unterminated_c_str(), src_len,
-                            dest_buf, dest_len, nullptr, nullptr);
-  bstr.ReleaseBuffer(dest_len);
-  return bstr;
-}
-
 }  // namespace
 
 namespace fxcrt {
@@ -91,6 +62,59 @@ namespace fxcrt {
 static_assert(sizeof(ByteString) <= sizeof(char*),
               "Strings must not require more space than pointers");
 
+#define FORCE_ANSI 0x10000
+#define FORCE_UNICODE 0x20000
+#define FORCE_INT64 0x40000
+
+// static
+ByteString ByteString::FormatInteger(int i) {
+  char buf[32];
+  FXSYS_snprintf(buf, sizeof(buf), "%d", i);
+  return ByteString(buf);
+}
+
+// static
+ByteString ByteString::FormatFloat(float d) {
+  char buf[32];
+  return ByteString(buf, FloatToString(d, buf));
+}
+
+// static
+ByteString ByteString::FormatV(const char* pFormat, va_list argList) {
+  va_list argListCopy;
+  va_copy(argListCopy, argList);
+  int nMaxLen = vsnprintf(nullptr, 0, pFormat, argListCopy);
+  va_end(argListCopy);
+
+  if (nMaxLen <= 0)
+    return ByteString();
+
+  ByteString ret;
+  {
+    // Span's lifetime must end before ReleaseBuffer() below.
+    pdfium::span<char> buf = ret.GetBuffer(nMaxLen);
+
+    // In the following two calls, there's always space in the buffer for
+    // a terminating NUL that's not included in nMaxLen.
+    memset(buf.data(), 0, nMaxLen + 1);
+    va_copy(argListCopy, argList);
+    vsnprintf(buf.data(), nMaxLen + 1, pFormat, argListCopy);
+    va_end(argListCopy);
+  }
+  ret.ReleaseBuffer(ret.GetStringLength());
+  return ret;
+}
+
+// static
+ByteString ByteString::Format(const char* pFormat, ...) {
+  va_list argList;
+  va_start(argList, pFormat);
+  ByteString ret = FormatV(pFormat, argList);
+  va_end(argList);
+
+  return ret;
+}
+
 ByteString::ByteString(const char* pStr, size_t nLen) {
   if (nLen)
     m_pData.Reset(StringData::Create(pStr, nLen));
@@ -118,13 +142,13 @@ ByteString::ByteString(char ch) {
 ByteString::ByteString(const char* ptr)
     : ByteString(ptr, ptr ? strlen(ptr) : 0) {}
 
-ByteString::ByteString(const ByteStringView& stringSrc) {
+ByteString::ByteString(ByteStringView stringSrc) {
   if (!stringSrc.IsEmpty())
     m_pData.Reset(StringData::Create(stringSrc.unterminated_c_str(),
                                      stringSrc.GetLength()));
 }
 
-ByteString::ByteString(const ByteStringView& str1, const ByteStringView& str2) {
+ByteString::ByteString(ByteStringView str1, ByteStringView str2) {
   FX_SAFE_SIZE_T nSafeLen = str1.GetLength();
   nSafeLen += str2.GetLength();
 
@@ -174,7 +198,7 @@ const ByteString& ByteString::operator=(const char* pStr) {
   return *this;
 }
 
-const ByteString& ByteString::operator=(const ByteStringView& stringSrc) {
+const ByteString& ByteString::operator=(ByteStringView stringSrc) {
   if (stringSrc.IsEmpty())
     clear();
   else
@@ -183,9 +207,16 @@ const ByteString& ByteString::operator=(const ByteStringView& stringSrc) {
   return *this;
 }
 
-const ByteString& ByteString::operator=(const ByteString& stringSrc) {
-  if (m_pData != stringSrc.m_pData)
-    m_pData = stringSrc.m_pData;
+const ByteString& ByteString::operator=(const ByteString& that) {
+  if (m_pData != that.m_pData)
+    m_pData = that.m_pData;
+
+  return *this;
+}
+
+const ByteString& ByteString::operator=(ByteString&& that) {
+  if (m_pData != that.m_pData)
+    m_pData = std::move(that.m_pData);
 
   return *this;
 }
@@ -209,7 +240,7 @@ const ByteString& ByteString::operator+=(const ByteString& str) {
   return *this;
 }
 
-const ByteString& ByteString::operator+=(const ByteStringView& str) {
+const ByteString& ByteString::operator+=(ByteStringView str) {
   if (!str.IsEmpty())
     Concat(str.unterminated_c_str(), str.GetLength());
 
@@ -227,7 +258,7 @@ bool ByteString::operator==(const char* ptr) const {
          memcmp(ptr, m_pData->m_String, m_pData->m_nDataLength) == 0;
 }
 
-bool ByteString::operator==(const ByteStringView& str) const {
+bool ByteString::operator==(ByteStringView str) const {
   if (!m_pData)
     return str.IsEmpty();
 
@@ -251,16 +282,33 @@ bool ByteString::operator==(const ByteString& other) const {
                 m_pData->m_nDataLength) == 0;
 }
 
-bool ByteString::operator<(const ByteString& str) const {
-  if (m_pData == str.m_pData)
+bool ByteString::operator<(const char* ptr) const {
+  if (!m_pData && !ptr)
+    return false;
+  if (c_str() == ptr)
     return false;
 
-  int result =
-      memcmp(c_str(), str.c_str(), std::min(GetLength(), str.GetLength()));
-  return result < 0 || (result == 0 && GetLength() < str.GetLength());
+  size_t len = GetLength();
+  size_t other_len = ptr ? strlen(ptr) : 0;
+  int result = memcmp(c_str(), ptr, std::min(len, other_len));
+  return result < 0 || (result == 0 && len < other_len);
 }
 
-bool ByteString::EqualNoCase(const ByteStringView& str) const {
+bool ByteString::operator<(ByteStringView str) const {
+  return Compare(str) < 0;
+}
+
+bool ByteString::operator<(const ByteString& other) const {
+  if (m_pData == other.m_pData)
+    return false;
+
+  size_t len = GetLength();
+  size_t other_len = other.GetLength();
+  int result = memcmp(c_str(), other.c_str(), std::min(len, other_len));
+  return result < 0 || (result == 0 && len < other_len);
+}
+
+bool ByteString::EqualNoCase(ByteStringView str) const {
   if (!m_pData)
     return str.IsEmpty();
 
@@ -272,8 +320,8 @@ bool ByteString::EqualNoCase(const ByteStringView& str) const {
   const uint8_t* pThat = str.raw_str();
   for (size_t i = 0; i < len; i++) {
     if ((*pThis) != (*pThat)) {
-      uint8_t bThis = FXSYS_tolower(*pThis);
-      uint8_t bThat = FXSYS_tolower(*pThat);
+      uint8_t bThis = tolower(*pThis);
+      uint8_t bThat = tolower(*pThat);
       if (bThis != bThat)
         return false;
     }
@@ -347,29 +395,29 @@ void ByteString::Reserve(size_t len) {
   GetBuffer(len);
 }
 
-char* ByteString::GetBuffer(size_t nMinBufLength) {
+pdfium::span<char> ByteString::GetBuffer(size_t nMinBufLength) {
   if (!m_pData) {
     if (nMinBufLength == 0)
-      return nullptr;
+      return pdfium::span<char>();
 
     m_pData.Reset(StringData::Create(nMinBufLength));
     m_pData->m_nDataLength = 0;
     m_pData->m_String[0] = 0;
-    return m_pData->m_String;
+    return pdfium::span<char>(m_pData->m_String, m_pData->m_nAllocLength);
   }
 
   if (m_pData->CanOperateInPlace(nMinBufLength))
-    return m_pData->m_String;
+    return pdfium::span<char>(m_pData->m_String, m_pData->m_nAllocLength);
 
   nMinBufLength = std::max(nMinBufLength, m_pData->m_nDataLength);
   if (nMinBufLength == 0)
-    return nullptr;
+    return pdfium::span<char>();
 
   RetainPtr<StringData> pNewData(StringData::Create(nMinBufLength));
   pNewData->CopyContents(*m_pData);
   pNewData->m_nDataLength = m_pData->m_nDataLength;
   m_pData.Swap(pNewData);
-  return m_pData->m_String;
+  return pdfium::span<char>(m_pData->m_String, m_pData->m_nAllocLength);
 }
 
 size_t ByteString::Delete(size_t index, size_t count) {
@@ -377,8 +425,7 @@ size_t ByteString::Delete(size_t index, size_t count) {
     return 0;
 
   size_t old_length = m_pData->m_nDataLength;
-  if (count == 0 ||
-      index != pdfium::clamp(index, static_cast<size_t>(0), old_length))
+  if (count == 0 || index != pdfium::clamp<size_t>(index, 0, old_length))
     return old_length;
 
   size_t removal_length = index + count;
@@ -408,13 +455,19 @@ void ByteString::Concat(const char* pSrcData, size_t nSrcLen) {
     return;
   }
 
+  size_t nConcatLen = std::max(m_pData->m_nDataLength / 2, nSrcLen);
   RetainPtr<StringData> pNewData(
-      StringData::Create(m_pData->m_nDataLength + nSrcLen));
+      StringData::Create(m_pData->m_nDataLength + nConcatLen));
   pNewData->CopyContents(*m_pData);
   pNewData->CopyContentsAt(m_pData->m_nDataLength, pSrcData, nSrcLen);
+  pNewData->m_nDataLength = m_pData->m_nDataLength + nSrcLen;
   m_pData.Swap(pNewData);
 }
 
+intptr_t ByteString::ReferenceCountForTesting() const {
+  return m_pData ? m_pData->m_nRefs : 0;
+}
+
 ByteString ByteString::Mid(size_t first, size_t count) const {
   if (!m_pData)
     return ByteString();
@@ -459,42 +512,6 @@ void ByteString::AllocCopy(ByteString& dest,
   dest.m_pData.Swap(pNewData);
 }
 
-#define FORCE_ANSI 0x10000
-#define FORCE_UNICODE 0x20000
-#define FORCE_INT64 0x40000
-
-ByteString ByteString::FormatInteger(int i) {
-  char buf[32];
-  FXSYS_snprintf(buf, 32, "%d", i);
-  return ByteString(buf);
-}
-
-void ByteString::FormatV(const char* pFormat, va_list argList) {
-  va_list argListCopy;
-  va_copy(argListCopy, argList);
-  int nMaxLen = vsnprintf(nullptr, 0, pFormat, argListCopy);
-  va_end(argListCopy);
-  if (nMaxLen > 0) {
-    GetBuffer(nMaxLen);
-    if (m_pData) {
-      // In the following two calls, there's always space in the buffer for
-      // a terminating NUL that's not included in nMaxLen.
-      memset(m_pData->m_String, 0, nMaxLen + 1);
-      va_copy(argListCopy, argList);
-      vsnprintf(m_pData->m_String, nMaxLen + 1, pFormat, argListCopy);
-      va_end(argListCopy);
-      ReleaseBuffer(GetStringLength());
-    }
-  }
-}
-
-void ByteString::Format(const char* pFormat, ...) {
-  va_list argList;
-  va_start(argList, pFormat);
-  FormatV(pFormat, argList);
-  va_end(argList);
-}
-
 void ByteString::SetAt(size_t index, char c) {
   ASSERT(IsValidIndex(index));
   ReallocBeforeWrite(m_pData->m_nDataLength);
@@ -502,7 +519,7 @@ void ByteString::SetAt(size_t index, char c) {
 }
 
 size_t ByteString::Insert(size_t location, char ch) {
-  const size_t cur_length = m_pData ? m_pData->m_nDataLength : 0;
+  const size_t cur_length = GetLength();
   if (!IsValidLength(location))
     return cur_length;
 
@@ -515,46 +532,43 @@ size_t ByteString::Insert(size_t location, char ch) {
   return new_length;
 }
 
-pdfium::Optional<size_t> ByteString::Find(char ch, size_t start) const {
+Optional<size_t> ByteString::Find(char ch, size_t start) const {
   if (!m_pData)
-    return pdfium::Optional<size_t>();
+    return Optional<size_t>();
 
   if (!IsValidIndex(start))
-    return pdfium::Optional<size_t>();
+    return Optional<size_t>();
 
   const char* pStr = static_cast<const char*>(
       memchr(m_pData->m_String + start, ch, m_pData->m_nDataLength - start));
-  return pStr ? pdfium::Optional<size_t>(
-                    static_cast<size_t>(pStr - m_pData->m_String))
-              : pdfium::Optional<size_t>();
+  return pStr ? Optional<size_t>(static_cast<size_t>(pStr - m_pData->m_String))
+              : Optional<size_t>();
 }
 
-pdfium::Optional<size_t> ByteString::Find(const ByteStringView& subStr,
-                                          size_t start) const {
+Optional<size_t> ByteString::Find(ByteStringView subStr, size_t start) const {
   if (!m_pData)
-    return pdfium::Optional<size_t>();
+    return Optional<size_t>();
 
   if (!IsValidIndex(start))
-    return pdfium::Optional<size_t>();
+    return Optional<size_t>();
 
   const char* pStr =
       FX_strstr(m_pData->m_String + start, m_pData->m_nDataLength - start,
                 subStr.unterminated_c_str(), subStr.GetLength());
-  return pStr ? pdfium::Optional<size_t>(
-                    static_cast<size_t>(pStr - m_pData->m_String))
-              : pdfium::Optional<size_t>();
+  return pStr ? Optional<size_t>(static_cast<size_t>(pStr - m_pData->m_String))
+              : Optional<size_t>();
 }
 
-pdfium::Optional<size_t> ByteString::ReverseFind(char ch) const {
+Optional<size_t> ByteString::ReverseFind(char ch) const {
   if (!m_pData)
-    return pdfium::Optional<size_t>();
+    return Optional<size_t>();
 
   size_t nLength = m_pData->m_nDataLength;
   while (nLength--) {
     if (m_pData->m_String[nLength] == ch)
-      return pdfium::Optional<size_t>(nLength);
+      return Optional<size_t>(nLength);
   }
-  return pdfium::Optional<size_t>();
+  return Optional<size_t>();
 }
 
 void ByteString::MakeLower() {
@@ -574,7 +588,7 @@ void ByteString::MakeUpper() {
 }
 
 size_t ByteString::Remove(char chRemove) {
-  if (!m_pData || m_pData->m_nDataLength < 1)
+  if (!m_pData || m_pData->m_nDataLength == 0)
     return 0;
 
   char* pstrSource = m_pData->m_String;
@@ -607,8 +621,7 @@ size_t ByteString::Remove(char chRemove) {
   return nCount;
 }
 
-size_t ByteString::Replace(const ByteStringView& pOld,
-                           const ByteStringView& pNew) {
+size_t ByteString::Replace(ByteStringView pOld, ByteStringView pNew) {
   if (!m_pData || pOld.IsEmpty())
     return 0;
 
@@ -654,79 +667,47 @@ size_t ByteString::Replace(const ByteStringView& pOld,
   return nCount;
 }
 
-WideString ByteString::UTF8Decode() const {
-  CFX_UTF8Decoder decoder;
-  for (size_t i = 0; i < GetLength(); i++) {
-    decoder.Input(static_cast<uint8_t>(m_pData->m_String[i]));
-  }
-  return WideString(decoder.GetResult());
-}
-
-// static
-ByteString ByteString::FromUnicode(const WideString& str) {
-  return GetByteString(0, str.AsStringView());
-}
-
-int ByteString::Compare(const ByteStringView& str) const {
-  if (!m_pData) {
+int ByteString::Compare(ByteStringView str) const {
+  if (!m_pData)
     return str.IsEmpty() ? 0 : -1;
-  }
+
   size_t this_len = m_pData->m_nDataLength;
   size_t that_len = str.GetLength();
   size_t min_len = std::min(this_len, that_len);
-  for (size_t i = 0; i < min_len; i++) {
-    if (static_cast<uint8_t>(m_pData->m_String[i]) < str[i]) {
-      return -1;
-    }
-    if (static_cast<uint8_t>(m_pData->m_String[i]) > str[i]) {
-      return 1;
-    }
-  }
-  if (this_len < that_len) {
-    return -1;
-  }
-  if (this_len > that_len) {
-    return 1;
-  }
-  return 0;
+  int result = memcmp(m_pData->m_String, str.unterminated_c_str(), min_len);
+  if (result != 0)
+    return result;
+  if (this_len == that_len)
+    return 0;
+  return this_len < that_len ? -1 : 1;
 }
 
-void ByteString::TrimRight(const ByteStringView& pTargets) {
-  if (!m_pData || pTargets.IsEmpty())
-    return;
-
-  size_t pos = GetLength();
-  if (pos == 0)
-    return;
-
-  while (pos) {
-    size_t i = 0;
-    while (i < pTargets.GetLength() &&
-           pTargets[i] != m_pData->m_String[pos - 1]) {
-      i++;
-    }
-    if (i == pTargets.GetLength()) {
-      break;
-    }
-    pos--;
-  }
-  if (pos < m_pData->m_nDataLength) {
-    ReallocBeforeWrite(m_pData->m_nDataLength);
-    m_pData->m_String[pos] = 0;
-    m_pData->m_nDataLength = pos;
-  }
+void ByteString::Trim() {
+  TrimRight(kTrimChars);
+  TrimLeft(kTrimChars);
 }
 
-void ByteString::TrimRight(char chTarget) {
-  TrimRight(ByteStringView(chTarget));
+void ByteString::Trim(char target) {
+  ByteStringView targets(target);
+  TrimRight(targets);
+  TrimLeft(targets);
 }
 
-void ByteString::TrimRight() {
-  TrimRight("\x09\x0a\x0b\x0c\x0d\x20");
+void ByteString::Trim(ByteStringView targets) {
+  TrimRight(targets);
+  TrimLeft(targets);
 }
 
-void ByteString::TrimLeft(const ByteStringView& pTargets) {
-  if (!m_pData || pTargets.IsEmpty())
+void ByteString::TrimLeft() {
+  TrimLeft(kTrimChars);
+}
+
+void ByteString::TrimLeft(char target) {
+  TrimLeft(ByteStringView(target));
+}
+
+void ByteString::TrimLeft(ByteStringView targets) {
+  if (!m_pData || targets.IsEmpty())
     return;
 
   size_t len = GetLength();
@@ -736,12 +717,10 @@ void ByteString::TrimLeft(const ByteStringView& pTargets) {
   size_t pos = 0;
   while (pos < len) {
     size_t i = 0;
-    while (i < pTargets.GetLength() && pTargets[i] != m_pData->m_String[pos]) {
+    while (i < targets.GetLength() && targets[i] != m_pData->m_String[pos])
       i++;
-    }
-    if (i == pTargets.GetLength()) {
+    if (i == targets.GetLength())
       break;
-    }
     pos++;
   }
   if (pos) {
@@ -753,25 +732,67 @@ void ByteString::TrimLeft(const ByteStringView& pTargets) {
   }
 }
 
-void ByteString::TrimLeft(char chTarget) {
-  TrimLeft(ByteStringView(chTarget));
+void ByteString::TrimRight() {
+  TrimRight(kTrimChars);
 }
 
-void ByteString::TrimLeft() {
-  TrimLeft("\x09\x0a\x0b\x0c\x0d\x20");
+void ByteString::TrimRight(char target) {
+  TrimRight(ByteStringView(target));
 }
 
-ByteString ByteString::FormatFloat(float d, int precision) {
-  char buf[32];
-  return ByteString(buf, FX_ftoa(d, buf));
+void ByteString::TrimRight(ByteStringView targets) {
+  if (!m_pData || targets.IsEmpty())
+    return;
+
+  size_t pos = GetLength();
+  if (pos == 0)
+    return;
+
+  while (pos) {
+    size_t i = 0;
+    while (i < targets.GetLength() && targets[i] != m_pData->m_String[pos - 1])
+      i++;
+    if (i == targets.GetLength())
+      break;
+    pos--;
+  }
+  if (pos < m_pData->m_nDataLength) {
+    ReallocBeforeWrite(m_pData->m_nDataLength);
+    m_pData->m_String[pos] = 0;
+    m_pData->m_nDataLength = pos;
+  }
 }
 
 std::ostream& operator<<(std::ostream& os, const ByteString& str) {
   return os.write(str.c_str(), str.GetLength());
 }
 
-std::ostream& operator<<(std::ostream& os, const ByteStringView& str) {
+std::ostream& operator<<(std::ostream& os, ByteStringView str) {
   return os.write(str.unterminated_c_str(), str.GetLength());
 }
 
 }  // namespace fxcrt
+
+uint32_t FX_HashCode_GetA(ByteStringView str, bool bIgnoreCase) {
+  uint32_t dwHashCode = 0;
+  if (bIgnoreCase) {
+    for (ByteStringView::UnsignedType c : str)
+      dwHashCode = 31 * dwHashCode + tolower(c);
+  } else {
+    for (ByteStringView::UnsignedType c : str)
+      dwHashCode = 31 * dwHashCode + c;
+  }
+  return dwHashCode;
+}
+
+uint32_t FX_HashCode_GetAsIfW(ByteStringView str, bool bIgnoreCase) {
+  uint32_t dwHashCode = 0;
+  if (bIgnoreCase) {
+    for (ByteStringView::UnsignedType c : str)
+      dwHashCode = 1313 * dwHashCode + FXSYS_towlower(c);
+  } else {
+    for (ByteStringView::UnsignedType c : str)
+      dwHashCode = 1313 * dwHashCode + c;
+  }
+  return dwHashCode;
+}
