@@ -6,6 +6,9 @@
 
 #include "core/fxcrt/css/cfx_cssdeclaration.h"
 
+#include <cmath>
+#include <utility>
+
 #include "core/fxcrt/css/cfx_csscolorvalue.h"
 #include "core/fxcrt/css/cfx_csscustomproperty.h"
 #include "core/fxcrt/css/cfx_cssenumvalue.h"
@@ -24,165 +27,16 @@ uint8_t Hex2Dec(uint8_t hexHigh, uint8_t hexLow) {
   return (FXSYS_HexCharToInt(hexHigh) << 4) + FXSYS_HexCharToInt(hexLow);
 }
 
-struct CFX_CSSPropertyValueTable {
-  CFX_CSSPropertyValue eName;
-  const wchar_t* pszName;
-  uint32_t dwHash;
-};
-const CFX_CSSPropertyValueTable g_CFX_CSSPropertyValues[] = {
-    {CFX_CSSPropertyValue::Bolder, L"bolder", 0x009F1058},
-    {CFX_CSSPropertyValue::None, L"none", 0x048B6670},
-    {CFX_CSSPropertyValue::Dot, L"dot", 0x0A48CB27},
-    {CFX_CSSPropertyValue::Sub, L"sub", 0x0BD37FAA},
-    {CFX_CSSPropertyValue::Top, L"top", 0x0BEDAF33},
-    {CFX_CSSPropertyValue::Right, L"right", 0x193ADE3E},
-    {CFX_CSSPropertyValue::Normal, L"normal", 0x247CF3E9},
-    {CFX_CSSPropertyValue::Auto, L"auto", 0x2B35B6D9},
-    {CFX_CSSPropertyValue::Text, L"text", 0x2D08AF85},
-    {CFX_CSSPropertyValue::XSmall, L"x-small", 0x2D2FCAFE},
-    {CFX_CSSPropertyValue::Thin, L"thin", 0x2D574D53},
-    {CFX_CSSPropertyValue::Small, L"small", 0x316A3739},
-    {CFX_CSSPropertyValue::Bottom, L"bottom", 0x399F02B5},
-    {CFX_CSSPropertyValue::Underline, L"underline", 0x3A0273A6},
-    {CFX_CSSPropertyValue::Double, L"double", 0x3D98515B},
-    {CFX_CSSPropertyValue::Lighter, L"lighter", 0x45BEB7AF},
-    {CFX_CSSPropertyValue::Oblique, L"oblique", 0x53EBDDB1},
-    {CFX_CSSPropertyValue::Super, L"super", 0x6A4F842F},
-    {CFX_CSSPropertyValue::Center, L"center", 0x6C51AFC1},
-    {CFX_CSSPropertyValue::XxLarge, L"xx-large", 0x70BB1508},
-    {CFX_CSSPropertyValue::Smaller, L"smaller", 0x849769F0},
-    {CFX_CSSPropertyValue::Baseline, L"baseline", 0x87436BA3},
-    {CFX_CSSPropertyValue::Thick, L"thick", 0x8CC35EB3},
-    {CFX_CSSPropertyValue::Justify, L"justify", 0x8D269CAE},
-    {CFX_CSSPropertyValue::Middle, L"middle", 0x947FA00F},
-    {CFX_CSSPropertyValue::Medium, L"medium", 0xA084A381},
-    {CFX_CSSPropertyValue::ListItem, L"list-item", 0xA32382B8},
-    {CFX_CSSPropertyValue::XxSmall, L"xx-small", 0xADE1FC76},
-    {CFX_CSSPropertyValue::Bold, L"bold", 0xB18313A1},
-    {CFX_CSSPropertyValue::SmallCaps, L"small-caps", 0xB299428D},
-    {CFX_CSSPropertyValue::Inline, L"inline", 0xC02D649F},
-    {CFX_CSSPropertyValue::Overline, L"overline", 0xC0EC9FA4},
-    {CFX_CSSPropertyValue::TextBottom, L"text-bottom", 0xC7D08D87},
-    {CFX_CSSPropertyValue::Larger, L"larger", 0xCD3C409D},
-    {CFX_CSSPropertyValue::InlineTable, L"inline-table", 0xD131F494},
-    {CFX_CSSPropertyValue::InlineBlock, L"inline-block", 0xD26A8BD7},
-    {CFX_CSSPropertyValue::Blink, L"blink", 0xDC36E390},
-    {CFX_CSSPropertyValue::Block, L"block", 0xDCD480AB},
-    {CFX_CSSPropertyValue::Italic, L"italic", 0xE31D5396},
-    {CFX_CSSPropertyValue::LineThrough, L"line-through", 0xE4C5A276},
-    {CFX_CSSPropertyValue::XLarge, L"x-large", 0xF008E390},
-    {CFX_CSSPropertyValue::Large, L"large", 0xF4434FCB},
-    {CFX_CSSPropertyValue::Left, L"left", 0xF5AD782B},
-    {CFX_CSSPropertyValue::TextTop, L"text-top", 0xFCB58D45},
-};
-const int32_t g_iCSSPropertyValueCount =
-    sizeof(g_CFX_CSSPropertyValues) / sizeof(CFX_CSSPropertyValueTable);
-static_assert(g_iCSSPropertyValueCount ==
-                  static_cast<int32_t>(CFX_CSSPropertyValue::LAST_MARKER),
-              "Property value table differs in size from property value enum");
-
-struct CFX_CSSLengthUnitTable {
-  uint16_t wHash;
-  CFX_CSSNumberType wValue;
-};
-const CFX_CSSLengthUnitTable g_CFX_CSSLengthUnits[] = {
-    {0x0672, CFX_CSSNumberType::EMS},
-    {0x067D, CFX_CSSNumberType::EXS},
-    {0x1AF7, CFX_CSSNumberType::Inches},
-    {0x2F7A, CFX_CSSNumberType::MilliMeters},
-    {0x3ED3, CFX_CSSNumberType::Picas},
-    {0x3EE4, CFX_CSSNumberType::Points},
-    {0x3EE8, CFX_CSSNumberType::Pixels},
-    {0xFC30, CFX_CSSNumberType::CentiMeters},
-};
-
-struct CFX_CSSColorTable {
-  uint32_t dwHash;
-  FX_ARGB dwValue;
-};
-const CFX_CSSColorTable g_CFX_CSSColors[] = {
-    {0x031B47FE, 0xff000080}, {0x0BB8DF5B, 0xffff0000},
-    {0x0D82A78C, 0xff800000}, {0x2ACC82E8, 0xff00ffff},
-    {0x2D083986, 0xff008080}, {0x4A6A6195, 0xffc0c0c0},
-    {0x546A8EF3, 0xff808080}, {0x65C9169C, 0xffffa500},
-    {0x8422BB61, 0xffffffff}, {0x9271A558, 0xff800080},
-    {0xA65A3EE3, 0xffff00ff}, {0xB1345708, 0xff0000ff},
-    {0xB6D2CF1F, 0xff808000}, {0xD19B5E1C, 0xffffff00},
-    {0xDB64391D, 0xff000000}, {0xF616D507, 0xff00ff00},
-    {0xF6EFFF31, 0xff008000},
-};
-
-const CFX_CSSPropertyValueTable* GetCSSPropertyValueByName(
-    const WideStringView& wsName) {
-  ASSERT(!wsName.IsEmpty());
-  uint32_t dwHash = FX_HashCode_GetW(wsName, true);
-  int32_t iEnd = g_iCSSPropertyValueCount;
-  int32_t iMid, iStart = 0;
-  uint32_t dwMid;
-  do {
-    iMid = (iStart + iEnd) / 2;
-    dwMid = g_CFX_CSSPropertyValues[iMid].dwHash;
-    if (dwHash == dwMid) {
-      return g_CFX_CSSPropertyValues + iMid;
-    } else if (dwHash > dwMid) {
-      iStart = iMid + 1;
-    } else {
-      iEnd = iMid - 1;
-    }
-  } while (iStart <= iEnd);
-  return nullptr;
-}
-
-const CFX_CSSLengthUnitTable* GetCSSLengthUnitByName(
-    const WideStringView& wsName) {
-  ASSERT(!wsName.IsEmpty());
-  uint16_t wHash = FX_HashCode_GetW(wsName, true);
-  int32_t iEnd =
-      sizeof(g_CFX_CSSLengthUnits) / sizeof(CFX_CSSLengthUnitTable) - 1;
-  int32_t iMid, iStart = 0;
-  uint16_t wMid;
-  do {
-    iMid = (iStart + iEnd) / 2;
-    wMid = g_CFX_CSSLengthUnits[iMid].wHash;
-    if (wHash == wMid) {
-      return g_CFX_CSSLengthUnits + iMid;
-    } else if (wHash > wMid) {
-      iStart = iMid + 1;
-    } else {
-      iEnd = iMid - 1;
-    }
-  } while (iStart <= iEnd);
-  return nullptr;
-}
-
-const CFX_CSSColorTable* GetCSSColorByName(const WideStringView& wsName) {
-  ASSERT(!wsName.IsEmpty());
-  uint32_t dwHash = FX_HashCode_GetW(wsName, true);
-  int32_t iEnd = sizeof(g_CFX_CSSColors) / sizeof(CFX_CSSColorTable) - 1;
-  int32_t iMid, iStart = 0;
-  uint32_t dwMid;
-  do {
-    iMid = (iStart + iEnd) / 2;
-    dwMid = g_CFX_CSSColors[iMid].dwHash;
-    if (dwHash == dwMid) {
-      return g_CFX_CSSColors + iMid;
-    } else if (dwHash > dwMid) {
-      iStart = iMid + 1;
-    } else {
-      iEnd = iMid - 1;
-    }
-  } while (iStart <= iEnd);
-  return nullptr;
-}
-
 bool ParseCSSNumber(const wchar_t* pszValue,
                     int32_t iValueLen,
                     float& fValue,
                     CFX_CSSNumberType& eUnit) {
-  ASSERT(pszValue && iValueLen > 0);
+  ASSERT(pszValue);
+  ASSERT(iValueLen > 0);
+
   int32_t iUsedLen = 0;
   fValue = FXSYS_wcstof(pszValue, iValueLen, &iUsedLen);
-  if (iUsedLen <= 0)
+  if (iUsedLen <= 0 || !std::isfinite(fValue))
     return false;
 
   iValueLen -= iUsedLen;
@@ -191,10 +45,10 @@ bool ParseCSSNumber(const wchar_t* pszValue,
   if (iValueLen >= 1 && *pszValue == '%') {
     eUnit = CFX_CSSNumberType::Percent;
   } else if (iValueLen == 2) {
-    const CFX_CSSLengthUnitTable* pUnit =
-        GetCSSLengthUnitByName(WideStringView(pszValue, 2));
+    const CFX_CSSData::LengthUnit* pUnit =
+        CFX_CSSData::GetLengthUnitByName(WideStringView(pszValue, 2));
     if (pUnit)
-      eUnit = pUnit->wValue;
+      eUnit = pUnit->type;
   }
   return true;
 }
@@ -206,7 +60,9 @@ bool CFX_CSSDeclaration::ParseCSSString(const wchar_t* pszValue,
                                         int32_t iValueLen,
                                         int32_t* iOffset,
                                         int32_t* iLength) {
-  ASSERT(pszValue && iValueLen > 0);
+  ASSERT(pszValue);
+  ASSERT(iValueLen > 0);
+
   *iOffset = 0;
   *iLength = iValueLen;
   if (iValueLen >= 2) {
@@ -223,7 +79,8 @@ bool CFX_CSSDeclaration::ParseCSSString(const wchar_t* pszValue,
 bool CFX_CSSDeclaration::ParseCSSColor(const wchar_t* pszValue,
                                        int32_t iValueLen,
                                        FX_ARGB* dwColor) {
-  ASSERT(pszValue && iValueLen > 0);
+  ASSERT(pszValue);
+  ASSERT(iValueLen > 0);
   ASSERT(dwColor);
 
   if (*pszValue == '#') {
@@ -256,7 +113,7 @@ bool CFX_CSSDeclaration::ParseCSSColor(const wchar_t* pszValue,
     CFX_CSSPrimitiveType eType;
     CFX_CSSValueListParser list(pszValue + 4, iValueLen - 5, ',');
     for (int32_t i = 0; i < 3; ++i) {
-      if (!list.NextValue(eType, pszValue, iValueLen))
+      if (!list.NextValue(&eType, &pszValue, &iValueLen))
         return false;
       if (eType != CFX_CSSPrimitiveType::Number)
         return false;
@@ -272,12 +129,12 @@ bool CFX_CSSDeclaration::ParseCSSColor(const wchar_t* pszValue,
     return true;
   }
 
-  const CFX_CSSColorTable* pColor =
-      GetCSSColorByName(WideStringView(pszValue, iValueLen));
+  const CFX_CSSData::Color* pColor =
+      CFX_CSSData::GetColorByName(WideStringView(pszValue, iValueLen));
   if (!pColor)
     return false;
 
-  *dwColor = pColor->dwValue;
+  *dwColor = pColor->value;
   return true;
 }
 
@@ -307,8 +164,8 @@ void CFX_CSSDeclaration::AddPropertyHolder(CFX_CSSProperty eProperty,
   properties_.push_back(std::move(pHolder));
 }
 
-void CFX_CSSDeclaration::AddProperty(const CFX_CSSPropertyTable* pTable,
-                                     const WideStringView& value) {
+void CFX_CSSDeclaration::AddProperty(const CFX_CSSData::Property* property,
+                                     WideStringView value) {
   ASSERT(!value.IsEmpty());
 
   const wchar_t* pszValue = value.unterminated_c_str();
@@ -321,20 +178,18 @@ void CFX_CSSDeclaration::AddProperty(const CFX_CSSPropertyTable* pTable,
 
     bImportant = true;
   }
-  const uint32_t dwType = pTable->dwType;
+  const uint32_t dwType = property->dwType;
   switch (dwType & 0x0F) {
     case CFX_CSSVALUETYPE_Primitive: {
-      static const uint32_t g_ValueGuessOrder[] = {
+      static constexpr CFX_CSSVALUETYPE kValueGuessOrder[] = {
           CFX_CSSVALUETYPE_MaybeNumber, CFX_CSSVALUETYPE_MaybeEnum,
           CFX_CSSVALUETYPE_MaybeColor, CFX_CSSVALUETYPE_MaybeString,
       };
-      static const int32_t g_ValueGuessCount =
-          sizeof(g_ValueGuessOrder) / sizeof(uint32_t);
-      for (int32_t i = 0; i < g_ValueGuessCount; ++i) {
-        const uint32_t dwMatch = dwType & g_ValueGuessOrder[i];
-        if (dwMatch == 0) {
+      for (uint32_t guess : kValueGuessOrder) {
+        const uint32_t dwMatch = dwType & guess;
+        if (dwMatch == 0)
           continue;
-        }
+
         RetainPtr<CFX_CSSValue> pCSSValue;
         switch (dwMatch) {
           case CFX_CSSVALUETYPE_MaybeNumber:
@@ -353,18 +208,18 @@ void CFX_CSSDeclaration::AddProperty(const CFX_CSSPropertyTable* pTable,
             break;
         }
         if (pCSSValue) {
-          AddPropertyHolder(pTable->eName, pCSSValue, bImportant);
+          AddPropertyHolder(property->eName, pCSSValue, bImportant);
           return;
         }
 
-        if ((dwType & ~(g_ValueGuessOrder[i])) == CFX_CSSVALUETYPE_Primitive)
+        if ((dwType & ~guess) == CFX_CSSVALUETYPE_Primitive)
           return;
       }
       break;
     }
     case CFX_CSSVALUETYPE_Shorthand: {
       RetainPtr<CFX_CSSValue> pWidth;
-      switch (pTable->eName) {
+      switch (property->eName) {
         case CFX_CSSProperty::Font:
           ParseFontProperty(pszValue, iValueLen, bImportant);
           return;
@@ -414,7 +269,7 @@ void CFX_CSSDeclaration::AddProperty(const CFX_CSSPropertyTable* pTable,
       }
     } break;
     case CFX_CSSVALUETYPE_List:
-      ParseValueListProperty(pTable, pszValue, iValueLen, bImportant);
+      ParseValueListProperty(property, pszValue, iValueLen, bImportant);
       return;
     default:
       NOTREACHED();
@@ -439,8 +294,8 @@ RetainPtr<CFX_CSSValue> CFX_CSSDeclaration::ParseNumber(const wchar_t* pszValue,
 
 RetainPtr<CFX_CSSValue> CFX_CSSDeclaration::ParseEnum(const wchar_t* pszValue,
                                                       int32_t iValueLen) {
-  const CFX_CSSPropertyValueTable* pValue =
-      GetCSSPropertyValueByName(WideStringView(pszValue, iValueLen));
+  const CFX_CSSData::PropertyValue* pValue =
+      CFX_CSSData::GetPropertyValueByName(WideStringView(pszValue, iValueLen));
   return pValue ? pdfium::MakeRetain<CFX_CSSEnumValue>(pValue->eName) : nullptr;
 }
 
@@ -466,18 +321,18 @@ RetainPtr<CFX_CSSValue> CFX_CSSDeclaration::ParseString(const wchar_t* pszValue,
 }
 
 void CFX_CSSDeclaration::ParseValueListProperty(
-    const CFX_CSSPropertyTable* pTable,
+    const CFX_CSSData::Property* pProperty,
     const wchar_t* pszValue,
     int32_t iValueLen,
     bool bImportant) {
   wchar_t separator =
-      (pTable->eName == CFX_CSSProperty::FontFamily) ? ',' : ' ';
+      (pProperty->eName == CFX_CSSProperty::FontFamily) ? ',' : ' ';
   CFX_CSSValueListParser parser(pszValue, iValueLen, separator);
 
-  const uint32_t dwType = pTable->dwType;
+  const uint32_t dwType = pProperty->dwType;
   CFX_CSSPrimitiveType eType;
   std::vector<RetainPtr<CFX_CSSValue>> list;
-  while (parser.NextValue(eType, pszValue, iValueLen)) {
+  while (parser.NextValue(&eType, &pszValue, &iValueLen)) {
     switch (eType) {
       case CFX_CSSPrimitiveType::Number:
         if (dwType & CFX_CSSVALUETYPE_MaybeNumber) {
@@ -497,8 +352,9 @@ void CFX_CSSDeclaration::ParseValueListProperty(
           }
         }
         if (dwType & CFX_CSSVALUETYPE_MaybeEnum) {
-          const CFX_CSSPropertyValueTable* pValue =
-              GetCSSPropertyValueByName(WideStringView(pszValue, iValueLen));
+          const CFX_CSSData::PropertyValue* pValue =
+              CFX_CSSData::GetPropertyValueByName(
+                  WideStringView(pszValue, iValueLen));
           if (pValue) {
             list.push_back(pdfium::MakeRetain<CFX_CSSEnumValue>(pValue->eName));
             continue;
@@ -524,7 +380,7 @@ void CFX_CSSDeclaration::ParseValueListProperty(
   if (list.empty())
     return;
 
-  switch (pTable->eName) {
+  switch (pProperty->eName) {
     case CFX_CSSProperty::BorderWidth:
       Add4ValuesProperty(list, bImportant, CFX_CSSProperty::BorderLeftWidth,
                          CFX_CSSProperty::BorderTopWidth,
@@ -545,7 +401,7 @@ void CFX_CSSDeclaration::ParseValueListProperty(
       return;
     default: {
       auto pList = pdfium::MakeRetain<CFX_CSSValueList>(list);
-      AddPropertyHolder(pTable->eName, pList, bImportant);
+      AddPropertyHolder(pProperty->eName, pList, bImportant);
       return;
     }
   }
@@ -596,7 +452,7 @@ bool CFX_CSSDeclaration::ParseBorderProperty(
 
   CFX_CSSValueListParser parser(pszValue, iValueLen, ' ');
   CFX_CSSPrimitiveType eType;
-  while (parser.NextValue(eType, pszValue, iValueLen)) {
+  while (parser.NextValue(&eType, &pszValue, &iValueLen)) {
     switch (eType) {
       case CFX_CSSPrimitiveType::Number: {
         if (pWidth)
@@ -609,13 +465,14 @@ bool CFX_CSSDeclaration::ParseBorderProperty(
         break;
       }
       case CFX_CSSPrimitiveType::String: {
-        const CFX_CSSColorTable* pColorItem =
-            GetCSSColorByName(WideStringView(pszValue, iValueLen));
+        const CFX_CSSData::Color* pColorItem =
+            CFX_CSSData::GetColorByName(WideStringView(pszValue, iValueLen));
         if (pColorItem)
           continue;
 
-        const CFX_CSSPropertyValueTable* pValue =
-            GetCSSPropertyValueByName(WideStringView(pszValue, iValueLen));
+        const CFX_CSSData::PropertyValue* pValue =
+            CFX_CSSData::GetPropertyValueByName(
+                WideStringView(pszValue, iValueLen));
         if (!pValue)
           continue;
 
@@ -653,11 +510,12 @@ void CFX_CSSDeclaration::ParseFontProperty(const wchar_t* pszValue,
   RetainPtr<CFX_CSSValue> pLineHeight;
   std::vector<RetainPtr<CFX_CSSValue>> familyList;
   CFX_CSSPrimitiveType eType;
-  while (parser.NextValue(eType, pszValue, iValueLen)) {
+  while (parser.NextValue(&eType, &pszValue, &iValueLen)) {
     switch (eType) {
       case CFX_CSSPrimitiveType::String: {
-        const CFX_CSSPropertyValueTable* pValue =
-            GetCSSPropertyValueByName(WideStringView(pszValue, iValueLen));
+        const CFX_CSSData::PropertyValue* pValue =
+            CFX_CSSData::GetPropertyValueByName(
+                WideStringView(pszValue, iValueLen));
         if (pValue) {
           switch (pValue->eName) {
             case CFX_CSSPropertyValue::XxSmall:
@@ -708,7 +566,7 @@ void CFX_CSSDeclaration::ParseFontProperty(const wchar_t* pszValue,
           familyList.push_back(pdfium::MakeRetain<CFX_CSSStringValue>(
               WideString(pszValue, iValueLen)));
         }
-        parser.m_Separator = ',';
+        parser.UseCommaSeparator();
         break;
       }
       case CFX_CSSPrimitiveType::Number: {
