@@ -6,8 +6,10 @@
 
 #include "core/fxcrt/fx_system.h"
 
+#include <cmath>
 #include <limits>
 
+#include "build/build_config.h"
 #include "core/fxcrt/fx_extension.h"
 
 namespace {
@@ -23,7 +25,7 @@ IntType FXSYS_StrToInt(const CharType* str) {
     str++;
 
   IntType num = 0;
-  while (*str && FXSYS_isDecimalDigit(*str)) {
+  while (*str && FXSYS_IsDecimalDigit(*str)) {
     IntType val = FXSYS_DecimalCharToInt(*str);
     if (num > (std::numeric_limits<IntType>::max() - val) / 10) {
       if (neg && std::numeric_limits<IntType>::is_signed) {
@@ -83,6 +85,8 @@ STR_T FXSYS_IntToStr(T value, STR_T str, int radix) {
 }  // namespace
 
 int FXSYS_round(float d) {
+  if (std::isnan(d))
+    return 0;
   if (d < static_cast<float>(std::numeric_limits<int>::min()))
     return std::numeric_limits<int>::min();
   if (d > static_cast<float>(std::numeric_limits<int>::max()))
@@ -106,7 +110,27 @@ const char* FXSYS_i64toa(int64_t value, char* str, int radix) {
   return FXSYS_IntToStr<int64_t, uint64_t, char*>(value, str, radix);
 }
 
-#if _FX_PLATFORM_ != _FX_PLATFORM_WINDOWS_
+#if defined(OS_WIN)
+
+size_t FXSYS_wcsftime(wchar_t* strDest,
+                      size_t maxsize,
+                      const wchar_t* format,
+                      const struct tm* timeptr) {
+  // Avoid tripping an invalid parameter handler and crashing process.
+  // Note: leap seconds may cause tm_sec == 60.
+  if (timeptr->tm_year < -1900 || timeptr->tm_year > 8099 ||
+      timeptr->tm_mon < 0 || timeptr->tm_mon > 11 || timeptr->tm_mday < 1 ||
+      timeptr->tm_mday > 31 || timeptr->tm_hour < 0 || timeptr->tm_hour > 23 ||
+      timeptr->tm_min < 0 || timeptr->tm_min > 59 || timeptr->tm_sec < 0 ||
+      timeptr->tm_sec > 60 || timeptr->tm_wday < 0 || timeptr->tm_wday > 6 ||
+      timeptr->tm_yday < 0 || timeptr->tm_yday > 365) {
+    strDest[0] = L'\0';
+    return 0;
+  }
+  return wcsftime(strDest, maxsize, format, timeptr);
+}
+
+#else  // defined(OS_WIN)
 
 int FXSYS_GetACP() {
   return 0;
@@ -118,7 +142,7 @@ char* FXSYS_strlwr(char* str) {
   }
   char* s = str;
   while (*str) {
-    *str = FXSYS_tolower(*str);
+    *str = tolower(*str);
     str++;
   }
   return s;
@@ -129,7 +153,7 @@ char* FXSYS_strupr(char* str) {
   }
   char* s = str;
   while (*str) {
-    *str = FXSYS_toupper(*str);
+    *str = toupper(*str);
     str++;
   }
   return s;
@@ -140,7 +164,7 @@ wchar_t* FXSYS_wcslwr(wchar_t* str) {
   }
   wchar_t* s = str;
   while (*str) {
-    *str = FXSYS_tolower(*str);
+    *str = FXSYS_towlower(*str);
     str++;
   }
   return s;
@@ -151,7 +175,7 @@ wchar_t* FXSYS_wcsupr(wchar_t* str) {
   }
   wchar_t* s = str;
   while (*str) {
-    *str = FXSYS_toupper(*str);
+    *str = FXSYS_towupper(*str);
     str++;
   }
   return s;
@@ -161,8 +185,8 @@ int FXSYS_stricmp(const char* dst, const char* src) {
   int f;
   int l;
   do {
-    f = FXSYS_toupper(*dst);
-    l = FXSYS_toupper(*src);
+    f = toupper(*dst);
+    l = toupper(*src);
     ++dst;
     ++src;
   } while (f && f == l);
@@ -173,8 +197,8 @@ int FXSYS_wcsicmp(const wchar_t* dst, const wchar_t* src) {
   wchar_t f;
   wchar_t l;
   do {
-    f = FXSYS_toupper(*dst);
-    l = FXSYS_toupper(*src);
+    f = FXSYS_towupper(*dst);
+    l = FXSYS_towupper(*src);
     ++dst;
     ++src;
   } while (f && f == l);
@@ -203,6 +227,7 @@ int FXSYS_WideCharToMultiByte(uint32_t codepage,
   }
   return len;
 }
+
 int FXSYS_MultiByteToWideChar(uint32_t codepage,
                               uint32_t dwFlags,
                               const char* bstr,
@@ -211,32 +236,11 @@ int FXSYS_MultiByteToWideChar(uint32_t codepage,
                               int buflen) {
   int wlen = 0;
   for (int i = 0; i < blen; i++) {
-    if (buf && wlen < buflen) {
-      buf[wlen] = bstr[i];
-    }
+    if (buf && wlen < buflen)
+      buf[wlen] = reinterpret_cast<const uint8_t*>(bstr)[i];
     wlen++;
   }
   return wlen;
 }
 
-#else  // _FX_PLATFORM_ != _FX_PLATFORM_WINDOWS_
-
-size_t FXSYS_wcsftime(wchar_t* strDest,
-                      size_t maxsize,
-                      const wchar_t* format,
-                      const struct tm* timeptr) {
-  // Avoid tripping an invalid parameter handler and crashing process.
-  // Note: leap seconds may cause tm_sec == 60.
-  if (timeptr->tm_year < -1900 || timeptr->tm_year > 8099 ||
-      timeptr->tm_mon < 0 || timeptr->tm_mon > 11 || timeptr->tm_mday < 1 ||
-      timeptr->tm_mday > 31 || timeptr->tm_hour < 0 || timeptr->tm_hour > 23 ||
-      timeptr->tm_min < 0 || timeptr->tm_min > 59 || timeptr->tm_sec < 0 ||
-      timeptr->tm_sec > 60 || timeptr->tm_wday < 0 || timeptr->tm_wday > 6 ||
-      timeptr->tm_yday < 0 || timeptr->tm_yday > 365) {
-    strDest[0] = L'\0';
-    return 0;
-  }
-  return wcsftime(strDest, maxsize, format, timeptr);
-}
-
-#endif  // _FX_PLATFORM_ != _FX_PLATFORM_WINDOWS_
+#endif  // defined(OS_WIN)
