@@ -203,9 +203,11 @@ TEST(RetainPtr, Equals) {
   {
     RetainPtr<PseudoRetainable> null_ptr2;
     EXPECT_TRUE(null_ptr1 == null_ptr2);
+    EXPECT_TRUE(null_ptr1 == nullptr);
 
     RetainPtr<PseudoRetainable> obj1_ptr2(&obj1);
     EXPECT_TRUE(obj1_ptr1 == obj1_ptr2);
+    EXPECT_TRUE(obj1_ptr2 == &obj1);
 
     RetainPtr<PseudoRetainable> obj2_ptr2(&obj2);
     EXPECT_TRUE(obj2_ptr1 == obj2_ptr2);
