@@ -13,6 +13,7 @@
 #include <utility>
 #include <vector>
 
+#include "build/build_config.h"
 #include "core/fxcrt/fx_codepage.h"
 #include "core/fxge/cfx_fontmgr.h"
 #include "core/fxge/cfx_substfont.h"
@@ -151,7 +152,7 @@ const AltFontFamily g_AltFontFamilies[] = {
 
 #if _FX_PLATFORM_ == _FX_PLATFORM_LINUX_
 const char kNarrowFamily[] = "LiberationSansNarrow";
-#elif _FX_PLATFORM_ == _FX_PLATFORM_ANDROID_
+#elif defined(OS_ANDROID)
 const char kNarrowFamily[] = "RobotoCondensed";
 #else
 const char kNarrowFamily[] = "ArialNarrow";
@@ -403,7 +404,7 @@ FXFT_Face CFX_FontMapper::FindSubstFont(const ByteString& name,
   SubstName.Remove(' ');
   if (bTrueType && name.GetLength() > 0 && name[0] == '@')
     SubstName = name.Right(name.GetLength() - 1);
-  PDF_GetStandardFontName(&SubstName);
+  GetStandardFontName(&SubstName);
   if (SubstName == "Symbol" && !bTrueType) {
     pSubstFont->m_Family = "Chrome Symbol";
     pSubstFont->m_Charset = FX_CHARSET_Symbol;
@@ -423,7 +424,7 @@ FXFT_Face CFX_FontMapper::FindSubstFont(const ByteString& name,
     Optional<size_t> pos = SubstName.Find(",", 0);
     if (pos.has_value()) {
       family = SubstName.Left(pos.value());
-      PDF_GetStandardFontName(&family);
+      GetStandardFontName(&family);
       style = SubstName.Right(SubstName.GetLength() - (pos.value() + 1));
       bHasComma = true;
     } else {
@@ -597,8 +598,7 @@ FXFT_Face CFX_FontMapper::FindSubstFont(const ByteString& name,
       }
     } else {
       if (Charset == FX_CHARSET_Symbol) {
-#if _FX_PLATFORM_ == _FX_PLATFORM_APPLE_ || \
-    _FX_PLATFORM_ == _FX_PLATFORM_ANDROID_
+#if defined(OS_MACOSX) || defined(OS_ANDROID)
         if (SubstName == "Symbol") {
           pSubstFont->m_Family = "Chrome Symbol";
           pSubstFont->m_Charset = FX_CHARSET_Symbol;
@@ -727,7 +727,8 @@ FXFT_Face CFX_FontMapper::GetCachedFace(void* hFont,
                                    m_pFontInfo->GetFaceIndex(hFont));
 }
 
-int PDF_GetStandardFontName(ByteString* name) {
+// static
+int CFX_FontMapper::GetStandardFontName(ByteString* name) {
   const auto* end = std::end(g_AltFontNames);
   const auto* found =
       std::lower_bound(std::begin(g_AltFontNames), end, name->c_str(),
