@@ -12,17 +12,18 @@
 #include "core/fxcrt/fx_coordinates.h"
 #include "core/fxcrt/fx_system.h"
 #include "core/fxcrt/retain_ptr.h"
+#include "core/fxge/fx_dib.h"
 
-class CFX_DIBitmap;
 class CFX_DIBBase;
+class CFX_DIBitmap;
 class CFX_Font;
 class CFX_GraphStateData;
 class CFX_ImageRenderer;
 class CFX_Matrix;
 class CFX_PathData;
 class CPDF_ShadingPattern;
-class FXTEXT_CHARPOS;
 class PauseIndicatorIface;
+class TextCharPos;
 struct FX_RECT;
 
 class RenderDeviceDriverIface {
@@ -48,15 +49,15 @@ class RenderDeviceDriverIface {
                         uint32_t fill_color,
                         uint32_t stroke_color,
                         int fill_mode,
-                        int blend_type) = 0;
+                        BlendMode blend_type) = 0;
   virtual bool SetPixel(int x, int y, uint32_t color);
   virtual bool FillRectWithBlend(const FX_RECT& rect,
                                  uint32_t fill_color,
-                                 int blend_type);
+                                 BlendMode blend_type);
   virtual bool DrawCosmeticLine(const CFX_PointF& ptMoveTo,
                                 const CFX_PointF& ptLineTo,
                                 uint32_t color,
-                                int blend_type);
+                                BlendMode blend_type);
 
   virtual bool GetClipBox(FX_RECT* pRect) = 0;
   virtual bool GetDIBits(const RetainPtr<CFX_DIBitmap>& pBitmap,
@@ -65,10 +66,10 @@ class RenderDeviceDriverIface {
   virtual RetainPtr<CFX_DIBitmap> GetBackDrop();
   virtual bool SetDIBits(const RetainPtr<CFX_DIBBase>& pBitmap,
                          uint32_t color,
-                         const FX_RECT* pSrcRect,
+                         const FX_RECT& src_rect,
                          int dest_left,
                          int dest_top,
-                         int blend_type) = 0;
+                         BlendMode blend_type) = 0;
   virtual bool StretchDIBits(const RetainPtr<CFX_DIBBase>& pBitmap,
                              uint32_t color,
                              int dest_left,
@@ -76,19 +77,19 @@ class RenderDeviceDriverIface {
                              int dest_width,
                              int dest_height,
                              const FX_RECT* pClipRect,
-                             uint32_t flags,
-                             int blend_type) = 0;
+                             const FXDIB_ResampleOptions& options,
+                             BlendMode blend_type) = 0;
   virtual bool StartDIBits(const RetainPtr<CFX_DIBBase>& pBitmap,
                            int bitmap_alpha,
                            uint32_t color,
-                           const CFX_Matrix* pMatrix,
-                           uint32_t flags,
+                           const CFX_Matrix& matrix,
+                           const FXDIB_ResampleOptions& options,
                            std::unique_ptr<CFX_ImageRenderer>* handle,
-                           int blend_type) = 0;
+                           BlendMode blend_type) = 0;
   virtual bool ContinueDIBits(CFX_ImageRenderer* handle,
                               PauseIndicatorIface* pPause);
   virtual bool DrawDeviceText(int nChars,
-                              const FXTEXT_CHARPOS* pCharPos,
+                              const TextCharPos* pCharPos,
                               CFX_Font* pFont,
                               const CFX_Matrix* pObject2Device,
                               float font_size,
@@ -105,7 +106,7 @@ class RenderDeviceDriverIface {
                                int left,
                                int top,
                                int bitmap_alpha,
-                               int blend_type);
+                               BlendMode blend_type);
 #if defined _SKIA_SUPPORT_ || defined _SKIA_SUPPORT_PATHS_
   virtual void Flush();
 #endif
