@@ -11,14 +11,14 @@
 
 CPDFSDK_XFAWidget::CPDFSDK_XFAWidget(CXFA_FFWidget* pAnnot,
                                      CPDFSDK_PageView* pPageView,
-                                     CPDFSDK_InterForm* pInterForm)
+                                     CPDFSDK_InteractiveForm* pInteractiveForm)
     : CPDFSDK_Annot(pPageView),
-      m_pInterForm(pInterForm),
+      m_pInteractiveForm(pInteractiveForm),
       m_hXFAWidget(pAnnot) {}
 
-CPDFSDK_XFAWidget::~CPDFSDK_XFAWidget() {}
+CPDFSDK_XFAWidget::~CPDFSDK_XFAWidget() = default;
 
-bool CPDFSDK_XFAWidget::IsXFAField() {
+bool CPDFSDK_XFAWidget::IsXFAField() const {
   return true;
 }
 
@@ -31,7 +31,5 @@ CPDF_Annot::Subtype CPDFSDK_XFAWidget::GetAnnotSubtype() const {
 }
 
 CFX_FloatRect CPDFSDK_XFAWidget::GetRect() const {
-  CFX_RectF rcBBox = GetXFAWidget()->GetRect(false);
-  return CFX_FloatRect(rcBBox.left, rcBBox.top, rcBBox.left + rcBBox.width,
-                       rcBBox.top + rcBBox.height);
+  return GetXFAWidget()->GetLayoutItem()->GetRect(false).ToFloatRect();
 }
