@@ -9,10 +9,11 @@
 #include "fpdfsdk/formfiller/cffl_interactiveformfiller.h"
 #include "fpdfsdk/pwl/cpwl_combo_box.h"
 #include "fpdfsdk/pwl/cpwl_wnd.h"
+#include "public/fpdf_fwlevent.h"
 #include "testing/embedder_test.h"
 #include "testing/gtest/include/gtest/gtest.h"
 
-class CPWLComboBoxEditEmbeddertest : public EmbedderTest {
+class CPWLComboBoxEditEmbedderTest : public EmbedderTest {
  protected:
   void SetUp() override {
     EmbedderTest::SetUp();
@@ -59,13 +60,12 @@ class CPWLComboBoxEditEmbeddertest : public EmbedderTest {
     }
 
     m_pFormFiller =
-        pInteractiveFormFiller->GetFormFiller(pAnnotCombobox, false);
+        pInteractiveFormFiller->GetFormFillerForTesting(pAnnotCombobox);
     ASSERT_TRUE(m_pFormFiller);
 
     CPWL_Wnd* pWindow =
         m_pFormFiller->GetPDFWindow(m_pFormFillEnv->GetPageView(0), false);
     ASSERT_TRUE(pWindow);
-    ASSERT_EQ("CPWL_ComboBox", pWindow->GetClassName());
     m_pComboBox = static_cast<CPWL_ComboBox*>(pWindow);
   }
 
@@ -97,7 +97,7 @@ class CPWLComboBoxEditEmbeddertest : public EmbedderTest {
   CPDFSDK_FormFillEnvironment* m_pFormFillEnv;
 };
 
-TEST_F(CPWLComboBoxEditEmbeddertest, GetSelectedTextEmptyAndBasicNormal) {
+TEST_F(CPWLComboBoxEditEmbedderTest, GetSelectedTextEmptyAndBasicNormal) {
   FormFillerAndWindowSetup(GetCPDFSDKAnnotNormal());
 
   // Automatically pre-filled with "Banana".
@@ -119,7 +119,7 @@ TEST_F(CPWLComboBoxEditEmbeddertest, GetSelectedTextEmptyAndBasicNormal) {
   EXPECT_FALSE(GetCFFLFormFiller()->OnChar(GetCPDFSDKAnnotNormal(), 'a', 0));
 }
 
-TEST_F(CPWLComboBoxEditEmbeddertest, GetSelectedTextFragmentsNormal) {
+TEST_F(CPWLComboBoxEditEmbedderTest, GetSelectedTextFragmentsNormal) {
   FormFillerAndWindowSetup(GetCPDFSDKAnnotNormal());
   EXPECT_STREQ(L"Banana", GetCPWLComboBox()->GetText().c_str());
 
@@ -145,7 +145,7 @@ TEST_F(CPWLComboBoxEditEmbeddertest, GetSelectedTextFragmentsNormal) {
   EXPECT_STREQ(L"a", GetCPWLComboBox()->GetSelectedText().c_str());
 }
 
-TEST_F(CPWLComboBoxEditEmbeddertest, GetSelectedTextEmptyAndBasicEditable) {
+TEST_F(CPWLComboBoxEditEmbedderTest, GetSelectedTextEmptyAndBasicEditable) {
   FormFillerAndWindowSetup(GetCPDFSDKAnnotUserEditable());
   EXPECT_TRUE(GetCPWLComboBox()->GetText().IsEmpty());
 
@@ -174,7 +174,7 @@ TEST_F(CPWLComboBoxEditEmbeddertest, GetSelectedTextEmptyAndBasicEditable) {
   EXPECT_STREQ(L"Baabc", GetCPWLComboBox()->GetSelectedText().c_str());
 }
 
-TEST_F(CPWLComboBoxEditEmbeddertest, GetSelectedTextFragmentsEditable) {
+TEST_F(CPWLComboBoxEditEmbedderTest, GetSelectedTextFragmentsEditable) {
   FormFillerAndWindowSetup(GetCPDFSDKAnnotUserEditable());
   TypeTextIntoTextField(50);
 
@@ -204,7 +204,7 @@ TEST_F(CPWLComboBoxEditEmbeddertest, GetSelectedTextFragmentsEditable) {
   EXPECT_STREQ(L"r", GetCPWLComboBox()->GetSelectedText().c_str());
 }
 
-TEST_F(CPWLComboBoxEditEmbeddertest, DeleteEntireTextSelection) {
+TEST_F(CPWLComboBoxEditEmbedderTest, DeleteEntireTextSelection) {
   FormFillerAndWindowSetup(GetCPDFSDKAnnotUserEditable());
   TypeTextIntoTextField(50);
 
@@ -216,7 +216,7 @@ TEST_F(CPWLComboBoxEditEmbeddertest, DeleteEntireTextSelection) {
   EXPECT_TRUE(GetCPWLComboBox()->GetText().IsEmpty());
 }
 
-TEST_F(CPWLComboBoxEditEmbeddertest, DeleteTextSelectionMiddle) {
+TEST_F(CPWLComboBoxEditEmbedderTest, DeleteTextSelectionMiddle) {
   FormFillerAndWindowSetup(GetCPDFSDKAnnotUserEditable());
   TypeTextIntoTextField(50);
 
@@ -228,7 +228,7 @@ TEST_F(CPWLComboBoxEditEmbeddertest, DeleteTextSelectionMiddle) {
                GetCPWLComboBox()->GetText().c_str());
 }
 
-TEST_F(CPWLComboBoxEditEmbeddertest, DeleteTextSelectionLeft) {
+TEST_F(CPWLComboBoxEditEmbedderTest, DeleteTextSelectionLeft) {
   FormFillerAndWindowSetup(GetCPDFSDKAnnotUserEditable());
   TypeTextIntoTextField(50);
 
@@ -240,7 +240,7 @@ TEST_F(CPWLComboBoxEditEmbeddertest, DeleteTextSelectionLeft) {
                GetCPWLComboBox()->GetText().c_str());
 }
 
-TEST_F(CPWLComboBoxEditEmbeddertest, DeleteTextSelectionRight) {
+TEST_F(CPWLComboBoxEditEmbedderTest, DeleteTextSelectionRight) {
   FormFillerAndWindowSetup(GetCPDFSDKAnnotUserEditable());
   TypeTextIntoTextField(50);
 
@@ -252,7 +252,7 @@ TEST_F(CPWLComboBoxEditEmbeddertest, DeleteTextSelectionRight) {
                GetCPWLComboBox()->GetText().c_str());
 }
 
-TEST_F(CPWLComboBoxEditEmbeddertest, DeleteEmptyTextSelection) {
+TEST_F(CPWLComboBoxEditEmbedderTest, DeleteEmptyTextSelection) {
   FormFillerAndWindowSetup(GetCPDFSDKAnnotUserEditable());
   TypeTextIntoTextField(50);
 
@@ -261,41 +261,39 @@ TEST_F(CPWLComboBoxEditEmbeddertest, DeleteEmptyTextSelection) {
                GetCPWLComboBox()->GetText().c_str());
 }
 
-TEST_F(CPWLComboBoxEditEmbeddertest, InsertTextInEmptyEditableComboBox) {
+TEST_F(CPWLComboBoxEditEmbedderTest, InsertTextInEmptyEditableComboBox) {
   FormFillerAndWindowSetup(GetCPDFSDKAnnotUserEditable());
   GetCPWLComboBox()->ReplaceSelection(L"Hello");
   EXPECT_STREQ(L"Hello", GetCPWLComboBox()->GetText().c_str());
 }
 
-TEST_F(CPWLComboBoxEditEmbeddertest,
+TEST_F(CPWLComboBoxEditEmbedderTest,
        InsertTextInPopulatedEditableComboBoxLeft) {
   FormFillerAndWindowSetup(GetCPDFSDKAnnotUserEditable());
   TypeTextIntoTextField(10);
 
   // Move cursor to beginning of user-editable combobox text field.
-  EXPECT_TRUE(GetCFFLFormFiller()->OnKeyDown(GetCPDFSDKAnnotUserEditable(),
-                                             FWL_VKEY_Home, 0));
+  EXPECT_TRUE(GetCFFLFormFiller()->OnKeyDown(FWL_VKEY_Home, 0));
 
   GetCPWLComboBox()->ReplaceSelection(L"Hello");
   EXPECT_STREQ(L"HelloABCDEFGHIJ", GetCPWLComboBox()->GetText().c_str());
 }
 
-TEST_F(CPWLComboBoxEditEmbeddertest,
+TEST_F(CPWLComboBoxEditEmbedderTest,
        InsertTextInPopulatedEditableComboBoxMiddle) {
   FormFillerAndWindowSetup(GetCPDFSDKAnnotUserEditable());
   TypeTextIntoTextField(10);
 
   // Move cursor to middle of user-editable combobox text field.
   for (int i = 0; i < 5; ++i) {
-    EXPECT_TRUE(GetCFFLFormFiller()->OnKeyDown(GetCPDFSDKAnnotUserEditable(),
-                                               FWL_VKEY_Left, 0));
+    EXPECT_TRUE(GetCFFLFormFiller()->OnKeyDown(FWL_VKEY_Left, 0));
   }
 
   GetCPWLComboBox()->ReplaceSelection(L"Hello");
   EXPECT_STREQ(L"ABCDEHelloFGHIJ", GetCPWLComboBox()->GetText().c_str());
 }
 
-TEST_F(CPWLComboBoxEditEmbeddertest,
+TEST_F(CPWLComboBoxEditEmbedderTest,
        InsertTextInPopulatedEditableComboBoxRight) {
   FormFillerAndWindowSetup(GetCPDFSDKAnnotUserEditable());
   TypeTextIntoTextField(10);
@@ -304,7 +302,7 @@ TEST_F(CPWLComboBoxEditEmbeddertest,
   EXPECT_STREQ(L"ABCDEFGHIJHello", GetCPWLComboBox()->GetText().c_str());
 }
 
-TEST_F(CPWLComboBoxEditEmbeddertest,
+TEST_F(CPWLComboBoxEditEmbedderTest,
        InsertTextAndReplaceSelectionInPopulatedEditableComboBoxWhole) {
   FormFillerAndWindowSetup(GetCPDFSDKAnnotUserEditable());
   TypeTextIntoTextField(10);
@@ -315,7 +313,7 @@ TEST_F(CPWLComboBoxEditEmbeddertest,
   EXPECT_STREQ(L"Hello", GetCPWLComboBox()->GetText().c_str());
 }
 
-TEST_F(CPWLComboBoxEditEmbeddertest,
+TEST_F(CPWLComboBoxEditEmbedderTest,
        InsertTextAndReplaceSelectionInPopulatedEditableComboBoxLeft) {
   FormFillerAndWindowSetup(GetCPDFSDKAnnotUserEditable());
   TypeTextIntoTextField(10);
@@ -326,7 +324,7 @@ TEST_F(CPWLComboBoxEditEmbeddertest,
   EXPECT_STREQ(L"HelloFGHIJ", GetCPWLComboBox()->GetText().c_str());
 }
 
-TEST_F(CPWLComboBoxEditEmbeddertest,
+TEST_F(CPWLComboBoxEditEmbedderTest,
        InsertTextAndReplaceSelectionInPopulatedEditableComboBoxMiddle) {
   FormFillerAndWindowSetup(GetCPDFSDKAnnotUserEditable());
   TypeTextIntoTextField(10);
@@ -337,7 +335,7 @@ TEST_F(CPWLComboBoxEditEmbeddertest,
   EXPECT_STREQ(L"ABHelloHIJ", GetCPWLComboBox()->GetText().c_str());
 }
 
-TEST_F(CPWLComboBoxEditEmbeddertest,
+TEST_F(CPWLComboBoxEditEmbedderTest,
        InsertTextAndReplaceSelectionInPopulatedEditableComboBoxRight) {
   FormFillerAndWindowSetup(GetCPDFSDKAnnotUserEditable());
   TypeTextIntoTextField(10);
