@@ -26,6 +26,7 @@
 #include <memory>
 #include <vector>
 
+#include "build/build_config.h"
 #include "core/fxge/cfx_defaultrenderdevice.h"
 #include "core/fxge/cfx_font.h"
 #include "core/fxge/cfx_graphstatedata.h"
@@ -33,23 +34,10 @@
 #include "core/fxge/cfx_renderdevice.h"
 #include "core/fxge/cfx_unicodeencodingex.h"
 #include "fxbarcode/BC_Writer.h"
-#include "third_party/base/ptr_util.h"
 
-CBC_OneDimWriter::CBC_OneDimWriter() {
-  m_locTextLoc = BC_TEXT_LOC_BELOWEMBED;
-  m_bPrintChecksum = true;
-  m_iDataLenth = 0;
-  m_bCalcChecksum = false;
-  m_pFont = nullptr;
-  m_fFontSize = 10;
-  m_iFontStyle = 0;
-  m_fontColor = 0xff000000;
-  m_iContentLen = 0;
-  m_bLeftPadding = false;
-  m_bRightPadding = false;
-}
+CBC_OneDimWriter::CBC_OneDimWriter() = default;
 
-CBC_OneDimWriter::~CBC_OneDimWriter() {}
+CBC_OneDimWriter::~CBC_OneDimWriter() = default;
 
 void CBC_OneDimWriter::SetPrintChecksum(bool checksum) {
   m_bPrintChecksum = checksum;
@@ -83,13 +71,6 @@ void CBC_OneDimWriter::SetFontColor(FX_ARGB color) {
   m_fontColor = color;
 }
 
-wchar_t CBC_OneDimWriter::Upper(wchar_t ch) {
-  if (ch >= 'a' && ch <= 'z') {
-    ch = ch - ('a' - 'A');
-  }
-  return ch;
-}
-
 uint8_t* CBC_OneDimWriter::EncodeWithHint(const ByteString& contents,
                                           BCFORMAT format,
                                           int32_t& outWidth,
@@ -110,41 +91,35 @@ int32_t CBC_OneDimWriter::AppendPattern(uint8_t* target,
                                         int32_t pos,
                                         const int8_t* pattern,
                                         int32_t patternLength,
-                                        int32_t startColor,
-                                        int32_t& e) {
-  if (startColor != 0 && startColor != 1) {
-    e = BCExceptionValueMustBeEither0or1;
-    return 0;
-  }
-  uint8_t color = (uint8_t)startColor;
+                                        bool startColor) {
+  bool color = startColor;
   int32_t numAdded = 0;
   for (int32_t i = 0; i < patternLength; i++) {
-    for (int32_t j = 0; j < pattern[i]; j++) {
-      target[pos++] = color;
-      numAdded += 1;
-    }
-    color ^= 1;
+    for (int32_t j = 0; j < pattern[i]; j++)
+      target[pos++] = color ? 1 : 0;
+    numAdded += pattern[i];
+    color = !color;
   }
   return numAdded;
 }
 
 void CBC_OneDimWriter::CalcTextInfo(const ByteString& text,
-                                    FXTEXT_CHARPOS* charPos,
+                                    TextCharPos* charPos,
                                     CFX_Font* cFont,
                                     float geWidth,
                                     int32_t fontSize,
                                     float& charsLen) {
   std::unique_ptr<CFX_UnicodeEncodingEx> encoding =
-      FX_CreateFontEncodingEx(cFont, FXFM_ENCODING_NONE);
+      FX_CreateFontEncodingEx(cFont);
 
-  size_t length = text.GetLength();
-  uint32_t* pCharCode = FX_Alloc(uint32_t, text.GetLength());
+  const size_t length = text.GetLength();
+  std::vector<uint32_t> charcodes(length);
   float charWidth = 0;
-  for (size_t j = 0; j < length; j++) {
-    pCharCode[j] = encoding->CharCodeFromUnicode(text[j]);
-    int32_t glyp_code = encoding->GlyphFromCharCode(pCharCode[j]);
-    int32_t glyp_value = cFont->GetGlyphWidth(glyp_code);
-    float temp = (float)((glyp_value)*fontSize / 1000.0);
+  for (size_t i = 0; i < length; ++i) {
+    charcodes[i] = encoding->CharCodeFromUnicode(text[i]);
+    int32_t glyph_code = encoding->GlyphFromCharCode(charcodes[i]);
+    uint32_t glyph_value = cFont->GetGlyphWidth(glyph_code);
+    float temp = glyph_value * fontSize / 1000.0;
     charWidth += temp;
   }
   charsLen = charWidth;
@@ -157,29 +132,28 @@ void CBC_OneDimWriter::CalcTextInfo(const ByteString& text,
   float left = leftPositon;
   float top = 0.0;
   charPos[0].m_Origin = CFX_PointF(penX + left, penY + top);
-  charPos[0].m_GlyphIndex = encoding->GlyphFromCharCode(pCharCode[0]);
+  charPos[0].m_GlyphIndex = encoding->GlyphFromCharCode(charcodes[0]);
   charPos[0].m_FontCharWidth = cFont->GetGlyphWidth(charPos[0].m_GlyphIndex);
-#if _FX_PLATFORM_ == _FX_PLATFORM_APPLE_
+#if defined(OS_MACOSX)
   charPos[0].m_ExtGID = charPos[0].m_GlyphIndex;
 #endif
   penX += (float)(charPos[0].m_FontCharWidth) * (float)fontSize / 1000.0f;
   for (size_t i = 1; i < length; i++) {
     charPos[i].m_Origin = CFX_PointF(penX + left, penY + top);
-    charPos[i].m_GlyphIndex = encoding->GlyphFromCharCode(pCharCode[i]);
+    charPos[i].m_GlyphIndex = encoding->GlyphFromCharCode(charcodes[i]);
     charPos[i].m_FontCharWidth = cFont->GetGlyphWidth(charPos[i].m_GlyphIndex);
-#if _FX_PLATFORM_ == _FX_PLATFORM_APPLE_
+#if defined(OS_MACOSX)
     charPos[i].m_ExtGID = charPos[i].m_GlyphIndex;
 #endif
     penX += (float)(charPos[i].m_FontCharWidth) * (float)fontSize / 1000.0f;
   }
-  FX_Free(pCharCode);
 }
 
 void CBC_OneDimWriter::ShowDeviceChars(CFX_RenderDevice* device,
                                        const CFX_Matrix* matrix,
                                        const ByteString str,
                                        float geWidth,
-                                       FXTEXT_CHARPOS* pCharPos,
+                                       TextCharPos* pCharPos,
                                        float locX,
                                        float locY,
                                        int32_t barWidth) {
@@ -191,7 +165,7 @@ void CBC_OneDimWriter::ShowDeviceChars(CFX_RenderDevice* device,
     rect.right -= 1;
   }
   FX_RECT re = matrix->TransformRect(rect).GetOuterRect();
-  device->FillRect(&re, m_backgroundColor);
+  device->FillRect(re, m_backgroundColor);
   CFX_Matrix affine_matrix(1.0, 0.0, 0.0, -1.0, (float)locX,
                            (float)(locY + iFontSize));
   if (matrix) {
@@ -202,7 +176,7 @@ void CBC_OneDimWriter::ShowDeviceChars(CFX_RenderDevice* device,
                          m_fontColor, FXTEXT_CLEARTYPE);
 }
 
-bool CBC_OneDimWriter::ShowChars(const WideStringView& contents,
+bool CBC_OneDimWriter::ShowChars(WideStringView contents,
                                  CFX_RenderDevice* device,
                                  const CFX_Matrix* matrix,
                                  int32_t barWidth,
@@ -211,8 +185,7 @@ bool CBC_OneDimWriter::ShowChars(const WideStringView& contents,
     return false;
 
   ByteString str = FX_UTF8Encode(contents);
-  int32_t iLen = str.GetLength();
-  std::vector<FXTEXT_CHARPOS> charpos(iLen);
+  std::vector<TextCharPos> charpos(str.GetLength());
   float charsLen = 0;
   float geWidth = 0;
   if (m_locTextLoc == BC_TEXT_LOC_ABOVEEMBED ||
@@ -261,7 +234,7 @@ bool CBC_OneDimWriter::ShowChars(const WideStringView& contents,
 
 bool CBC_OneDimWriter::RenderDeviceResult(CFX_RenderDevice* device,
                                           const CFX_Matrix* matrix,
-                                          const WideStringView& contents) {
+                                          WideStringView contents) {
   if (m_output.empty())
     return false;
 
@@ -274,7 +247,7 @@ bool CBC_OneDimWriter::RenderDeviceResult(CFX_RenderDevice* device,
   CFX_Matrix scaledMatrix(m_outputHScale, 0.0, 0.0,
                           static_cast<float>(m_Height), 0.0, 0.0);
   scaledMatrix.Concat(*matrix);
-  for (auto& rect : m_output) {
+  for (const auto& rect : m_output) {
     CFX_GraphStateData data;
     device->DrawPath(&rect, &scaledMatrix, &data, m_barColor, 0,
                      FXFILL_WINDING);
@@ -284,7 +257,7 @@ bool CBC_OneDimWriter::RenderDeviceResult(CFX_RenderDevice* device,
          ShowChars(contents, device, matrix, m_barWidth, m_multiple);
 }
 
-bool CBC_OneDimWriter::RenderResult(const WideStringView& contents,
+bool CBC_OneDimWriter::RenderResult(WideStringView contents,
                                     uint8_t* code,
                                     int32_t codeLength) {
   if (codeLength < 1)
@@ -299,17 +272,12 @@ bool CBC_OneDimWriter::RenderResult(const WideStringView& contents,
   m_outputHScale =
       m_Width > 0 ? static_cast<float>(m_Width) / static_cast<float>(codeLength)
                   : 1.0;
-  float dataLengthScale = 1.0;
-  if (m_iDataLenth > 0 && contents.GetLength() != 0)
-    dataLengthScale = float(contents.GetLength()) / float(m_iDataLenth);
-  if (m_iDataLenth > 0 && contents.GetLength() == 0)
-    dataLengthScale = float(1) / float(m_iDataLenth);
   m_multiple = 1;
-  const int32_t outputHeight = 1;
   const int32_t outputWidth = codeLength;
   m_barWidth = m_Width;
 
   m_output.clear();
+  m_output.reserve(codeOldLength * m_multiple);
   for (int32_t inputX = 0, outputX = leftPadding * m_multiple;
        inputX < codeOldLength; ++inputX, outputX += m_multiple) {
     if (code[inputX] != 1)
@@ -319,27 +287,23 @@ bool CBC_OneDimWriter::RenderResult(const WideStringView& contents,
       return true;
 
     if (outputX + m_multiple > outputWidth && outputWidth - outputX > 0) {
-      RenderVerticalBars(outputX, outputWidth - outputX, outputHeight);
+      RenderVerticalBars(outputX, outputWidth - outputX);
       return true;
     }
 
-    RenderVerticalBars(outputX, m_multiple, outputHeight);
+    RenderVerticalBars(outputX, m_multiple);
   }
   return true;
 }
 
-void CBC_OneDimWriter::RenderVerticalBars(int32_t outputX,
-                                          int32_t width,
-                                          int32_t height) {
+void CBC_OneDimWriter::RenderVerticalBars(int32_t outputX, int32_t width) {
   for (int i = 0; i < width; ++i) {
     float x = outputX + i;
-    CFX_PathData rect;
-    rect.AppendRect(x, 0.0f, x + 1, static_cast<float>(height));
-    m_output.push_back(rect);
+    m_output.emplace_back();
+    m_output.back().AppendRect(x, 0.0f, x + 1, 1.0f);
   }
 }
 
-WideString CBC_OneDimWriter::RenderTextContents(
-    const WideStringView& contents) {
+WideString CBC_OneDimWriter::RenderTextContents(WideStringView contents) {
   return WideString();
 }
