@@ -8,28 +8,19 @@
 
 #include <utility>
 
-#include "fpdfsdk/cpdfsdk_interform.h"
+#include "fpdfsdk/cpdfsdk_interactiveform.h"
 #include "fxjs/cjs_document.h"
 #include "fxjs/cjs_timerobj.h"
 #include "fxjs/global_timer.h"
 #include "fxjs/ijs_event_context.h"
 #include "fxjs/js_resources.h"
+#include "third_party/base/ptr_util.h"
+#include "third_party/base/stl_util.h"
 
 #ifdef PDF_ENABLE_XFA
 #include "fpdfsdk/fpdfxfa/cpdfxfa_context.h"
 #endif  // PDF_ENABLE_XFA
 
-namespace {
-
-bool IsTypeKnown(v8::Local<v8::Value> value) {
-  return !value.IsEmpty() &&
-         (value->IsString() || value->IsNumber() || value->IsBoolean() ||
-          value->IsDate() || value->IsObject() || value->IsNull() ||
-          value->IsUndefined());
-}
-
-}  // namespace
-
 #define JS_STR_VIEWERTYPE L"pdfium"
 #define JS_STR_VIEWERVARIATION L"Full"
 #define JS_STR_PLATFORM L"WIN"
@@ -104,11 +95,10 @@ CJS_App::~CJS_App() = default;
 CJS_Result CJS_App::get_active_docs(CJS_Runtime* pRuntime) {
   v8::Local<v8::Object> pObj = pRuntime->GetThisObj();
   auto pJSDocument = JSGetObject<CJS_Document>(pObj);
+  if (!pJSDocument)
+    return CJS_Result::Failure(JSMessage::kObjectTypeError);
   v8::Local<v8::Array> aDocs = pRuntime->NewArray();
-  pRuntime->PutArrayElement(
-      aDocs, 0,
-      pJSDocument ? v8::Local<v8::Value>(pJSDocument->ToV8Object())
-                  : v8::Local<v8::Value>());
+  pRuntime->PutArrayElement(aDocs, 0, pJSDocument->ToV8Object());
   if (pRuntime->GetArrayLength(aDocs) > 0)
     return CJS_Result::Success(aDocs);
 
@@ -127,7 +117,8 @@ CJS_Result CJS_App::get_calculate(CJS_Runtime* pRuntime) {
 CJS_Result CJS_App::set_calculate(CJS_Runtime* pRuntime,
                                   v8::Local<v8::Value> vp) {
   m_bCalculate = pRuntime->ToBoolean(vp);
-  pRuntime->GetFormFillEnv()->GetInterForm()->EnableCalculate(m_bCalculate);
+  pRuntime->GetFormFillEnv()->GetInteractiveForm()->EnableCalculate(
+      m_bCalculate);
   return CJS_Result::Success();
 }
 
@@ -232,9 +223,9 @@ CJS_Result CJS_App::openFDF(CJS_Runtime* pRuntime,
 CJS_Result CJS_App::alert(CJS_Runtime* pRuntime,
                           const std::vector<v8::Local<v8::Value>>& params) {
   std::vector<v8::Local<v8::Value>> newParams = ExpandKeywordParams(
-      pRuntime, params, 4, L"cMsg", L"nIcon", L"nType", L"cTitle");
+      pRuntime, params, 4, "cMsg", "nIcon", "nType", "cTitle");
 
-  if (!IsTypeKnown(newParams[0]))
+  if (!IsExpandedParamKnown(newParams[0]))
     return CJS_Result::Failure(JSMessage::kParamError);
 
   CPDFSDK_FormFillEnvironment* pFormFillEnv = pRuntime->GetFormFillEnv();
@@ -257,15 +248,15 @@ CJS_Result CJS_App::alert(CJS_Runtime* pRuntime,
   }
 
   int iIcon = JSPLATFORM_ALERT_ICON_DEFAULT;
-  if (IsTypeKnown(newParams[1]))
+  if (IsExpandedParamKnown(newParams[1]))
     iIcon = pRuntime->ToInt32(newParams[1]);
 
   int iType = JSPLATFORM_ALERT_BUTTON_DEFAULT;
-  if (IsTypeKnown(newParams[2]))
+  if (IsExpandedParamKnown(newParams[2]))
     iType = pRuntime->ToInt32(newParams[2]);
 
   WideString swTitle;
-  if (IsTypeKnown(newParams[3]))
+  if (IsExpandedParamKnown(newParams[3]))
     swTitle = pRuntime->ToWideString(newParams[3]);
   else
     swTitle = JSGetStringFromID(JSMessage::kAlert);
@@ -285,7 +276,7 @@ CJS_Result CJS_App::beep(CJS_Runtime* pRuntime,
     return CJS_Result::Failure(JSMessage::kParamError);
 
   int type = JSPLATFORM_BEEP_DEFAULT;
-  if (IsTypeKnown(params[0]))
+  if (IsExpandedParamKnown(params[0]))
     type = pRuntime->ToInt32(params[0]);
 
   pRuntime->GetFormFillEnv()->JS_appBeep(type);
@@ -324,7 +315,8 @@ CJS_Result CJS_App::setInterval(
 
   uint32_t dwInterval = params.size() > 1 ? pRuntime->ToInt32(params[1]) : 1000;
   auto timerRef = pdfium::MakeUnique<GlobalTimer>(
-      this, pRuntime->GetFormFillEnv(), pRuntime, 0, script, dwInterval, 0);
+      this, pRuntime->GetFormFillEnv(), pRuntime, GlobalTimer::Type::kRepeating,
+      script, dwInterval, 0);
   GlobalTimer* pTimerRef = timerRef.get();
   m_Timers.insert(std::move(timerRef));
 
@@ -352,8 +344,8 @@ CJS_Result CJS_App::setTimeOut(
 
   uint32_t dwTimeOut = params.size() > 1 ? pRuntime->ToInt32(params[1]) : 1000;
   auto timerRef = pdfium::MakeUnique<GlobalTimer>(
-      this, pRuntime->GetFormFillEnv(), pRuntime, 1, script, dwTimeOut,
-      dwTimeOut);
+      this, pRuntime->GetFormFillEnv(), pRuntime, GlobalTimer::Type::kOneShot,
+      script, dwTimeOut, dwTimeOut);
   GlobalTimer* pTimerRef = timerRef.get();
   m_Timers.insert(std::move(timerRef));
 
@@ -405,12 +397,6 @@ void CJS_App::ClearTimerCommon(CJS_Runtime* pRuntime,
 CJS_Result CJS_App::execMenuItem(
     CJS_Runtime* pRuntime,
     const std::vector<v8::Local<v8::Value>>& params) {
-  if (params.size() > 0 && IsTypeKnown(params[0])) {
-    WideString item = pRuntime->ToWideString(params[0]);
-    if (item == L"SaveAs" && pRuntime->GetFormFillEnv())
-      pRuntime->GetFormFillEnv()->SaveCalled();
-  }
-
   return CJS_Result::Failure(JSMessage::kNotSupportedError);
 }
 
@@ -447,16 +433,15 @@ CJS_Result CJS_App::goForward(CJS_Runtime* pRuntime,
 
 CJS_Result CJS_App::mailMsg(CJS_Runtime* pRuntime,
                             const std::vector<v8::Local<v8::Value>>& params) {
-  std::vector<v8::Local<v8::Value>> newParams =
-      ExpandKeywordParams(pRuntime, params, 6, L"bUI", L"cTo", L"cCc", L"cBcc",
-                          L"cSubject", L"cMsg");
+  std::vector<v8::Local<v8::Value>> newParams = ExpandKeywordParams(
+      pRuntime, params, 6, "bUI", "cTo", "cCc", "cBcc", "cSubject", "cMsg");
 
-  if (!IsTypeKnown(newParams[0]))
+  if (!IsExpandedParamKnown(newParams[0]))
     return CJS_Result::Failure(JSMessage::kParamError);
 
   bool bUI = pRuntime->ToBoolean(newParams[0]);
   WideString cTo;
-  if (IsTypeKnown(newParams[1])) {
+  if (IsExpandedParamKnown(newParams[1])) {
     cTo = pRuntime->ToWideString(newParams[1]);
   } else {
     // cTo parameter required when UI not invoked.
@@ -465,19 +450,19 @@ CJS_Result CJS_App::mailMsg(CJS_Runtime* pRuntime,
   }
 
   WideString cCc;
-  if (IsTypeKnown(newParams[2]))
+  if (IsExpandedParamKnown(newParams[2]))
     cCc = pRuntime->ToWideString(newParams[2]);
 
   WideString cBcc;
-  if (IsTypeKnown(newParams[3]))
+  if (IsExpandedParamKnown(newParams[3]))
     cBcc = pRuntime->ToWideString(newParams[3]);
 
   WideString cSubject;
-  if (IsTypeKnown(newParams[4]))
+  if (IsExpandedParamKnown(newParams[4]))
     cSubject = pRuntime->ToWideString(newParams[4]);
 
   WideString cMsg;
-  if (IsTypeKnown(newParams[5]))
+  if (IsExpandedParamKnown(newParams[5]))
     cMsg = pRuntime->ToWideString(newParams[5]);
 
   pRuntime->BeginBlock();
@@ -546,27 +531,27 @@ CJS_Result CJS_App::openDoc(CJS_Runtime* pRuntime,
 CJS_Result CJS_App::response(CJS_Runtime* pRuntime,
                              const std::vector<v8::Local<v8::Value>>& params) {
   std::vector<v8::Local<v8::Value>> newParams =
-      ExpandKeywordParams(pRuntime, params, 5, L"cQuestion", L"cTitle",
-                          L"cDefault", L"bPassword", L"cLabel");
+      ExpandKeywordParams(pRuntime, params, 5, "cQuestion", "cTitle",
+                          "cDefault", "bPassword", "cLabel");
 
-  if (!IsTypeKnown(newParams[0]))
+  if (!IsExpandedParamKnown(newParams[0]))
     return CJS_Result::Failure(JSMessage::kParamError);
 
   WideString swQuestion = pRuntime->ToWideString(newParams[0]);
   WideString swTitle = L"PDF";
-  if (IsTypeKnown(newParams[1]))
+  if (IsExpandedParamKnown(newParams[1]))
     swTitle = pRuntime->ToWideString(newParams[1]);
 
   WideString swDefault;
-  if (IsTypeKnown(newParams[2]))
+  if (IsExpandedParamKnown(newParams[2]))
     swDefault = pRuntime->ToWideString(newParams[2]);
 
   bool bPassword = false;
-  if (IsTypeKnown(newParams[3]))
+  if (IsExpandedParamKnown(newParams[3]))
     bPassword = pRuntime->ToBoolean(newParams[3]);
 
   WideString swLabel;
-  if (IsTypeKnown(newParams[4]))
+  if (IsExpandedParamKnown(newParams[4]))
     swLabel = pRuntime->ToWideString(newParams[4]);
 
   const int MAX_INPUT_BYTES = 2048;
