@@ -12,9 +12,11 @@
 
 #include "core/fxcrt/cfx_memorystream.h"
 #include "core/fxcrt/fx_codepage.h"
-#include "fxjs/cfxjse_engine.h"
-#include "fxjs/cfxjse_value.h"
+#include "core/fxcrt/xml/cfx_xmldocument.h"
+#include "core/fxcrt/xml/cfx_xmlnode.h"
 #include "fxjs/js_resources.h"
+#include "fxjs/xfa/cfxjse_engine.h"
+#include "fxjs/xfa/cfxjse_value.h"
 #include "third_party/base/ptr_util.h"
 #include "xfa/fxfa/cxfa_eventparam.h"
 #include "xfa/fxfa/cxfa_ffdoc.h"
@@ -22,11 +24,12 @@
 #include "xfa/fxfa/parser/cxfa_document.h"
 #include "xfa/fxfa/parser/cxfa_document_parser.h"
 #include "xfa/fxfa/parser/cxfa_node.h"
+#include "xfa/fxfa/parser/xfa_basic_data.h"
 #include "xfa/fxfa/parser/xfa_utils.h"
 
 namespace {
 
-enum class EventAppliesToo {
+enum class EventAppliesToo : uint8_t {
   kNone = 0,
   kAll = 1,
   kAllNonRecursive = 2,
@@ -39,33 +42,75 @@ enum class EventAppliesToo {
 
 struct XFA_ExecEventParaInfo {
  public:
-  uint32_t m_uHash;
-  const wchar_t* m_lpcEventName;
+  uint32_t m_uHash;  // hashed as wide string.
   XFA_EVENTTYPE m_eventType;
   EventAppliesToo m_validFlags;
 };
 
+#undef PARA
+#define PARA(a, b, c, d) a, c, d
 const XFA_ExecEventParaInfo gs_eventParaInfos[] = {
-    {0x109d7ce7, L"mouseEnter", XFA_EVENT_MouseEnter, EventAppliesToo::kField},
-    {0x1bfc72d9, L"preOpen", XFA_EVENT_PreOpen, EventAppliesToo::kChoiceList},
-    {0x2196a452, L"initialize", XFA_EVENT_Initialize, EventAppliesToo::kAll},
-    {0x27410f03, L"mouseExit", XFA_EVENT_MouseExit, EventAppliesToo::kField},
-    {0x36f1c6d8, L"preSign", XFA_EVENT_PreSign, EventAppliesToo::kSignature},
-    {0x4731d6ba, L"exit", XFA_EVENT_Exit, EventAppliesToo::kAllNonRecursive},
-    {0x7233018a, L"validate", XFA_EVENT_Validate, EventAppliesToo::kAll},
-    {0x8808385e, L"indexChange", XFA_EVENT_IndexChange,
-     EventAppliesToo::kSubform},
-    {0x891f4606, L"change", XFA_EVENT_Change,
-     EventAppliesToo::kFieldOrExclusion},
-    {0x9f693b21, L"mouseDown", XFA_EVENT_MouseDown, EventAppliesToo::kField},
-    {0xcdce56b3, L"full", XFA_EVENT_Full, EventAppliesToo::kFieldOrExclusion},
-    {0xd576d08e, L"mouseUp", XFA_EVENT_MouseUp, EventAppliesToo::kField},
-    {0xd95657a6, L"click", XFA_EVENT_Click, EventAppliesToo::kFieldOrExclusion},
-    {0xdbfbe02e, L"calculate", XFA_EVENT_Calculate, EventAppliesToo::kAll},
-    {0xe25fa7b8, L"postOpen", XFA_EVENT_PostOpen, EventAppliesToo::kChoiceList},
-    {0xe28dce7e, L"enter", XFA_EVENT_Enter, EventAppliesToo::kAllNonRecursive},
-    {0xfd54fbb7, L"postSign", XFA_EVENT_PostSign, EventAppliesToo::kSignature},
+    {PARA(0x109d7ce7,
+          "mouseEnter",
+          XFA_EVENT_MouseEnter,
+          EventAppliesToo::kField)},
+    {PARA(0x1bfc72d9,
+          "preOpen",
+          XFA_EVENT_PreOpen,
+          EventAppliesToo::kChoiceList)},
+    {PARA(0x2196a452,
+          "initialize",
+          XFA_EVENT_Initialize,
+          EventAppliesToo::kAll)},
+    {PARA(0x27410f03,
+          "mouseExit",
+          XFA_EVENT_MouseExit,
+          EventAppliesToo::kField)},
+    {PARA(0x36f1c6d8,
+          "preSign",
+          XFA_EVENT_PreSign,
+          EventAppliesToo::kSignature)},
+    {PARA(0x4731d6ba,
+          "exit",
+          XFA_EVENT_Exit,
+          EventAppliesToo::kAllNonRecursive)},
+    {PARA(0x7233018a, "validate", XFA_EVENT_Validate, EventAppliesToo::kAll)},
+    {PARA(0x8808385e,
+          "indexChange",
+          XFA_EVENT_IndexChange,
+          EventAppliesToo::kSubform)},
+    {PARA(0x891f4606,
+          "change",
+          XFA_EVENT_Change,
+          EventAppliesToo::kFieldOrExclusion)},
+    {PARA(0x9f693b21,
+          "mouseDown",
+          XFA_EVENT_MouseDown,
+          EventAppliesToo::kField)},
+    {PARA(0xcdce56b3,
+          "full",
+          XFA_EVENT_Full,
+          EventAppliesToo::kFieldOrExclusion)},
+    {PARA(0xd576d08e, "mouseUp", XFA_EVENT_MouseUp, EventAppliesToo::kField)},
+    {PARA(0xd95657a6,
+          "click",
+          XFA_EVENT_Click,
+          EventAppliesToo::kFieldOrExclusion)},
+    {PARA(0xdbfbe02e, "calculate", XFA_EVENT_Calculate, EventAppliesToo::kAll)},
+    {PARA(0xe25fa7b8,
+          "postOpen",
+          XFA_EVENT_PostOpen,
+          EventAppliesToo::kChoiceList)},
+    {PARA(0xe28dce7e,
+          "enter",
+          XFA_EVENT_Enter,
+          EventAppliesToo::kAllNonRecursive)},
+    {PARA(0xfd54fbb7,
+          "postSign",
+          XFA_EVENT_PostSign,
+          EventAppliesToo::kSignature)},
 };
+#undef PARA
 
 const XFA_ExecEventParaInfo* GetEventParaInfoByName(
     WideStringView wsEventName) {
@@ -104,6 +149,10 @@ CJX_Node::CJX_Node(CXFA_Node* node) : CJX_Tree(node) {
 
 CJX_Node::~CJX_Node() = default;
 
+bool CJX_Node::DynamicTypeIs(TypeTag eType) const {
+  return eType == static_type__ || ParentType__::DynamicTypeIs(eType);
+}
+
 CXFA_Node* CJX_Node::GetXFANode() const {
   return ToNode(GetXFAObject());
 }
@@ -134,9 +183,8 @@ CJS_Result CJX_Node::clone(CFX_V8* runtime,
 
   CXFA_Node* pCloneNode = GetXFANode()->Clone(runtime->ToBoolean(params[0]));
   CFXJSE_Value* value =
-      GetDocument()->GetScriptContext()->GetJSValueFromMap(pCloneNode);
-  if (!value)
-    return CJS_Result::Success(runtime->NewNull());
+      GetDocument()->GetScriptContext()->GetOrCreateJSBindingFromMap(
+          pCloneNode);
 
   return CJS_Result::Success(
       value->DirectGetValue().Get(runtime->GetIsolate()));
@@ -150,7 +198,7 @@ CJS_Result CJX_Node::getAttribute(
 
   WideString expression = runtime->ToWideString(params[0]);
   return CJS_Result::Success(runtime->NewString(
-      GetAttribute(expression.AsStringView()).UTF8Encode().AsStringView()));
+      GetAttribute(expression.AsStringView()).ToUTF8().AsStringView()));
 }
 
 CJS_Result CJX_Node::getElement(
@@ -161,15 +209,16 @@ CJS_Result CJX_Node::getElement(
 
   WideString expression = runtime->ToWideString(params[0]);
   int32_t iValue = params.size() >= 2 ? runtime->ToInt32(params[1]) : 0;
-  CXFA_Node* pNode = GetOrCreateProperty<CXFA_Node>(
-      iValue, CXFA_Node::NameToElement(expression));
+  XFA_Element eElement = XFA_GetElementByName(expression.AsStringView());
+  if (eElement == XFA_Element::Unknown)
+    return CJS_Result::Success(runtime->NewNull());
+
+  CXFA_Node* pNode = GetOrCreateProperty<CXFA_Node>(iValue, eElement);
   if (!pNode)
     return CJS_Result::Success(runtime->NewNull());
 
   CFXJSE_Value* value =
-      GetDocument()->GetScriptContext()->GetJSValueFromMap(pNode);
-  if (!value)
-    return CJS_Result::Success(runtime->NewNull());
+      GetDocument()->GetScriptContext()->GetOrCreateJSBindingFromMap(pNode);
 
   return CJS_Result::Success(
       value->DirectGetValue().Get(runtime->GetIsolate()));
@@ -182,18 +231,22 @@ CJS_Result CJX_Node::isPropertySpecified(
     return CJS_Result::Failure(JSMessage::kParamError);
 
   WideString expression = runtime->ToWideString(params[0]);
-  XFA_Attribute attr = CXFA_Node::NameToAttribute(expression.AsStringView());
-  if (attr != XFA_Attribute::Unknown && HasAttribute(attr))
+  Optional<XFA_ATTRIBUTEINFO> attr =
+      XFA_GetAttributeByName(expression.AsStringView());
+  if (attr.has_value() && HasAttribute(attr.value().attribute))
     return CJS_Result::Success(runtime->NewBoolean(true));
 
+  XFA_Element eType = XFA_GetElementByName(expression.AsStringView());
+  if (eType == XFA_Element::Unknown)
+    return CJS_Result::Success(runtime->NewBoolean(false));
+
   bool bParent = params.size() < 2 || runtime->ToBoolean(params[1]);
   int32_t iIndex = params.size() == 3 ? runtime->ToInt32(params[2]) : 0;
-  XFA_Element eType = CXFA_Node::NameToElement(expression);
   bool bHas = !!GetOrCreateProperty<CXFA_Node>(iIndex, eType);
   if (!bHas && bParent && GetXFANode()->GetParent()) {
     // Also check on the parent.
     auto* jsnode = GetXFANode()->GetParent()->JSObject();
-    bHas = jsnode->HasAttribute(attr) ||
+    bHas = jsnode->HasAttribute(attr.value().attribute) ||
            !!jsnode->GetOrCreateProperty<CXFA_Node>(iIndex, eType);
   }
   return CJS_Result::Success(runtime->NewBoolean(bHas));
@@ -221,8 +274,12 @@ CJS_Result CJX_Node::loadXML(CFX_V8* runtime,
   if (!pXMLNode)
     return CJS_Result::Success();
 
+  CFX_XMLDocument* top_xml_doc =
+      GetXFANode()->GetDocument()->GetNotify()->GetHDOC()->GetXMLDocument();
+  top_xml_doc->AppendNodesFrom(pParser->GetXMLDoc().get());
+
   if (bIgnoreRoot &&
-      (pXMLNode->GetType() != FX_XMLNODE_Element ||
+      (pXMLNode->GetType() != CFX_XMLNode::Type::kElement ||
        XFA_RecognizeRichText(static_cast<CFX_XMLElement*>(pXMLNode)))) {
     bIgnoreRoot = false;
   }
@@ -239,19 +296,10 @@ CJS_Result CJX_Node::loadXML(CFX_V8* runtime,
     CFX_XMLNode* pThisXMLRoot = GetXFANode()->GetXMLMappingNode();
     CFX_XMLNode* clone;
     if (pThisXMLRoot) {
-      clone = pThisXMLRoot->Clone(GetXFANode()
-                                      ->GetDocument()
-                                      ->GetNotify()
-                                      ->GetHDOC()
-                                      ->GetXMLDocument());
+      clone = pThisXMLRoot->Clone(top_xml_doc);
     } else {
-      clone = GetXFANode()
-                  ->GetDocument()
-                  ->GetNotify()
-                  ->GetHDOC()
-                  ->GetXMLDocument()
-                  ->CreateNode<CFX_XMLElement>(
-                      WideString(GetXFANode()->GetClassName()));
+      clone = top_xml_doc->CreateNode<CFX_XMLElement>(
+          WideString::FromASCII(GetXFANode()->GetClassName()));
     }
     pFakeXMLRoot = clone;
   }
@@ -338,8 +386,10 @@ CJS_Result CJX_Node::saveXML(CFX_V8* runtime,
   if (params.size() > 1)
     return CJS_Result::Failure(JSMessage::kParamError);
 
-  if (params.size() == 1 && runtime->ToWideString(params[0]) != L"pretty")
+  if (params.size() == 1 &&
+      !runtime->ToWideString(params[0]).EqualsASCII("pretty")) {
     return CJS_Result::Failure(JSMessage::kValueError);
+  }
 
   // TODO(weili): Check whether we need to save pretty print XML, pdfium:501.
 
@@ -352,7 +402,7 @@ CJS_Result CJX_Node::saveXML(CFX_V8* runtime,
   CFX_XMLNode* pElement = nullptr;
   if (GetXFANode()->GetPacketType() == XFA_PacketType::Datasets) {
     pElement = GetXFANode()->GetXMLMappingNode();
-    if (!pElement || pElement->GetType() != FX_XMLNODE_Element) {
+    if (!pElement || pElement->GetType() != CFX_XMLNode::Type::kElement) {
       return CJS_Result::Success(
           runtime->NewString(bsXMLHeader.AsStringView()));
     }
@@ -379,8 +429,12 @@ CJS_Result CJX_Node::setAttribute(
   if (params.size() != 2)
     return CJS_Result::Failure(JSMessage::kParamError);
 
+  // Note: yes, arglist is spec'd absolutely backwards from what any sane
+  // person would do, namely value first, attribute second.
   WideString attributeValue = runtime->ToWideString(params[0]);
   WideString attribute = runtime->ToWideString(params[1]);
+
+  // Pass them to our method, however, in the more usual manner.
   SetAttribute(attribute.AsStringView(), attributeValue.AsStringView(), true);
   return CJS_Result::Success();
 }
@@ -395,12 +449,6 @@ CJS_Result CJX_Node::setElement(
   return CJS_Result::Success();
 }
 
-void CJX_Node::id(CFXJSE_Value* pValue,
-                  bool bSetting,
-                  XFA_Attribute eAttribute) {
-  Script_Attribute_String(pValue, bSetting, eAttribute);
-}
-
 void CJX_Node::ns(CFXJSE_Value* pValue,
                   bool bSetting,
                   XFA_Attribute eAttribute) {
@@ -409,7 +457,7 @@ void CJX_Node::ns(CFXJSE_Value* pValue,
     return;
   }
   pValue->SetString(
-      TryNamespace().value_or(WideString()).UTF8Encode().AsStringView());
+      TryNamespace().value_or(WideString()).ToUTF8().AsStringView());
 }
 
 void CJX_Node::model(CFXJSE_Value* pValue,
@@ -419,7 +467,7 @@ void CJX_Node::model(CFXJSE_Value* pValue,
     ThrowInvalidPropertyException();
     return;
   }
-  pValue->Assign(GetDocument()->GetScriptContext()->GetJSValueFromMap(
+  pValue->Assign(GetDocument()->GetScriptContext()->GetOrCreateJSBindingFromMap(
       GetXFANode()->GetModelNode()));
 }
 
@@ -458,12 +506,13 @@ void CJX_Node::oneOfChild(CFXJSE_Value* pValue,
   std::vector<CXFA_Node*> properties = GetXFANode()->GetNodeList(
       XFA_NODEFILTER_OneOfProperty, XFA_Element::Unknown);
   if (!properties.empty()) {
-    pValue->Assign(GetDocument()->GetScriptContext()->GetJSValueFromMap(
-        properties.front()));
+    pValue->Assign(
+        GetDocument()->GetScriptContext()->GetOrCreateJSBindingFromMap(
+            properties.front()));
   }
 }
 
-int32_t CJX_Node::execSingleEventByName(const WideStringView& wsEventName,
+int32_t CJX_Node::execSingleEventByName(WideStringView wsEventName,
                                         XFA_Element eType) {
   CXFA_FFNotify* pNotify = GetDocument()->GetNotify();
   if (!pNotify)
