@@ -5,7 +5,7 @@
  *                            | (__| |_| |  _ <| |___
  *                             \___|\___/|_| \_\_____|
  *
- * Copyright (C) 1998 - 2018, Daniel Stenberg, <daniel@haxx.se>, et al.
+ * Copyright (C) 1998 - 2019, Daniel Stenberg, <daniel@haxx.se>, et al.
  *
  * This software is licensed as described in the file COPYING, which
  * you should have received as part of this distribution. The terms
@@ -94,7 +94,7 @@ iconv_open_CCSID(unsigned int ccsidout, unsigned int ccsidin,
 
   makeOS400IconvCode(fromcode, ccsidin);
   makeOS400IconvCode(tocode, ccsidout);
-  memset(tocode + 13, 0, sizeof tocode - 13);   /* Dest. code id format. */
+  memset(tocode + 13, 0, sizeof(tocode) - 13);   /* Dest. code id format. */
 
   if(cstr)
     fromcode[18] = '1';                         /* Set null-terminator flag. */
@@ -104,8 +104,8 @@ iconv_open_CCSID(unsigned int ccsidout, unsigned int ccsidin,
 
 
 static int
-convert(char * d, size_t dlen, int dccsid,
-        const char * s, int slen, int sccsid)
+convert(char *d, size_t dlen, int dccsid,
+        const char *s, int slen, int sccsid)
 
 {
   int i;
@@ -161,11 +161,11 @@ convert(char * d, size_t dlen, int dccsid,
 
 
 static char *
-dynconvert(int dccsid, const char * s, int slen, int sccsid)
+dynconvert(int dccsid, const char *s, int slen, int sccsid)
 
 {
-  char * d;
-  char * cp;
+  char *d;
+  char *cp;
   size_t dlen;
   int l;
   static const char nullbyte = 0;
@@ -213,19 +213,26 @@ dynconvert(int dccsid, const char * s, int slen, int sccsid)
 
 
 static struct curl_slist *
-slist_convert(int dccsid, struct curl_slist * from, int sccsid)
+slist_convert(int dccsid, struct curl_slist *from, int sccsid)
 
 {
-  struct curl_slist * to = (struct curl_slist *) NULL;
+  struct curl_slist *to = (struct curl_slist *) NULL;
 
   for(; from; from = from->next) {
-    char * cp = dynconvert(dccsid, from->data, -1, sccsid);
+    struct curl_slist *nl;
+    char *cp = dynconvert(dccsid, from->data, -1, sccsid);
 
     if(!cp) {
       curl_slist_free_all(to);
       return (struct curl_slist *) NULL;
     }
-    to = Curl_slist_append_nodup(to, cp);
+    nl = Curl_slist_append_nodup(to, cp);
+    if(!nl) {
+      curl_slist_free_all(to);
+      free(cp);
+      return NULL;
+    }
+    to = nl;
   }
   return to;
 }
@@ -236,8 +243,8 @@ curl_version_ccsid(unsigned int ccsid)
 
 {
   int i;
-  char * aversion;
-  char * eversion;
+  char *aversion;
+  char *eversion;
 
   aversion = curl_version();
 
@@ -247,7 +254,8 @@ curl_version_ccsid(unsigned int ccsid)
   i = strlen(aversion) + 1;
   i *= MAX_CONV_EXPANSION;
 
-  if(!(eversion = Curl_thread_buffer(LK_CURL_VERSION, i)))
+  eversion = Curl_thread_buffer(LK_CURL_VERSION, i);
+  if(!eversion)
     return (char *) NULL;
 
   if(convert(eversion, i, ccsid, aversion, -1, ASCII_CCSID) < 0)
@@ -258,12 +266,12 @@ curl_version_ccsid(unsigned int ccsid)
 
 
 char *
-curl_easy_escape_ccsid(CURL * handle, const char * string, int length,
+curl_easy_escape_ccsid(CURL *handle, const char *string, int length,
                        unsigned int sccsid, unsigned int dccsid)
 
 {
-  char * s;
-  char * d;
+  char *s;
+  char *d;
 
   if(!string) {
     errno = EINVAL;
@@ -288,13 +296,13 @@ curl_easy_escape_ccsid(CURL * handle, const char * string, int length,
 
 
 char *
-curl_easy_unescape_ccsid(CURL * handle, const char * string, int length,
-                         int * outlength,
+curl_easy_unescape_ccsid(CURL *handle, const char *string, int length,
+                         int *outlength,
                          unsigned int sccsid, unsigned int dccsid)
 
 {
-  char * s;
-  char * d;
+  char *s;
+  char *d;
 
   if(!string) {
     errno = EINVAL;
@@ -323,11 +331,11 @@ curl_easy_unescape_ccsid(CURL * handle, const char * string, int length,
 
 
 struct curl_slist *
-curl_slist_append_ccsid(struct curl_slist * list,
-                        const char * data, unsigned int ccsid)
+curl_slist_append_ccsid(struct curl_slist *list,
+                        const char *data, unsigned int ccsid)
 
 {
-  char * s;
+  char *s;
 
   s = (char *) NULL;
 
@@ -346,10 +354,10 @@ curl_slist_append_ccsid(struct curl_slist * list,
 
 
 time_t
-curl_getdate_ccsid(const char * p, const time_t * unused, unsigned int ccsid)
+curl_getdate_ccsid(const char *p, const time_t * unused, unsigned int ccsid)
 
 {
-  char * s;
+  char *s;
   time_t t;
 
   if(!p)
@@ -368,7 +376,7 @@ curl_getdate_ccsid(const char * p, const time_t * unused, unsigned int ccsid)
 
 static int
 convert_version_info_string(const char * * stringp,
-                            char * * bufp, int * left, unsigned int ccsid)
+                            char * * bufp, int *left, unsigned int ccsid)
 
 {
   /* Helper for curl_version_info_ccsid(): convert a string if defined.
@@ -396,7 +404,7 @@ curl_version_info_ccsid(CURLversion stamp, unsigned int ccsid)
 
 {
   curl_version_info_data * p;
-  char * cp;
+  char *cp;
   int n;
   int nproto;
   curl_version_info_data * id;
@@ -461,17 +469,17 @@ curl_version_info_ccsid(CURLversion stamp, unsigned int ccsid)
 
   cp = Curl_thread_buffer(LK_VERSION_INFO_DATA, n);
   id = (curl_version_info_data *) Curl_thread_buffer(LK_VERSION_INFO,
-                                                     sizeof *id);
+                                                     sizeof(*id));
 
   if(!id || !cp)
     return (curl_version_info_data *) NULL;
 
   /* Copy data and convert strings. */
 
-  memcpy((char *) id, (char *) p, sizeof *p);
+  memcpy((char *) id, (char *) p, sizeof(*p));
 
   if(id->protocols) {
-    int i = nproto * sizeof id->protocols[0];
+    int i = nproto * sizeof(id->protocols[0]);
 
     id->protocols = (const char * const *) cp;
     memcpy(cp, (char *) p->protocols, i);
@@ -514,8 +522,8 @@ curl_easy_strerror_ccsid(CURLcode error, unsigned int ccsid)
 
 {
   int i;
-  const char * s;
-  char * buf;
+  const char *s;
+  char *buf;
 
   s = curl_easy_strerror(error);
 
@@ -524,7 +532,8 @@ curl_easy_strerror_ccsid(CURLcode error, unsigned int ccsid)
 
   i = MAX_CONV_EXPANSION * (strlen(s) + 1);
 
-  if(!(buf = Curl_thread_buffer(LK_EASY_STRERROR, i)))
+  buf = Curl_thread_buffer(LK_EASY_STRERROR, i);
+  if(!buf)
     return (const char *) NULL;
 
   if(convert(buf, i, ccsid, s, -1, ASCII_CCSID) < 0)
@@ -539,8 +548,8 @@ curl_share_strerror_ccsid(CURLSHcode error, unsigned int ccsid)
 
 {
   int i;
-  const char * s;
-  char * buf;
+  const char *s;
+  char *buf;
 
   s = curl_share_strerror(error);
 
@@ -549,7 +558,8 @@ curl_share_strerror_ccsid(CURLSHcode error, unsigned int ccsid)
 
   i = MAX_CONV_EXPANSION * (strlen(s) + 1);
 
-  if(!(buf = Curl_thread_buffer(LK_SHARE_STRERROR, i)))
+  buf = Curl_thread_buffer(LK_SHARE_STRERROR, i);
+  if(!buf)
     return (const char *) NULL;
 
   if(convert(buf, i, ccsid, s, -1, ASCII_CCSID) < 0)
@@ -564,8 +574,8 @@ curl_multi_strerror_ccsid(CURLMcode error, unsigned int ccsid)
 
 {
   int i;
-  const char * s;
-  char * buf;
+  const char *s;
+  char *buf;
 
   s = curl_multi_strerror(error);
 
@@ -574,7 +584,8 @@ curl_multi_strerror_ccsid(CURLMcode error, unsigned int ccsid)
 
   i = MAX_CONV_EXPANSION * (strlen(s) + 1);
 
-  if(!(buf = Curl_thread_buffer(LK_MULTI_STRERROR, i)))
+  buf = Curl_thread_buffer(LK_MULTI_STRERROR, i);
+  if(!buf)
     return (const char *) NULL;
 
   if(convert(buf, i, ccsid, s, -1, ASCII_CCSID) < 0)
@@ -603,11 +614,11 @@ curl_certinfo_free_all(struct curl_certinfo *info)
 
 
 CURLcode
-curl_easy_getinfo_ccsid(CURL * curl, CURLINFO info, ...)
+curl_easy_getinfo_ccsid(CURL *curl, CURLINFO info, ...)
 
 {
   va_list arg;
-  void * paramp;
+  void *paramp;
   CURLcode ret;
   unsigned int ccsid;
   char * * cpp;
@@ -625,7 +636,7 @@ curl_easy_getinfo_ccsid(CURL * curl, CURLINFO info, ...)
   ret = Curl_getinfo(data, info, paramp);
 
   if(ret == CURLE_OK)
-    switch ((int) info & CURLINFO_TYPEMASK) {
+    switch((int) info & CURLINFO_TYPEMASK) {
 
     case CURLINFO_STRING:
       ccsid = va_arg(arg, unsigned int);
@@ -642,16 +653,17 @@ curl_easy_getinfo_ccsid(CURL * curl, CURLINFO info, ...)
 
     case CURLINFO_SLIST:
       ccsid = va_arg(arg, unsigned int);
-      switch (info) {
+      switch(info) {
       case CURLINFO_CERTINFO:
         cipf = *(struct curl_certinfo * *) paramp;
         if(cipf) {
-          if(!(cipt = (struct curl_certinfo *) malloc(sizeof *cipt)))
+          cipt = (struct curl_certinfo *) malloc(sizeof(*cipt));
+          if(!cipt)
             ret = CURLE_OUT_OF_MEMORY;
           else {
-            cipt->certinfo = (struct curl_slist * *)
-                             calloc(cipf->num_of_certs +
-                                    1, sizeof(struct curl_slist *));
+            cipt->certinfo = (struct curl_slist **)
+              calloc(cipf->num_of_certs +
+                     1, sizeof(struct curl_slist *));
             if(!cipt->certinfo)
               ret = CURLE_OUT_OF_MEMORY;
             else {
@@ -685,10 +697,12 @@ curl_easy_getinfo_ccsid(CURL * curl, CURLINFO info, ...)
         break;
 
       default:
-        slp = (struct curl_slist * *) paramp;
-        if(*slp)
-          if(!(*slp = slist_convert(ccsid, *slp, ASCII_CCSID)))
+        slp = (struct curl_slist **) paramp;
+        if(*slp) {
+          *slp = slist_convert(ccsid, *slp, ASCII_CCSID);
+          if(!*slp)
             ret = CURLE_OUT_OF_MEMORY;
+        }
         break;
       }
     }
@@ -702,7 +716,7 @@ static int
 Curl_is_formadd_string(CURLformoption option)
 
 {
-  switch (option) {
+  switch(option) {
 
   case CURLFORM_FILENAME:
   case CURLFORM_CONTENTTYPE:
@@ -738,8 +752,8 @@ Curl_formadd_convert(struct curl_forms * forms,
 
 {
   int l;
-  char * cp;
-  char * cp2;
+  char *cp;
+  char *cp2;
 
   if(formx < 0 || !forms[formx].value)
     return 0;
@@ -788,7 +802,7 @@ curl_formadd_ccsid(struct curl_httppost * * httppost,
   struct curl_forms * lforms;
   struct curl_forms * tforms;
   unsigned int lformlen;
-  const char * value;
+  const char *value;
   unsigned int ccsid;
   int nargs;
   int namex;
@@ -816,7 +830,7 @@ curl_formadd_ccsid(struct curl_httppost * * httppost,
   /* Allocate the local curl_forms array. */
 
   lformlen = ALLOC_GRANULE;
-  lforms = malloc(lformlen * sizeof *lforms);
+  lforms = malloc(lformlen * sizeof(*lforms));
 
   if(!lforms)
     return CURL_FORMADD_MEMORY;
@@ -838,7 +852,7 @@ curl_formadd_ccsid(struct curl_httppost * * httppost,
 
     if(nargs >= lformlen) {
       lformlen += ALLOC_GRANULE;
-      tforms = realloc(lforms, lformlen * sizeof *lforms);
+      tforms = realloc(lforms, lformlen * sizeof(*lforms));
 
       if(!tforms) {
         result = CURL_FORMADD_MEMORY;
@@ -868,7 +882,7 @@ curl_formadd_ccsid(struct curl_httppost * * httppost,
 
     /* Dispatch by option. */
 
-    switch (option) {
+    switch(option) {
 
     case CURLFORM_END:
       forms = (struct curl_forms *) NULL;       /* Leave array mode. */
@@ -1058,11 +1072,11 @@ typedef struct {
 
 
 static size_t
-Curl_formget_callback_ccsid(void * arg, const char * buf, size_t len)
+Curl_formget_callback_ccsid(void *arg, const char *buf, size_t len)
 
 {
   cfcdata * p;
-  char * b;
+  char *b;
   int l;
   size_t ret;
 
@@ -1090,7 +1104,7 @@ Curl_formget_callback_ccsid(void * arg, const char * buf, size_t len)
 
 
 int
-curl_formget_ccsid(struct curl_httppost * form, void * arg,
+curl_formget_ccsid(struct curl_httppost *form, void *arg,
                    curl_formget_callback append, unsigned int ccsid)
 
 {
@@ -1104,14 +1118,14 @@ curl_formget_ccsid(struct curl_httppost * form, void * arg,
 
 
 CURLcode
-curl_easy_setopt_ccsid(CURL * curl, CURLoption tag, ...)
+curl_easy_setopt_ccsid(CURL *curl, CURLoption tag, ...)
 
 {
   CURLcode result;
   va_list arg;
-  struct Curl_easy * data;
-  char * s;
-  char * cp;
+  struct Curl_easy *data;
+  char *s;
+  char *cp;
   unsigned int ccsid;
   curl_off_t pfsize;
   static char testwarn = 1;
@@ -1125,18 +1139,24 @@ curl_easy_setopt_ccsid(CURL * curl, CURLoption tag, ...)
   if(testwarn) {
     testwarn = 0;
 
-    if((int) STRING_LASTZEROTERMINATED != (int) STRING_TARGET + 1 ||
+    if(
+#ifdef USE_ALTSVC
+       (int) STRING_LASTZEROTERMINATED != (int) STRING_ALTSVC + 1 ||
+#else
+       (int) STRING_LASTZEROTERMINATED != (int) STRING_DOH + 1 ||
+#endif
        (int) STRING_LAST != (int) STRING_COPYPOSTFIELDS + 1)
       curl_mfprintf(stderr,
        "*** WARNING: curl_easy_setopt_ccsid() should be reworked ***\n");
-    }
+  }
 
   data = (struct Curl_easy *) curl;
   va_start(arg, tag);
 
-  switch (tag) {
+  switch(tag) {
 
   case CURLOPT_ABSTRACT_UNIX_SOCKET:
+  case CURLOPT_ALTSVC:
   case CURLOPT_CAINFO:
   case CURLOPT_CAPATH:
   case CURLOPT_COOKIE:
@@ -1147,6 +1167,7 @@ curl_easy_setopt_ccsid(CURL * curl, CURLoption tag, ...)
   case CURLOPT_CUSTOMREQUEST:
   case CURLOPT_DEFAULT_PROTOCOL:
   case CURLOPT_DNS_SERVERS:
+  case CURLOPT_DOH_URL:
   case CURLOPT_EGDSOCKET:
   case CURLOPT_ENCODING:
   case CURLOPT_FTPPORT:
@@ -1221,8 +1242,8 @@ curl_easy_setopt_ccsid(CURL * curl, CURLoption tag, ...)
       if(!s) {
         result = CURLE_OUT_OF_MEMORY;
         break;
-        }
       }
+    }
 
     result = curl_easy_setopt(curl, tag, s);
     free(s);
@@ -1240,7 +1261,7 @@ curl_easy_setopt_ccsid(CURL * curl, CURLoption tag, ...)
     if(!s || !pfsize || ccsid == NOCONV_CCSID || ccsid == ASCII_CCSID) {
       result = curl_easy_setopt(curl, CURLOPT_COPYPOSTFIELDS, s);
       break;
-      }
+    }
 
     if(pfsize == -1) {
       /* Data is null-terminated. */
@@ -1258,7 +1279,7 @@ curl_easy_setopt_ccsid(CURL * curl, CURLoption tag, ...)
       if(pfsize < 0 || pfsize > SIZE_MAX) {
         result = CURLE_OUT_OF_MEMORY;
         break;
-        }
+      }
 
       len = pfsize;
       pfsize = len * MAX_CONV_EXPANSION;
@@ -1271,7 +1292,7 @@ curl_easy_setopt_ccsid(CURL * curl, CURLoption tag, ...)
       if(!cp) {
         result = CURLE_OUT_OF_MEMORY;
         break;
-        }
+      }
 
       pfsize = convert(cp, pfsize, ASCII_CCSID, s, len, ccsid);
 
@@ -1279,11 +1300,11 @@ curl_easy_setopt_ccsid(CURL * curl, CURLoption tag, ...)
         free(cp);
         result = CURLE_OUT_OF_MEMORY;
         break;
-        }
+      }
 
       data->set.postfieldsize = pfsize;         /* Replace data size. */
       s = cp;
-      }
+    }
 
     result = curl_easy_setopt(curl, CURLOPT_POSTFIELDS, s);
     data->set.str[STRING_COPYPOSTFIELDS] = s;   /* Give to library. */
@@ -1291,9 +1312,12 @@ curl_easy_setopt_ccsid(CURL * curl, CURLoption tag, ...)
 
   case CURLOPT_ERRORBUFFER:                     /* This is an output buffer. */
   default:
-    result = Curl_vsetopt(data, tag, arg);
+  {
+    long val = va_arg(arg, long);
+    result = curl_easy_setopt(curl, tag, val);
     break;
-    }
+  }
+  }
 
   va_end(arg);
   return result;
@@ -1424,3 +1448,47 @@ curl_mime_data_ccsid(curl_mimepart *part,
   free(s);
   return result;
 }
+
+CURLUcode
+curl_url_get_ccsid(CURLU *handle, CURLUPart what, char **part,
+                   unsigned int flags, unsigned int ccsid)
+
+{
+  char *s = (char *)NULL;
+  CURLUcode result;
+
+  if(!part)
+    return CURLUE_BAD_PARTPOINTER;
+
+  *part = (char *)NULL;
+  result = curl_url_get(handle, what, &s, flags);
+  if(result == CURLUE_OK) {
+    if(s) {
+      *part = dynconvert(ccsid, s, -1, ASCII_CCSID);
+      if(!*part)
+        result = CURLUE_OUT_OF_MEMORY;
+    }
+  }
+  if(s)
+    free(s);
+  return result;
+}
+
+CURLUcode
+curl_url_set_ccsid(CURLU *handle, CURLUPart what, const char *part,
+                   unsigned int flags, unsigned int ccsid)
+
+{
+  char *s = (char *)NULL;
+  CURLUcode result;
+
+  if(part) {
+    s = dynconvert(ASCII_CCSID, part, -1, ccsid);
+    if(!s)
+      return CURLUE_OUT_OF_MEMORY;
+  }
+  result = curl_url_set(handle, what, s, flags);
+  if(s)
+    free(s);
+  return result;
+}
