@@ -71,26 +71,26 @@ enum { Avail = 0xffffffff, Eof = 0xfffffffe, Bat = 0xfffffffd, MetaBat = 0xfffff
 
 static inline unsigned short readU16(const unsigned char *ptr)
 {
-	return (unsigned short)(ptr[0]+(ptr[1]<<8));
+	return static_cast<unsigned short>(ptr[0]+(ptr[1]<<8));
 }
 
 static inline unsigned readU32(const unsigned char *ptr)
 {
-	return (unsigned)(ptr[0]+(ptr[1]<<8)+(ptr[2]<<16)+(ptr[3]<<24));
+	return unsigned(ptr[0]+(ptr[1]<<8)+(ptr[2]<<16)+(ptr[3]<<24));
 }
 
 static inline void writeU16(unsigned char *ptr, unsigned long data)
 {
-	ptr[0] = (unsigned char)(data & 0xff);
-	ptr[1] = (unsigned char)((data >> 8) & 0xff);
+	ptr[0] = static_cast<unsigned char>(data & 0xff);
+	ptr[1] = static_cast<unsigned char>((data >> 8) & 0xff);
 }
 
 static inline void writeU32(unsigned char *ptr, unsigned long data)
 {
-	ptr[0] = (unsigned char)(data & 0xff);
-	ptr[1] = (unsigned char)((data >> 8) & 0xff);
-	ptr[2] = (unsigned char)((data >> 16) & 0xff);
-	ptr[3] = (unsigned char)((data >> 24) & 0xff);
+	ptr[0] = static_cast<unsigned char>(data & 0xff);
+	ptr[1] = static_cast<unsigned char>((data >> 8) & 0xff);
+	ptr[2] = static_cast<unsigned char>((data >> 16) & 0xff);
+	ptr[3] = static_cast<unsigned char>((data >> 24) & 0xff);
 }
 
 static unsigned long getLength(librevenge::RVNGInputStream *const input)
@@ -166,7 +166,7 @@ public:
 	void clear();
 	unsigned long count() const
 	{
-		return (unsigned long) m_data.size();
+		return static_cast<unsigned long>(m_data.size());
 	}
 	void resize(unsigned long newsize)
 	{
@@ -194,7 +194,7 @@ public:
 	void setChain(std::vector<unsigned long> chain, unsigned end);
 	void save(unsigned char *buffer) const
 	{
-		auto cnt=(unsigned) count();
+		auto cnt=unsigned(count());
 		unsigned i = 0;
 		for (i = 0; i < cnt; i++)
 			writeU32(buffer + i*4, m_data[i]);
@@ -206,7 +206,7 @@ public:
 	// return space required to save the allocation table
 	unsigned saveSize() const
 	{
-		auto cnt=(unsigned)(((count()+127)/128)*128);
+		auto cnt=unsigned(((count()+127)/128)*128);
 		return cnt * 4;
 	}
 private:
@@ -780,20 +780,20 @@ void libwps_OLE::Header::load(const unsigned char *buffer, unsigned long size)
 {
 	if (size < 512)
 		return;
-	m_revision = (unsigned) readU16(buffer+0x18);
-	m_shift_bbat      = (unsigned int) readU16(buffer + 0x1e);
+	m_revision = unsigned(readU16(buffer+0x18));
+	m_shift_bbat      =static_cast<unsigned int>(readU16(buffer + 0x1e));
 	if (m_shift_bbat > 31)
 		m_shift_bbat = 31;
-	m_shift_sbat      = (unsigned int) readU16(buffer + 0x20);
+	m_shift_sbat      =static_cast<unsigned int>(readU16(buffer + 0x20));
 	if (m_shift_sbat > 31)
 		m_shift_sbat = 31;
-	m_num_bat      = (unsigned int) readU32(buffer + 0x2c);
-	m_start_dirent = (unsigned int) readU32(buffer + 0x30);
-	m_threshold    = (unsigned int) readU32(buffer + 0x38);
-	m_start_sbat   = (unsigned int) readU32(buffer + 0x3c);
-	m_num_sbat     = (unsigned int) readU32(buffer + 0x40);
-	m_start_mbat   = (unsigned int) readU32(buffer + 0x44);
-	m_num_mbat     = (unsigned int) readU32(buffer + 0x48);
+	m_num_bat      =static_cast<unsigned int>(readU32(buffer + 0x2c));
+	m_start_dirent =static_cast<unsigned int>(readU32(buffer + 0x30));
+	m_threshold    =static_cast<unsigned int>(readU32(buffer + 0x38));
+	m_start_sbat   =static_cast<unsigned int>(readU32(buffer + 0x3c));
+	m_num_sbat     =static_cast<unsigned int>(readU32(buffer + 0x40));
+	m_start_mbat   =static_cast<unsigned int>(readU32(buffer + 0x44));
+	m_num_mbat     =static_cast<unsigned int>(readU32(buffer + 0x48));
 
 	for (unsigned i = 0; i < 8; i++)
 		m_magic[i] = buffer[i];
@@ -871,7 +871,7 @@ void libwps_OLE::DirEntry::load(unsigned char *buffer, unsigned len)
 
 	// parse name of this entry, which stored as Unicode 16-bit
 	m_name=std::string("");
-	auto name_len = (unsigned) readU16(buffer + 0x40);
+	auto name_len = unsigned(readU16(buffer + 0x40));
 	if (name_len > 64) name_len = 64;
 	if (name_len==2 && m_type==5 && readU16(buffer)==0x5200)
 	{
@@ -887,16 +887,16 @@ void libwps_OLE::DirEntry::load(unsigned char *buffer, unsigned len)
 
 	int i = 0;
 	for (i = 0; i < 4; i++)
-		m_info.m_clsid[i]=(unsigned) readU32(buffer + 0x50 + 4*i);
+		m_info.m_clsid[i]=unsigned(readU32(buffer + 0x50 + 4*i));
 	for (i = 0; i < 4; i++)
-		m_info.m_time[i]=(unsigned) readU32(buffer + 0x64 + 4*i);
+		m_info.m_time[i]=unsigned(readU32(buffer + 0x64 + 4*i));
 
 	m_valid = true;
-	m_start = (unsigned int) readU32(buffer + 0x74);
-	m_size = (unsigned int) readU32(buffer + 0x78);
-	m_left = (unsigned int) readU32(buffer + 0x44);
-	m_right = (unsigned int) readU32(buffer + 0x48);
-	m_child = (unsigned int) readU32(buffer + 0x4C);
+	m_start =static_cast<unsigned int>(readU32(buffer + 0x74));
+	m_size =static_cast<unsigned int>(readU32(buffer + 0x78));
+	m_left =static_cast<unsigned int>(readU32(buffer + 0x44));
+	m_right =static_cast<unsigned int>(readU32(buffer + 0x48));
+	m_child =static_cast<unsigned int>(readU32(buffer + 0x4C));
 
 	// sanity checks
 	if ((m_type != 2) && (m_type != 1) && (m_type != 5)) m_valid = false;
@@ -908,19 +908,19 @@ void libwps_OLE::DirEntry::save(unsigned char *buffer) const
 	int i = 0;
 	for (i = 0; i < 128; i++) buffer[i]=0;
 
-	auto name_len = (unsigned) m_name.length();
+	auto name_len = unsigned(m_name.length());
 	if (name_len>31) name_len = 31;
 	if (name_len==2 && m_macRootEntry && m_type==5)
 		buffer[1]='R';
 	else
 	{
 		for (size_t j = 0; j < name_len; j++)
-			writeU16(buffer+2*j, (unsigned) m_name[j]);
+			writeU16(buffer+2*j, unsigned(m_name[j]));
 	}
 	writeU16(buffer+0x40, 2*name_len+2);
 
-	buffer[0x42]=(unsigned char) m_type;
-	buffer[0x43]=(unsigned char) m_colour;
+	buffer[0x42]=static_cast<unsigned char>(m_type);
+	buffer[0x43]=static_cast<unsigned char>(m_colour);
 	for (i = 0; i < 4; i++)
 		writeU32(buffer + 0x50+4*i, m_info.m_clsid[i]);
 	for (i = 0; i < 4; i++)
@@ -1246,7 +1246,7 @@ void libwps_OLE::IStorage::load()
 	{
 		std::vector<unsigned char> buffer(blocks.size()*m_bbat.m_blockSize);
 		unsigned long readData=loadBigBlocks(blocks, &buffer[0], buffer.size());
-		m_bbat.load(&buffer[0], (unsigned)readData);
+		m_bbat.load(&buffer[0], unsigned(readData));
 	}
 
 	// load small bat
@@ -1256,7 +1256,7 @@ void libwps_OLE::IStorage::load()
 	{
 		std::vector<unsigned char> buffer(blocks.size()*m_bbat.m_blockSize);
 		unsigned long readData=loadBigBlocks(blocks, &buffer[0], buffer.size());
-		m_sbat.load(&buffer[0], (unsigned)readData);
+		m_sbat.load(&buffer[0], unsigned(readData));
 	}
 
 	// load directory tree
@@ -1266,7 +1266,7 @@ void libwps_OLE::IStorage::load()
 	{
 		std::vector<unsigned char> buffer(blocks.size()*m_bbat.m_blockSize);
 		unsigned long readData=loadBigBlocks(blocks, &buffer[0], buffer.size());
-		m_dirtree.load(&buffer[0], (unsigned)readData);
+		m_dirtree.load(&buffer[0], unsigned(readData));
 		if (readData >= 0x74 + 4)
 		{
 			unsigned sb_start = readU32(&buffer[0x74]);
@@ -1417,11 +1417,11 @@ bool libwps_OLE::OStorage::updateToSave()
 		// FIXME: set m_header.m_start_sbat
 		buffer.resize(sbatSize);
 		m_sbat.save(&buffer[0]);
-		m_header.m_num_sbat = (unsigned)(sbatSize+511)/512;
+		m_header.m_num_sbat = unsigned(sbatSize+511)/512;
 		m_header.m_start_sbat = insertData(&buffer[0], sbatSize, true, unsigned(Eof));
 		if (m_sb_blocks.size())
 		{
-			rEntry->m_start =(unsigned) m_sb_blocks[0];
+			rEntry->m_start = unsigned(m_sb_blocks[0]);
 			m_bbat.setChain(m_sb_blocks, unsigned(Eof));
 		}
 		rEntry->m_size = 64*m_num_sbat;
@@ -1493,7 +1493,7 @@ bool libwps_OLE::OStorage::updateToSave()
 			writeU32(&buffer[wPos], Avail);
 			wPos+=4;
 		}
-		m_header.m_start_mbat = insertData((unsigned char const *)&buffer[0], 512*numMAlloc, true, unsigned(Eof));
+		m_header.m_start_mbat = insertData(reinterpret_cast<unsigned char const *>(buffer.data()), 512*numMAlloc, true, unsigned(Eof));
 		m_header.m_start_mbat = numBBlock+numBAlloc;
 	}
 	m_header.m_num_bat = (m_num_bbat+127)/128;
@@ -1530,7 +1530,7 @@ unsigned libwps_OLE::OStorage::insertData(unsigned char const *buffer, unsigned
 		m_bbat.setChain(chain, end);
 	else
 		m_sbat.setChain(chain, end);
-	return (unsigned) chain[0];
+	return unsigned(chain[0]);
 }
 
 // =========== IStream ==========
@@ -1582,7 +1582,7 @@ libwps_OLE::IStream::IStream(libwps_OLE::IStorage *s, std::string const &name) :
 unsigned long libwps_OLE::IStream::readData(unsigned long pos, unsigned char *data, unsigned long maxlen)
 {
 	// sanity checks
-	if (!data || (maxlen == 0) || (unsigned long)m_data.size() != m_size || !m_size)
+	if (!data || (maxlen == 0) || static_cast<unsigned long>(m_data.size()) != m_size || !m_size)
 		return 0;
 
 	if (pos >= m_size)
@@ -1743,7 +1743,7 @@ bool libwps_OLE::IStream::createOleFromDirectory(IStorage *io, std::string const
 			return false;
 		}
 		// ok, we can update size
-		m_size = (unsigned long) m_data.size();
+		m_size = static_cast<unsigned long>(m_data.size());
 		return true;
 	}
 	catch (...)
@@ -1823,7 +1823,7 @@ public:
 	/** returns the ith sub streams name*/
 	const char *subStreamName(unsigned id) final
 	{
-		if (id>=(unsigned) m_streamNameList.size())
+		if (id>=unsigned(m_streamNameList.size()))
 			return nullptr;
 		return m_streamNameList[size_t(id)].c_str();
 	}
@@ -1858,12 +1858,12 @@ private:
 librevenge::RVNGInputStream *WPSOLEStream::getSubStreamByName(const char *name)
 {
 	if (!isStructured())
-		return 0;
+		return nullptr;
 
 	IStorage storage(m_input.get());
 	IStream tmpStream(&storage, name);
 	if (storage.m_result != storage.Ok  || !tmpStream.size())
-		return (RVNGInputStream *)nullptr;
+		return nullptr;
 
 	std::vector<unsigned char> buf(tmpStream.size());
 	unsigned long tmpLength;
@@ -1873,9 +1873,9 @@ librevenge::RVNGInputStream *WPSOLEStream::getSubStreamByName(const char *name)
 	if (tmpLength != tmpStream.size())
 		/* something went wrong here and we do not trust the
 		   resulting buffer */
-		return (RVNGInputStream *)nullptr;
+		return nullptr;
 
-	return new WPSStringStream(&buf[0], (unsigned)tmpLength);
+	return new WPSStringStream(&buf[0], unsigned(tmpLength));
 }
 
 RVNGInputStreamPtr getOLEInputStream(RVNGInputStreamPtr input)
