@@ -14,7 +14,10 @@ CFWL_MessageSetFocus::CFWL_MessageSetFocus(CFWL_Widget* pSrcTarget,
                                            CFWL_Widget* pDstTarget)
     : CFWL_Message(CFWL_Message::Type::SetFocus, pSrcTarget, pDstTarget) {}
 
-CFWL_MessageSetFocus::~CFWL_MessageSetFocus() {}
+CFWL_MessageSetFocus::CFWL_MessageSetFocus(const CFWL_MessageSetFocus& that) =
+    default;
+
+CFWL_MessageSetFocus::~CFWL_MessageSetFocus() = default;
 
 std::unique_ptr<CFWL_Message> CFWL_MessageSetFocus::Clone() {
   return pdfium::MakeUnique<CFWL_MessageSetFocus>(*this);
