@@ -10,7 +10,6 @@
 #include <memory>
 #include <utility>
 
-#include "third_party/base/ptr_util.h"
 #include "third_party/base/stl_util.h"
 #include "xfa/fwl/cfwl_messagemouse.h"
 #include "xfa/fwl/cfwl_messagemousewheel.h"
@@ -33,30 +32,7 @@ CFWL_ScrollBar::CFWL_ScrollBar(
     std::unique_ptr<CFWL_WidgetProperties> properties,
     CFWL_Widget* pOuter)
     : CFWL_Widget(app, std::move(properties), pOuter),
-      m_pTimerInfo(nullptr),
-      m_fRangeMin(0),
-      m_fRangeMax(-1),
-      m_fPageSize(0),
-      m_fStepSize(0),
-      m_fPos(0),
-      m_fTrackPos(0),
-      m_iMinButtonState(CFWL_PartState_Normal),
-      m_iMaxButtonState(CFWL_PartState_Normal),
-      m_iThumbButtonState(CFWL_PartState_Normal),
-      m_iMinTrackState(CFWL_PartState_Normal),
-      m_iMaxTrackState(CFWL_PartState_Normal),
-      m_fLastTrackPos(0),
-      m_iMouseWheel(0),
-      m_bMouseDown(false),
-      m_fButtonLen(0),
-      m_bMinSize(false),
       m_Timer(this) {
-  m_rtClient.Reset();
-  m_rtThumb.Reset();
-  m_rtMinBtn.Reset();
-  m_rtMaxBtn.Reset();
-  m_rtMinTrack.Reset();
-  m_rtMaxTrack.Reset();
 }
 
 CFWL_ScrollBar::~CFWL_ScrollBar() {}
@@ -74,21 +50,21 @@ void CFWL_ScrollBar::Update() {
   Layout();
 }
 
-void CFWL_ScrollBar::DrawWidget(CFX_Graphics* pGraphics,
-                                const CFX_Matrix* pMatrix) {
+void CFWL_ScrollBar::DrawWidget(CXFA_Graphics* pGraphics,
+                                const CFX_Matrix& matrix) {
   if (!pGraphics)
     return;
   if (!m_pProperties->m_pThemeProvider)
     return;
 
-  IFWL_ThemeProvider* pTheme = m_pProperties->m_pThemeProvider;
+  IFWL_ThemeProvider* pTheme = m_pProperties->m_pThemeProvider.Get();
   if (HasBorder())
-    DrawBorder(pGraphics, CFWL_Part::Border, pTheme, pMatrix);
-  DrawTrack(pGraphics, pTheme, true, pMatrix);
-  DrawTrack(pGraphics, pTheme, false, pMatrix);
-  DrawArrowBtn(pGraphics, pTheme, true, pMatrix);
-  DrawArrowBtn(pGraphics, pTheme, false, pMatrix);
-  DrawThumb(pGraphics, pTheme, pMatrix);
+    DrawBorder(pGraphics, CFWL_Part::Border, pTheme, matrix);
+  DrawTrack(pGraphics, pTheme, true, &matrix);
+  DrawTrack(pGraphics, pTheme, false, &matrix);
+  DrawArrowBtn(pGraphics, pTheme, true, &matrix);
+  DrawArrowBtn(pGraphics, pTheme, false, &matrix);
+  DrawThumb(pGraphics, pTheme, &matrix);
 }
 
 void CFWL_ScrollBar::SetTrackPos(float fTrackPos) {
@@ -104,7 +80,7 @@ bool CFWL_ScrollBar::DoScroll(CFWL_EventScroll::Code dwCode, float fPos) {
   return OnScroll(dwCode, fPos);
 }
 
-void CFWL_ScrollBar::DrawTrack(CFX_Graphics* pGraphics,
+void CFWL_ScrollBar::DrawTrack(CXFA_Graphics* pGraphics,
                                IFWL_ThemeProvider* pTheme,
                                bool bLower,
                                const CFX_Matrix* pMatrix) {
@@ -117,10 +93,10 @@ void CFWL_ScrollBar::DrawTrack(CFX_Graphics* pGraphics,
   param.m_pGraphics = pGraphics;
   param.m_matrix.Concat(*pMatrix);
   param.m_rtPart = bLower ? m_rtMinTrack : m_rtMaxTrack;
-  pTheme->DrawBackground(&param);
+  pTheme->DrawBackground(param);
 }
 
-void CFWL_ScrollBar::DrawArrowBtn(CFX_Graphics* pGraphics,
+void CFWL_ScrollBar::DrawArrowBtn(CXFA_Graphics* pGraphics,
                                   IFWL_ThemeProvider* pTheme,
                                   bool bMinBtn,
                                   const CFX_Matrix* pMatrix) {
@@ -134,10 +110,10 @@ void CFWL_ScrollBar::DrawArrowBtn(CFX_Graphics* pGraphics,
   param.m_matrix.Concat(*pMatrix);
   param.m_rtPart = bMinBtn ? m_rtMinBtn : m_rtMaxBtn;
   if (param.m_rtPart.height > 0 && param.m_rtPart.width > 0)
-    pTheme->DrawBackground(&param);
+    pTheme->DrawBackground(param);
 }
 
-void CFWL_ScrollBar::DrawThumb(CFX_Graphics* pGraphics,
+void CFWL_ScrollBar::DrawThumb(CXFA_Graphics* pGraphics,
                                IFWL_ThemeProvider* pTheme,
                                const CFX_Matrix* pMatrix) {
   CFWL_ThemeBackground param;
@@ -149,7 +125,7 @@ void CFWL_ScrollBar::DrawThumb(CFX_Graphics* pGraphics,
   param.m_pGraphics = pGraphics;
   param.m_matrix.Concat(*pMatrix);
   param.m_rtPart = m_rtThumb;
-  pTheme->DrawBackground(&param);
+  pTheme->DrawBackground(param);
 }
 
 void CFWL_ScrollBar::Layout() {
@@ -355,9 +331,9 @@ void CFWL_ScrollBar::OnProcessMessage(CFWL_Message* pMessage) {
   }
 }
 
-void CFWL_ScrollBar::OnDrawWidget(CFX_Graphics* pGraphics,
-                                  const CFX_Matrix* pMatrix) {
-  DrawWidget(pGraphics, pMatrix);
+void CFWL_ScrollBar::OnDrawWidget(CXFA_Graphics* pGraphics,
+                                  const CFX_Matrix& matrix) {
+  DrawWidget(pGraphics, matrix);
 }
 
 void CFWL_ScrollBar::OnLButtonDown(const CFX_PointF& point) {
@@ -385,7 +361,11 @@ void CFWL_ScrollBar::OnLButtonDown(const CFX_PointF& point) {
 }
 
 void CFWL_ScrollBar::OnLButtonUp(const CFX_PointF& point) {
-  m_pTimerInfo->StopTimer();
+  if (m_pTimerInfo) {
+    m_pTimerInfo->StopTimer();
+    m_pTimerInfo = nullptr;
+  }
+
   m_bMouseDown = false;
   DoMouseUp(0, m_rtMinBtn, m_iMinButtonState, point);
   DoMouseUp(1, m_rtThumb, m_iThumbButtonState, point);
@@ -485,11 +465,12 @@ void CFWL_ScrollBar::DoMouseHover(int32_t iItem,
 CFWL_ScrollBar::Timer::Timer(CFWL_ScrollBar* pToolTip) : CFWL_Timer(pToolTip) {}
 
 void CFWL_ScrollBar::Timer::Run(CFWL_TimerInfo* pTimerInfo) {
-  CFWL_ScrollBar* pButton = static_cast<CFWL_ScrollBar*>(m_pWidget);
+  CFWL_ScrollBar* pScrollBar = static_cast<CFWL_ScrollBar*>(m_pWidget.Get());
+  if (pScrollBar->m_pTimerInfo) {
+    pScrollBar->m_pTimerInfo->StopTimer();
+    pScrollBar->m_pTimerInfo = nullptr;
+  }
 
-  if (pButton->m_pTimerInfo)
-    pButton->m_pTimerInfo->StopTimer();
-
-  if (!pButton->SendEvent())
-    pButton->m_pTimerInfo = StartTimer(0, true);
+  if (!pScrollBar->SendEvent())
+    pScrollBar->m_pTimerInfo = StartTimer(0, true);
 }
