@@ -23,74 +23,70 @@
 namespace {
 
 const BarCodeInfo g_BarCodeData[] = {
-    {0x7fb4a18, L"ean13", BarcodeType::ean13, BC_EAN13},
-    {0x8d13a3d, L"code11", BarcodeType::code11, BC_UNKNOWN},
-    {0x8d149a8, L"code49", BarcodeType::code49, BC_UNKNOWN},
-    {0x8d16347, L"code93", BarcodeType::code93, BC_UNKNOWN},
-    {0x91a92e2, L"upsMaxicode", BarcodeType::upsMaxicode, BC_UNKNOWN},
-    {0xa7d48dc, L"fim", BarcodeType::fim, BC_UNKNOWN},
-    {0xb359fe9, L"msi", BarcodeType::msi, BC_UNKNOWN},
-    {0x121f738c, L"code2Of5Matrix", BarcodeType::code2Of5Matrix, BC_UNKNOWN},
-    {0x15358616, L"ucc128", BarcodeType::ucc128, BC_UNKNOWN},
-    {0x1f4bfa05, L"rfid", BarcodeType::rfid, BC_UNKNOWN},
-    {0x1fda71bc, L"rss14Stacked", BarcodeType::rss14Stacked, BC_UNKNOWN},
-    {0x22065087, L"ean8add2", BarcodeType::ean8add2, BC_UNKNOWN},
-    {0x2206508a, L"ean8add5", BarcodeType::ean8add5, BC_UNKNOWN},
-    {0x2278366c, L"codabar", BarcodeType::codabar, BC_CODABAR},
-    {0x2a039a8d, L"telepen", BarcodeType::telepen, BC_UNKNOWN},
-    {0x323ed337, L"upcApwcd", BarcodeType::upcApwcd, BC_UNKNOWN},
-    {0x347a1846, L"postUSIMB", BarcodeType::postUSIMB, BC_UNKNOWN},
-    {0x391bb836, L"code128", BarcodeType::code128, BC_CODE128},
-    {0x398eddaf, L"dataMatrix", BarcodeType::dataMatrix, BC_DATAMATRIX},
-    {0x3cff60a8, L"upcEadd2", BarcodeType::upcEadd2, BC_UNKNOWN},
-    {0x3cff60ab, L"upcEadd5", BarcodeType::upcEadd5, BC_UNKNOWN},
-    {0x402cb188, L"code2Of5Standard", BarcodeType::code2Of5Standard,
+    {0x7fb4a18, "ean13", BarcodeType::ean13, BC_EAN13},
+    {0x8d13a3d, "code11", BarcodeType::code11, BC_UNKNOWN},
+    {0x8d149a8, "code49", BarcodeType::code49, BC_UNKNOWN},
+    {0x8d16347, "code93", BarcodeType::code93, BC_UNKNOWN},
+    {0x91a92e2, "upsMaxicode", BarcodeType::upsMaxicode, BC_UNKNOWN},
+    {0xa7d48dc, "fim", BarcodeType::fim, BC_UNKNOWN},
+    {0xb359fe9, "msi", BarcodeType::msi, BC_UNKNOWN},
+    {0x121f738c, "code2Of5Matrix", BarcodeType::code2Of5Matrix, BC_UNKNOWN},
+    {0x15358616, "ucc128", BarcodeType::ucc128, BC_UNKNOWN},
+    {0x1f4bfa05, "rfid", BarcodeType::rfid, BC_UNKNOWN},
+    {0x1fda71bc, "rss14Stacked", BarcodeType::rss14Stacked, BC_UNKNOWN},
+    {0x22065087, "ean8add2", BarcodeType::ean8add2, BC_UNKNOWN},
+    {0x2206508a, "ean8add5", BarcodeType::ean8add5, BC_UNKNOWN},
+    {0x2278366c, "codabar", BarcodeType::codabar, BC_CODABAR},
+    {0x2a039a8d, "telepen", BarcodeType::telepen, BC_UNKNOWN},
+    {0x323ed337, "upcApwcd", BarcodeType::upcApwcd, BC_UNKNOWN},
+    {0x347a1846, "postUSIMB", BarcodeType::postUSIMB, BC_UNKNOWN},
+    {0x391bb836, "code128", BarcodeType::code128, BC_CODE128},
+    {0x398eddaf, "dataMatrix", BarcodeType::dataMatrix, BC_DATAMATRIX},
+    {0x3cff60a8, "upcEadd2", BarcodeType::upcEadd2, BC_UNKNOWN},
+    {0x3cff60ab, "upcEadd5", BarcodeType::upcEadd5, BC_UNKNOWN},
+    {0x402cb188, "code2Of5Standard", BarcodeType::code2Of5Standard, BC_UNKNOWN},
+    {0x411764f7, "aztec", BarcodeType::aztec, BC_UNKNOWN},
+    {0x44d4e84c, "ean8", BarcodeType::ean8, BC_EAN8},
+    {0x48468902, "ucc128sscc", BarcodeType::ucc128sscc, BC_UNKNOWN},
+    {0x4880aea4, "upcAadd2", BarcodeType::upcAadd2, BC_UNKNOWN},
+    {0x4880aea7, "upcAadd5", BarcodeType::upcAadd5, BC_UNKNOWN},
+    {0x54f18256, "code2Of5Industrial", BarcodeType::code2Of5Industrial,
      BC_UNKNOWN},
-    {0x411764f7, L"aztec", BarcodeType::aztec, BC_UNKNOWN},
-    {0x44d4e84c, L"ean8", BarcodeType::ean8, BC_EAN8},
-    {0x48468902, L"ucc128sscc", BarcodeType::ucc128sscc, BC_UNKNOWN},
-    {0x4880aea4, L"upcAadd2", BarcodeType::upcAadd2, BC_UNKNOWN},
-    {0x4880aea7, L"upcAadd5", BarcodeType::upcAadd5, BC_UNKNOWN},
-    {0x54f18256, L"code2Of5Industrial", BarcodeType::code2Of5Industrial,
+    {0x58e15f25, "rss14Limited", BarcodeType::rss14Limited, BC_UNKNOWN},
+    {0x5c08d1b9, "postAUSReplyPaid", BarcodeType::postAUSReplyPaid, BC_UNKNOWN},
+    {0x5fa700bd, "rss14", BarcodeType::rss14, BC_UNKNOWN},
+    {0x631a7e35, "logmars", BarcodeType::logmars, BC_UNKNOWN},
+    {0x6a236236, "pdf417", BarcodeType::pdf417, BC_PDF417},
+    {0x6d098ece, "upcean2", BarcodeType::upcean2, BC_UNKNOWN},
+    {0x6d098ed1, "upcean5", BarcodeType::upcean5, BC_UNKNOWN},
+    {0x76b04eed, "code3Of9extended", BarcodeType::code3Of9extended, BC_UNKNOWN},
+    {0x7c7db84a, "maxicode", BarcodeType::maxicode, BC_UNKNOWN},
+    {0x8266f7f7, "ucc128random", BarcodeType::ucc128random, BC_UNKNOWN},
+    {0x83eca147, "postUSDPBC", BarcodeType::postUSDPBC, BC_UNKNOWN},
+    {0x8dd71de0, "postAUSStandard", BarcodeType::postAUSStandard, BC_UNKNOWN},
+    {0x98adad85, "plessey", BarcodeType::plessey, BC_UNKNOWN},
+    {0x9f84cce6, "ean13pwcd", BarcodeType::ean13pwcd, BC_UNKNOWN},
+    {0xb514fbe9, "upcA", BarcodeType::upcA, BC_UPCA},
+    {0xb514fbed, "upcE", BarcodeType::upcE, BC_UNKNOWN},
+    {0xb5c6a853, "ean13add2", BarcodeType::ean13add2, BC_UNKNOWN},
+    {0xb5c6a856, "ean13add5", BarcodeType::ean13add5, BC_UNKNOWN},
+    {0xb81fc512, "postUKRM4SCC", BarcodeType::postUKRM4SCC, BC_UNKNOWN},
+    {0xbad34b22, "code128SSCC", BarcodeType::code128SSCC, BC_UNKNOWN},
+    {0xbfbe0cf6, "postUS5Zip", BarcodeType::postUS5Zip, BC_UNKNOWN},
+    {0xc56618e8, "pdf417macro", BarcodeType::pdf417macro, BC_UNKNOWN},
+    {0xca730f8a, "code2Of5Interleaved", BarcodeType::code2Of5Interleaved,
      BC_UNKNOWN},
-    {0x58e15f25, L"rss14Limited", BarcodeType::rss14Limited, BC_UNKNOWN},
-    {0x5c08d1b9, L"postAUSReplyPaid", BarcodeType::postAUSReplyPaid,
-     BC_UNKNOWN},
-    {0x5fa700bd, L"rss14", BarcodeType::rss14, BC_UNKNOWN},
-    {0x631a7e35, L"logmars", BarcodeType::logmars, BC_UNKNOWN},
-    {0x6a236236, L"pdf417", BarcodeType::pdf417, BC_PDF417},
-    {0x6d098ece, L"upcean2", BarcodeType::upcean2, BC_UNKNOWN},
-    {0x6d098ed1, L"upcean5", BarcodeType::upcean5, BC_UNKNOWN},
-    {0x76b04eed, L"code3Of9extended", BarcodeType::code3Of9extended,
-     BC_UNKNOWN},
-    {0x7c7db84a, L"maxicode", BarcodeType::maxicode, BC_UNKNOWN},
-    {0x8266f7f7, L"ucc128random", BarcodeType::ucc128random, BC_UNKNOWN},
-    {0x83eca147, L"postUSDPBC", BarcodeType::postUSDPBC, BC_UNKNOWN},
-    {0x8dd71de0, L"postAUSStandard", BarcodeType::postAUSStandard, BC_UNKNOWN},
-    {0x98adad85, L"plessey", BarcodeType::plessey, BC_UNKNOWN},
-    {0x9f84cce6, L"ean13pwcd", BarcodeType::ean13pwcd, BC_UNKNOWN},
-    {0xb514fbe9, L"upcA", BarcodeType::upcA, BC_UPCA},
-    {0xb514fbed, L"upcE", BarcodeType::upcE, BC_UNKNOWN},
-    {0xb5c6a853, L"ean13add2", BarcodeType::ean13add2, BC_UNKNOWN},
-    {0xb5c6a856, L"ean13add5", BarcodeType::ean13add5, BC_UNKNOWN},
-    {0xb81fc512, L"postUKRM4SCC", BarcodeType::postUKRM4SCC, BC_UNKNOWN},
-    {0xbad34b22, L"code128SSCC", BarcodeType::code128SSCC, BC_UNKNOWN},
-    {0xbfbe0cf6, L"postUS5Zip", BarcodeType::postUS5Zip, BC_UNKNOWN},
-    {0xc56618e8, L"pdf417macro", BarcodeType::pdf417macro, BC_UNKNOWN},
-    {0xca730f8a, L"code2Of5Interleaved", BarcodeType::code2Of5Interleaved,
-     BC_UNKNOWN},
-    {0xd0097ac6, L"rss14Expanded", BarcodeType::rss14Expanded, BC_UNKNOWN},
-    {0xd25a0240, L"postAUSCust2", BarcodeType::postAUSCust2, BC_UNKNOWN},
-    {0xd25a0241, L"postAUSCust3", BarcodeType::postAUSCust3, BC_UNKNOWN},
-    {0xd53ed3e7, L"rss14Truncated", BarcodeType::rss14Truncated, BC_UNKNOWN},
-    {0xe72bcd57, L"code128A", BarcodeType::code128A, BC_UNKNOWN},
-    {0xe72bcd58, L"code128B", BarcodeType::code128B, BC_CODE128_B},
-    {0xe72bcd59, L"code128C", BarcodeType::code128C, BC_CODE128_C},
-    {0xee83c50f, L"rss14StackedOmni", BarcodeType::rss14StackedOmni,
-     BC_UNKNOWN},
-    {0xf2a18f7e, L"QRCode", BarcodeType::QRCode, BC_QR_CODE},
-    {0xfaeaf37f, L"postUSStandard", BarcodeType::postUSStandard, BC_UNKNOWN},
-    {0xfb48155c, L"code3Of9", BarcodeType::code3Of9, BC_CODE39},
+    {0xd0097ac6, "rss14Expanded", BarcodeType::rss14Expanded, BC_UNKNOWN},
+    {0xd25a0240, "postAUSCust2", BarcodeType::postAUSCust2, BC_UNKNOWN},
+    {0xd25a0241, "postAUSCust3", BarcodeType::postAUSCust3, BC_UNKNOWN},
+    {0xd53ed3e7, "rss14Truncated", BarcodeType::rss14Truncated, BC_UNKNOWN},
+    {0xe72bcd57, "code128A", BarcodeType::code128A, BC_UNKNOWN},
+    {0xe72bcd58, "code128B", BarcodeType::code128B, BC_CODE128_B},
+    {0xe72bcd59, "code128C", BarcodeType::code128C, BC_CODE128_C},
+    {0xee83c50f, "rss14StackedOmni", BarcodeType::rss14StackedOmni, BC_UNKNOWN},
+    {0xf2a18f7e, "QRCode", BarcodeType::QRCode, BC_QR_CODE},
+    {0xfaeaf37f, "postUSStandard", BarcodeType::postUSStandard, BC_UNKNOWN},
+    {0xfb48155c, "code3Of9", BarcodeType::code3Of9, BC_CODE39},
 };
 
 }  // namespace.
@@ -106,7 +102,7 @@ const BarCodeInfo* CXFA_FFBarcode::GetBarcodeTypeByName(
       FX_HashCode_GetW(wsName.AsStringView(), true),
       [](const BarCodeInfo& arg, uint32_t hash) { return arg.uHash < hash; });
 
-  if (it != std::end(g_BarCodeData) && wsName.AsStringView() == it->pName)
+  if (it != std::end(g_BarCodeData) && wsName.EqualsASCII(it->pName))
     return it;
 
   return nullptr;
@@ -121,7 +117,7 @@ bool CXFA_FFBarcode::LoadWidget() {
   auto pNew = pdfium::MakeUnique<CFWL_Barcode>(GetFWLApp());
   CFWL_Barcode* pFWLBarcode = pNew.get();
   m_pNormalWidget = std::move(pNew);
-  m_pNormalWidget->SetLayoutItem(this);
+  m_pNormalWidget->SetFFWidget(this);
 
   CFWL_NoteDriver* pNoteDriver =
       m_pNormalWidget->GetOwnerApp()->GetNoteDriver();
@@ -139,14 +135,14 @@ bool CXFA_FFBarcode::LoadWidget() {
 
 void CXFA_FFBarcode::RenderWidget(CXFA_Graphics* pGS,
                                   const CFX_Matrix& matrix,
-                                  uint32_t dwStatus) {
-  if (!IsMatchVisibleStatus(dwStatus))
+                                  HighlightOption highlight) {
+  if (!HasVisibleStatus())
     return;
 
   CFX_Matrix mtRotate = GetRotateMatrix();
   mtRotate.Concat(matrix);
 
-  CXFA_FFWidget::RenderWidget(pGS, mtRotate, dwStatus);
+  CXFA_FFWidget::RenderWidget(pGS, mtRotate, highlight);
   DrawBorder(pGS, m_pNode->GetUIBorder(), m_rtUI, mtRotate);
   RenderCaption(pGS, &mtRotate);
   CFX_RectF rtWidget = m_pNormalWidget->GetWidgetRect();
